#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: FtuFttActionBase.py,v 1.1 2004/09/06 14:38:37 cguirao Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  06/02/2001  Created
#

"""
Base class for classes handling actions.

This class should not be instantiated directly in Python FTTs. The
appropriate class implemented to handle the specific action should
be used.
"""

import exceptions
import PccUtString
from   FtuFttDefs import *


class FtuFttActionBase:
    """
    Base class for FTT Actions (general).
    """


    def __init__(self,
                 action       = "",
                 valueSrcList = None,
                 fct          = ""):
        """
        Constructor method to initialize the object.

        action          Action related to the object.
        
        valueSrcList    Python List of FtuFttValueSrc objects.

        fct             Conversion Function related to the action.
        """
        self.setAction(action)
        if valueSrcList:
            self.setValueSrcList(valueSrcList)
        else:
            self.setValueSrcList([])
        self.setFct(fct)

        # Build up the Action-2-Key-Map.
        self.__key2ActionMap = {"UNDEFINED":          ftuACTION_UNDEF,
                                "HDR.DUMP":           ftuACTION_HDR_DUMP,

                                "FILE.PATH.BASE":     ftuACTION_FILE_PATH,
                                "FILE.RENAME.PREFIX": ftuACTION_FILE_RENAME,
                                "FILE.SPLIT.PREFIX":  ftuACTION_FILE_SPLIT,
                              
                                "ADD.KEY":            ftuACTION_ADD,
                                "CHANGE.KEY":         ftuACTION_CHANGE,
                                "RENAME.KEY":         ftuACTION_RENAME, 
                                "MOVE.KEY":           ftuACTION_MOVE, 
                                "COPY.SRCKEY":        ftuACTION_COPY,   
                                "REMOVE.KEYLIST":     ftuACTION_REMOVE}
        keys = self.__key2ActionMap.keys()
        self.__action2KeyMap = {}
        for key in keys:
            self.__action2KeyMap[self.__key2ActionMap[key]] = key


    def _setFctFromFtt(self,
                       action,
                       keyCount,
                       fttPafObj):
        """
        Check if a Conversion Function is contained in the Translation
        Block.
        
        action         The action concerned (see documentation for class
                       for the names of the constants).
        
        keyCount       The present key count, at which the function keyword
                       is supposed to be.
        
        fttPafObj      Reference to the FTT Object.

        Returns:       Key counter in the FTT Object.
        """
        if (keyCount < fttPafObj.getNoOfKeys()):
            if (fttPafObj.getKeyObjNo(keyCount).getKey() == action + ".FCT"):
                self.setFct(fttPafObj.getKeyObjNo(keyCount).getValue())
                keyCount = keyCount + 1
        return keyCount

 
    def setAction(self,
                  action):
        """
        Set the Action of the object.

        action           Action for object. See various constants in
                         documentation for class.

        Returns:         Reference to object itself.
        """
        self.__action = PccUtString.trimString(action, " \"")
        return self


    def getAction(self):
        """
        Return the Translation Action set for the object.

        Returns:        Translation Action.
        """
        return self.__action


    def getValueSrcList(self):
        """
        Return reference to internal list of FtuFttValueSrc objects.

        Returns:      Reference to Value Source List.
        """
        return self.__valueSrcList


    def addValueSrc(self,
                    valueSrcObj):
        """
        Add a Value Source Object to the Translation Action.

        valueSrcObj   Reference to instance of FtuFttValueSrc class.
        
        Returns:      Reference to object itself.
        """
        self.__valueSrcList.append(valueSrcObj)
        return self


    def clearValueSrcList(self):
        """
        Clear the Value Sources for the Translation Action.

        Returns:      Reference to object itself. 
        """
        self.__valueSrcList = []
        return self


    def setValueSrcList(self,
                        valueSrcList):
        """
        """
        if valueSrcList:
            self.__valueSrcList = valueSrcList
        else:
            self.__valueSrcList = []
        return self
        

    def setFct(self,
               fct):
        """
        Set Conversion Function for the Translation Action.

        fct           Name of Conversion Function.

        Returns:      Reference to object itself. 
        """
        self.__fct = PccUtString.trimString(fct, " \"")
        return self
    

    def getFct(self):
        """
        Return Conversion Function for Translation Action.

        Returns:       Conversion Function.
        """
        return self.__fct


    def action2Key(self,
                   action):
        """
        Convert a Translation Action name into its corresponding keyword
        name.

        action      Translation Action.

        Returns:    Key corresponding to Translation Action.
        """
        return self.__action2KeyMap[action]


    def key2Action(self,
                   key):
        """
        Convert a keyword into its corresponding Translation Action.

        key         Keyword.

        Returns:    Translation Action corresponding to Key.
        """
        return self.__key2ActionMap[key]



#
# ___oOo___
