#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: FtuFttFileAction.py,v 1.1 2004/09/06 14:38:37 cguirao Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  16/02/2001  Created
#

"""
Base class for File Actions.

This class should not be used directly in the P-FTTs. One the derived class
FtuFttFilePath, FtuFttFileRename or FtuFttFileSplit should be used.

Some of the methods of this class, may be useful to emply from the
classes mentioned above.
"""


import types
import PccUtUtils, PccPaf
from   FtuFttActionBase import *
import FtuFttValueSrc, FtuFttUt


class FtuFttFileAction(FtuFttActionBase):
    """
    Base class for Filename (Manipulation) Actions.
    """


    def __init__(self,
                 prefix       = "",
                 valueSrcList = None):
        """
        Constructor method initializing the object class variables.

        prefix:        Prefix/base for the File Action. This corresponds to
                       to FILE.PATH.BASE or FILE.RENAME.PREFIX.
        
        valueSrcList   List of FtuFttValueSrc objects.
        """
        FtuFttActionBase.__init__(self)
        self.setPrefix(prefix)
        self.setValueSrcList(valueSrcList)


    def isDefined(self):
        """
        Indicates if it seems as though the object has been properly
        defined.

        Returns:   Returns 1 if object seems defined, otherwise 0.
        """
        if (self.getPrefix() != ""):
            return 1
        else:
            return 0


    def setPrefix(self,
                  prefix):
        """
        Set the prefix for the File Action (corresponds to FILE.PATH.BASE
        or FILE.RENAME.PREFIX).

        prefix:   String containing the prefix.
        
        Returns:  Reference to object itself.
        """
        tmpPrefix = PccUtString.trimString(prefix, " \"")
        PccUtUtils.checkType("prefix", tmpPrefix,
                             "FtuFttFileAction.setPrefix()", types.StringType)
        self.__prefix = tmpPrefix
        return self


    def getPrefix(self):
        """
        Return prefix for File Action.

        Returns:   Prefix.
        """
        return self.__prefix

    
    def _setFileActionFromFtt(self,
                              action,
                              keyCount,
                              fttPafObj):
        """
        Extract an Add or Change Translation Action from an FTT.

        action:         Action to extract: FtuFttAction.ftuACTION_ADD or
                        FtuFttAction.ftuACTION_CHANGE.
        
        keyCount:       The present key count.
        
        fttPafObj:      Reference to the FTT Object.

        Returns:        Key counter in the FTT object.
        """
        PccUtUtils.checkType("keyCount", keyCount,
                             "FtuFttFileAction._setFileActionFromFtt()",
                             types.IntType)
        PccUtUtils.checkType("fttPafObj", fttPafObj,
                             "FtuFttFileAction._setFileActionFromFtt()",
                             PccPaf.PccPaf)
        
        self.setAction(action)

        initKey = self.action2Key(action)
            
        # Handle <init key>.
        FtuFttUt.checkExpKey(fttPafObj.getKeyObjNo(keyCount).getKey(), initKey)
        self.setPrefix(fttPafObj.getKeyObjNo(keyCount).getValue())
        keyCount = keyCount + 1

        # Handle value sources (optional).
        if (keyCount < fttPafObj.getNoOfKeys()):
            keyObj = fttPafObj.getKeyObjNo(keyCount)        
        while ((keyCount < fttPafObj.getNoOfKeys()) and \
               ((keyObj.getKey() == action + ".CONCAT.KEY") or
                (keyObj.getKey() == action + ".CONCAT.CONST") or
                (keyObj.getKey() == action + ".CONCAT.FCT"))):
            valueSrc = FtuFttValueSrc.FtuFttValueSrc()
            keyCount = valueSrc.setFromPaf(keyCount, fttPafObj)
            self.addValueSrc(valueSrc)
            if (keyCount < fttPafObj.getNoOfKeys()):
                keyObj = fttPafObj.getKeyObjNo(keyCount)

        # Handle function (optional).
        keyCount = self._setFctFromFtt(action, keyCount, fttPafObj)
              
        return keyCount
  

    def _setFileAction(self,
                       prefix,
                       valueSrcList = None):
        """
        Set the File Action for the 
        
        prefix:        Prefix/base for the File Action.
        
        valueSrcList   Reference to list containing FtuFttValueSrc objects.

        Returns:       Reference to object itself.
        """
        self.setPrefix(prefix)
        if valueSrcList:
            self.setValueSrcList(valueSrcList)
        return self


    def dumpBuf(self):
        """
        Dump the contents of the object into a string buffer in PAF
        format.

        Returns:    Reference to string buffer containing the generated
                    File Action in PAF format.
        """
        retBuf = '%-27s "%s"' % (self.action2Key(self.getAction()),
                                 self.getPrefix())
        for valueSrc in self.getValueSrcList():
            retBuf = retBuf + "\n" + valueSrc.dumpBuf()
        return retBuf


    def dump(self):
        """
        Dump the contents of the object on stdout in PAF format.

        Returns:    Reference to object itself.
        """
        print self.dumpBuf()
        return self

#
# ___oOo___
