#include "Mem.h"

/*
 * E.S.O. - VLT project / ESO Archive
 *
 * "@(#) $Id: Mem.C,v 1.5 2005/12/09 14:25:23 szampier Exp $" 
 *
 * The purpose of this class is to replace only the very basic functionalities
 * of the "original" Mem class, which was used to manage memory areas
 * and to mmap files.
 * This class is used by OLAS and it's not meant to be used by other (new)
 * applications
 * See previous CVS revisions for original code.
 * 
 * who                when          what
 * ----------------   -----------   -------------------------------------
 * Stefano Zampieri   15 Sep 2005   Created
 */

/*
 * constructor with filename. No memory is allocated.
 */
Mem::Mem(const char * filename, int opt, int verbose) : 
  owner_(0),
  data_(0),
  size_(0),
  options_(opt),
  status_(0)
{
  // get the name of the real file (if a link)
  char realname[1024];
  const char* fname = fileRealname(filename, realname, sizeof(realname));
  if (fname == realname)
    linkname_ = filename;  // remember the name of the link
  filename_ = fname;
  status_ = size() <= 0;
}


/*
 * constructor with size of memory to allocate.
 */
Mem::Mem(int aSize) :
  owner_(0),
  status_(0),
  size_(aSize)
{
  if (size() <= 0) {
    fmt_error("Invalid size requested to Mem: %d bytes", size());
    throw "Mem exception";
  }
#ifdef DEBUG
  log_message("allocating %d bytes", size());
#endif
  data_ = new char[size()];
}

/*
 * dummy constructor
 */
Mem::Mem() :
  owner_(0),
  data_(0)
{
}

/*
 * destructor : allocated memory is freed
 */
Mem::~Mem()
{
  //if (owner_ == 1)
  //unlink(filename_.data()); // what if filename is a symbolic link ?  
  if (data_) {
    delete data_;
#ifdef DEBUG
    log_message("releasing %d bytes", size());
#endif
  }
}

/*
 * if the constructor with filename was used, this
 * method can be used to load in memory the file contents
 *
 */
int Mem::loadFileInMemory()
{
  if (!filename_.length()) {
    fmt_error("null filename");
    return -1;
  }
  if (size() <= 0) {
    fmt_error("Invalid size requested to Mem: %d bytes", size());
    throw "Mem exception";
  }
  if (data_) {
    fmt_error("loadFileInMemory: memory already allocated. Please report this error to the developer.");
  }
  int fd = open(filename_.data(), O_RDONLY);
  if (fd <= 0) {
    sys_error("Can't open : ", filename_.data());
    return -1;
  }
#ifdef DEBUG
  log_message("allocating %d bytes", size());
#endif
  data_ = new char[size()];
  if (read(fd, data_, size()) != size()) {
    fmt_error("can't read from %s", filename_.data());
    return -1;
  }
  close(fd);
}

/*
 * return pointer to data.
 */
void * Mem::myptr() const
{
  return data_;
}

// set owner flag
void Mem::owner(int o)
{
  owner_ = o;
}

// get file size or data size, depending on which constructor was used
int Mem::size() const
{
  if (size_ > 0)
    return size_;
  
  if (!filename_.length()) {
    fmt_error("null filename");
    return -1;
  }
  
  struct stat buf;
  if (stat(filename_.data(), &buf) != 0) {
    sys_error("Can't stat: ", filename_.data());
    return -1;
  }
  return buf.st_size;
}

// get real file name or link name if flag is set
const char* Mem::filename(int flag) const
{
  if (flag && linkname_.length())
    return linkname_.data();
  if (filename_.length())
    return filename_.data();
  return NULL;
}
