/*
 * E.S.O. - VLT project/ESO Archive
 * $Id: TimeStamp.C,v 1.2 2004/11/24 15:22:42 nferguso Exp $
 *
 * TimeStamp.C - method definitions for class TimeStamp, a class 
 *               representing a timestamp value in either MJD
 *               (modified julian day) or string format:
 *               "YYYY-MM-DDThh:mm:ss[.mss]".
 *
 * who             when         what
 * --------------  -----------  ------------------------------------------- 
 * Allan Brighton  11 Jul 1997  Created. 
 * Neil Ferguson   08 Apr 2004  Methods added to replace RogueWave RWDate & 
 *                              RWTime functionality used by the DFS. 
 */
static const char* const rcsId="@(#) $Id: TimeStamp.C,v 1.2 2004/11/24 15:22:42 nferguso Exp $";

#include <math.h>
#include <stdio.h>
#include <string>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include "error.h"
#include "TimeStamp.h"

#include <locale>
#include <sstream>

// #include <DfsCTokenizer.h> 

/*
 * constructor: initialize with the current time
 */
TimeStamp::TimeStamp()
    : status_(0)
{
    struct timeval tv;
    struct timezone tz;
    gettimeofday(&tv, &tz);

    time_t t = tv.tv_sec;
    struct tm* tm = gmtime(&t);
    double secs = tm->tm_sec + (tv.tv_usec/1000000.);
    status_ = tmToMjd(*tm, secs, mjd_);
}


/*
 * constructor: initialize from string of form [...]YYYY-MM-DDThh:mm:ss[.mss]
 * or from string of form YYYY-MM-DD
 *
 * Note: leading strings starting with non-numeric chars and ending with
 * "." are ignored, for example, "ONTT.1997-07-21T20:03:01.234" will ignore
 * the "ONTT." part.
 *
 */
TimeStamp::TimeStamp(const char* s)
    : status_(0)
{
    if (!s) {
	status_ = error("null timestamp string");
	return;
    }
    
    // ignore leading non-numeric string
    const char* p = s;
    if (! isdigit(*p))
    {
      p = strchr(p, '.');
      if (!p) 
      {
	status_ = error("bad TimeStamp string format: ", s);
	return;
      }
       
      p++;
    }
    
    struct tm tm;
    bzero(&tm,sizeof(tm));
    
    char dateFormatString[20];
    if(strlen(p) > 10)
        // the date format is YYYY-MM-DDThh:mm:ss
    	strcpy(dateFormatString, "%Y-%m-%d %T");
    else
        // the date format is YYYY-MM-DD
    	strcpy(dateFormatString, "%Y-%m-%d");
    
    char buf[20];
    strncpy(buf, p, 19);
    buf[19]='\0';
    // strptime does not like an alphanumeric character ('T')
    // in between 2 conversions specifications, so we just blank it
    char *q = strchr(buf,'T');
    if (q)
	*q = ' ';
    
    if (strptime(buf, dateFormatString, &tm) == NULL) {
	status_ = error("bad TimeStamp string: ", s);
	return;
    }
    
    // convert tm to MJD time
    double secs = 0;
    sscanf(p, "%*d-%*02d-%*02dT%*02d:%*02d:%lf", &secs);
    status_ = tmToMjd(tm, secs, mjd_);
}


/*
 * get string time value as YYYY-MM-DDThh:mm:ss[.mss]
 */
const char* TimeStamp::getTimeStamp() const
{
    double secs;  // seconds with fractional part
    struct tm tm;
    mjdToTm(mjd_, tm, secs);
    char tmpBuf[10];
    char s[10];
    // the following piece of code is needed to
    // format the seconds and milliseconds as ss.mss
    sprintf(tmpBuf,"%.3f", secs);
    if (strlen(tmpBuf) == 6)
	strcpy(s,tmpBuf);
    else {
	s[0] = '0';
	strcpy(&s[1],tmpBuf);
    }
    
    sprintf((char*)buf_, "%d-%02d-%02dT%02d:%02d:%s",
	    tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday, 
	    tm.tm_hour, tm.tm_min, s);
    return buf_;
}


/*
 * get string date value as YYYY-MM-DD
 */
const char* TimeStamp::getDate() const
{
    double secs;  // seconds with fractional part
    struct tm tm;
    mjdToTm(mjd_, tm, secs);
    sprintf((char*)buf_, "%d-%02d-%02d", 
	    tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday);
    return buf_;
}


/*
 * get string date value as YYYY-MM-DD for the
 * night (noon to noon)
 */
const char* TimeStamp::getNight() const
{
    double secs;  // seconds with fractional part
    struct tm tm;
    mjdToTm(mjd_-0.5, tm, secs);
    sprintf((char*)buf_, "%d-%02d-%02d", 
	    tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday);
    return buf_;
}

/*
 * get string time value as HH:MM:SS.sss
 */
const char* TimeStamp::getTime() const
{
    double secs;  // seconds with fractional part
    struct tm tm;
    mjdToTm(mjd_, tm, secs);
    char tmpBuf[10];
    char s[10];
    // the following piece of code is needed to
    // format the seconds and milliseconds as ss.mss
    sprintf(tmpBuf,"%.3f", secs);
    if (strlen(tmpBuf) == 6)
	strcpy(s,tmpBuf);
    else {
	s[0] = '0';
	strcpy(&s[1],tmpBuf);
    }

    sprintf((char*)buf_, "%02d:%02d:%s", 
	    tm.tm_hour, tm.tm_min, s);
    return buf_;
}


/*
 * convert the mjd (modified julian day) value to a struct tm time
 * value. Since the tm struct doesn't contain the fractional seconds, a
 * more exact value for seconds is returned in secs.
 * Returns 0 if all is OK.
 */
int TimeStamp::mjdToTm(double mjd, struct tm& tm, double& secs) 
{
    secs = (mjd - MJD_1970) * 86400;
    time_t t = (time_t)secs;
    secs = fmod(secs, 60.);
    
    // fix for cases like secs = 59.9996, that would lead to a value of
    // 60 seconds (wrong!) because the resolution supported is 1 ms
    if (fabs(60. - secs) < 0.0005) {
	t += 1;
	secs = 0.0;
    }
    tm = *gmtime(&t);
    return 0;
}


/* 
 * convert a struct tm to MJD time and put the results in
 * mjd. The secs argument may contain more exact seconds than
 * the tm struct. Returns 0 if ok.
 */
int TimeStamp::tmToMjd(const struct tm& tm, double secs, double& mjd) 
{
    long int jm, j;
    double jd;

    int year = tm.tm_year + 1900;  // allan: tm is years since 1900
    if (year <= -4712) {
	j = 1 + (year + 4712)/400;   	/* Number of 400-yr cycles */
	jm = -j * 146097;
	j  = year + 400*j;
    }
    else
	jm = 0, j = year;

    j  -= (11 - tm.tm_mon)/10;
    jm += (1461* ( j + 4712))/4 + (306 * ((tm.tm_mon+10)%12) + 5)/10
	- (3* ((j + 4900)/100))/4 + tm.tm_mday + 96;
    jd = jm;

    jm  = ((long)(12+tm.tm_hour))*3600 + tm.tm_min*60 + tm.tm_sec;
    jd += (jm/86400.);

    mjd = jd - 2400000.5;

    // allan: replace integer seconds with double secs
    mjd = (mjd - (tm.tm_sec/86400.)) + (secs/86400.);
    return 0;
}


/*
 * return time in struct tm and a more exact representation of
 * the seconds within the minute in secs
 */
int TimeStamp::getTmTime(struct tm& tm, double& secs) const
{
    return mjdToTm(mjd_, tm, secs);
}


// RogueWave RWDate/RWTime Replacements [Neil Ferguson] 

// Construct for the given date. Time is zero hours on that date. 
TimeStamp::TimeStamp(unsigned int day, unsigned int month, unsigned int year) :
    status_(0)
{
    // Unused tm members must be zeroed. 
    struct tm tm; 
    bzero(&tm, sizeof(tm));

    tm.tm_year = year - 1900;   // tm year is years since 1900. 
    tm.tm_mon = month - 1;      // tm month is months since January. 
    tm.tm_mday = day;

    // Only the date is specified; time components are zero. 
    double secs = 0;
    status_ = tmToMjd(tm, secs, mjd_);                  // Convert to MJD. 
}

// Construct for the given date, but with the specified time. 
TimeStamp::TimeStamp(const TimeStamp& date, 
                     unsigned int hour, 
                     unsigned int minute, 
                     unsigned int second) :
    status_(0)
{
    double dayFraction = (hour * 3600 + minute * 60 + second) / 86400.0; 
    mjd_ = floor(date.mjd_) + dayFraction; 
}

// Decrements this TimeStamp by one day, but returns the original value. 
TimeStamp 
TimeStamp::operator--(int dummy)
{
    (void)dummy;        // Prevent compiler warning on unused parameter. 

    TimeStamp original(*this);
    --mjd_;
    return(original);
}

// Returns the current time as a string, formatted according to the given 
// strftime() formatting string. 
DfsCString 
TimeStamp::asString(const char* format) const 
{
    using namespace std; 

    ostringstream outputData; 

    // Get the locale for output formatting. 
    const locale& loc = outputData.getloc(); 

    // Get the time output facet of the locale. 
    const time_put<char>& formatter = use_facet<time_put<char> >(loc); 

    // Convert the stored MJD to a C tm struct. 
    struct tm tm;
    double secs = 0;                    // Not used. 
    mjdToTm(mjd_, tm, secs);

    // Write the C tm struct to the output stream, as a string with 
    // the given strftime() format. 
    formatter.put(ostreambuf_iterator<char>(outputData), 
                  outputData, 
                  ' ', 
                  &tm, 
                  format, 
                  format + strlen(format)); 
    
    // Convert the stream data to a DfsCString and return it. 
    return(DfsCString(outputData.str().c_str())); 
}

// Compares this TimeStamp with the compareTime. 
int 
TimeStamp::compareTo(const TimeStamp* compareTime) const
{
    int result = (mjd_ < compareTime->mjd_) ? -1 : 0;
    result     = (mjd_ > compareTime->mjd_) ?  1 : result;
    return(result); 
}

// Returns the day of the year (1-366) for the stored date. 
unsigned int 
TimeStamp::day() const
{
    // Convert the stored MJD to a C tm struct. 
    struct tm tm;
    double secs = 0;                                    // Not used. 
    mjdToTm(mjd_, tm, secs);

    // Return the day of the year. 
    return(tm.tm_yday + 1);             // tm day is days since Jan 1. 
}

// Returns the hour of the time stored in this TimeStamp. 
unsigned int 
TimeStamp::hour() const
{
    // Convert the stored MJD to a C tm struct. 
    struct tm tm;
    double secs = 0;                                    // Not used. 
    mjdToTm(mjd_, tm, secs);

    // Return the hour value. 
    return(tm.tm_hour);
}
    
// Returns the (unmodified) Julian day number of the stored date. 
double 
TimeStamp::julian() const
{
    // Add the offset between modified and unmodified Julian day numbers. 
    return(mjd_ + 2400000.5);
}

// Returns the year of the stored date. 
unsigned int 
TimeStamp::year() const
{
    // Convert the stored MJD to a C tm struct. 
    struct tm tm;
    double secs = 0;                                    // Not used. 
    mjdToTm(mjd_, tm, secs);

    // Return the year. 
    return(tm.tm_year + 1900);          // tm year is years since 1900. 
}

// Returns true if the given TimeStamps are not equal. 
DfsBoolean 
operator!=(const TimeStamp& leftOperand, const TimeStamp& rightOperand) 
{
    return((leftOperand.mjd() != rightOperand.mjd()) 
           || (leftOperand.status() != rightOperand.status())); 
}

// STL ostream output operator for TimeStamp objects. 
std::ostream& 
operator<<(std::ostream& stream, const TimeStamp& source)
{
    // Discard any stream.width() setting.
    stream.width(0);

    // Formatting is delegated to the asString() method. 
    stream << source.asString("%m/%d/%y %H:%M:%S");

    return(stream);
}
