/******************************************************************************
* E.S.O. - VLT project
*
* slxAddFITSBlocks.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  02/09/94  Created 1. version 
* epozna    10/09/02  VLTSW20020319 added slxAddFITSBlocksWithEnd
* epo/blo   10/09/02  updated slxAddFITSBlocksWithEnd to remove only the 
*                     last occurance of the END keyword only.
* epozna    10/09/02  VLTSW20020319 updated for case when empty file 
*                     loaded
*/

/************************************************************************
*   NAME
*   slxAddFITSBlocks - adds a number of FITS blocks to an image data header
* 
*   SYNOPSIS
*   #include "slx.h"
*
*
*   ccsCOMPL_STAT slxAddFITSBlocks(const int         fd, 
*                                  const vltUINT8    noOfBlocks,
*                                  const vltLOGICAL  writeEND,
*                                  ccsERROR          *error)
*   ccsCOMPL_STAT slxAddFITSBlocksWithEnd(const int         fd, 
*                                   const vltUINT8    noOfBlocks,
*                                   ccsERROR          *error);
* 
*   DESCRIPTION
*   The functions are used to reserve space for the FITS header of an image data
*   file. This is done by adding a number of FITS blocks, only consisting of 
*   blanks, to an already existing image data file.
*
*   If 0 is specified as "noOfBlocks", the function will check that the number
*   of characters in the file is equal to: 'n x slxFITS_HD_BLOCK_LEN',
*   where n is 1, 2, 3, ..., and slxFITS_HD_BLOCK_LEN, is the number of 
*   characters in one FITS header block. If this is not fulfilled, there is
*   padded with an adequate number of blanks. Otherwise if "noOfBlocks" is
*   larger that 0, the function will check for the completion of existing 
*   blocks and add a number of blocks corresponding to "noOfBlocks".
*
*   The file pointer will point to the end of the file when the function
*   terminates successfully.
*
*   The function slxAddFITSBlocksWithEnd always writes the END keyword at the 
*   end of the block. If the header already ends with an END keyword this is 
*   removed before the new block is added. 
*   For this function the file must be opened with read and write (O_RDWR) access.
* 
*   fd           IN   File descriptor for a file which is already opened.
*
*   noOfBlocks   IN   The number of blocks that should be allocated in the 
*                     image header.
*
*   writeEND     IN   If this flag is TRUE the function will write the
*                     END keyword at the end of the header.
*
*   error        OUT  Structure in which a possible error occurring is 
*                     specified.
*
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successful.
*   FAILURE   In case the operation failed. The following errors can be
*             generated by the function:
*
*             slxERR_READ_FILE   An error occurred while reading from the file.
*             
*             slxERR_WRITE_FILE  An error occurred while writing in the file.
*
*
*   CAUIONS
*   The file managed by "fd" must be opened for reading and writing.
*
*   The function assumes that there are no newline characters in the file.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxAddFITSBlocks.c,v 1.145 2005/07/27 09:03:15 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <unistd.h>
#include <string.h>
#ifdef LINUX
    #include <sys/types.h>
#endif

#include <errno.h>

/* Local Headers
 */
#include "slx.h"

ccsCOMPL_STAT slxAddFITSBlocksWithEnd(const int         fd, 
				      const vltUINT8    noOfBlocks,
				      ccsERROR          *error)
{
   
   slxKEYW_LINE          blankLine;
   slxKEYW_LINE          endLine;
   slxKEYW_LINE          kwLine;
   int i,n;
   off_t endOffset;

   /* initialise the variables
    */
    i=0;n=0; errno=0;

    memset((char*)endLine,   '\0', sizeof(endLine));
    memset((char*)blankLine, '\0', sizeof(blankLine));
    memset((char*)kwLine,    '\0', sizeof(kwLine));

    memset((char *)blankLine, ' ', (size_t) slxMAX_FITS_LI_LEN);    

    memset((char *)endLine,   ' ',   (size_t) (slxMAX_FITS_LI_LEN));
    memcpy((char *)endLine, "END", 3 );

    
     /* Go to the end of the file  
      */
     if ((endOffset = lseek(fd, (off_t)0, SEEK_END)) == -1)
         {
         errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, NULL,
                   modUNIX, ".");
         return FAILURE;
         }
     /*printf("C endOffset=%d\n",endOffset);*/
     if(endOffset<slxMAX_FITS_LI_LEN)
	 goto slxAddFITSblocks;

     /*  Now go back one line 
      */
     endOffset -= slxMAX_FITS_LI_LEN;
     if (lseek(fd, endOffset, SEEK_SET) == -1)
         {
         errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, NULL,
                   modUNIX, ".");
         return FAILURE;
         }

     /* Read this last line and search for an END keyword
      */
     if ((n=read(fd,(char*)kwLine, (size_t)slxMAX_FITS_LI_LEN))>0)
         {
         if (strncmp(kwLine,endLine,slxMAX_FITS_LI_LEN)==0)
	     {
             /* Go back to the last line
              */
             if (lseek(fd, endOffset, SEEK_SET) == -1)
                 {
                 errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, NULL,
                           modUNIX, ".");
                 return FAILURE;
                 }
             /* Overwrite line with blanks
              */
             if (write(fd, (char *)blankLine, (size_t)slxMAX_FITS_LI_LEN) == -1)
                 {
                 errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                           NULL, modUNIX, ".");
                 return FAILURE;
                 }
             }
         }

     if(errno!=0)
       { 
       errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, 
                 NULL, modUNIX, ".");
       return FAILURE;
       }


    /* Now that the last occurance of the END keywordline 
     * has been removed call the original function to add the
     * fits blocks with the END keywordline at the end.
     */
  slxAddFITSblocks:
    return (slxAddFITSBlocks(fd,noOfBlocks,TRUE,error));

}

/* Function definition
 */
ccsCOMPL_STAT slxAddFITSBlocks(const int         fd, 
                               const vltUINT8    noOfBlocks,
                               const vltLOGICAL  writeEND,
                               ccsERROR          *error)
{
    char          blankBuf[slxFITS_HD_BLOCK_LEN];
    int           count = 0;
    int           completeBlocks;
    off_t         endOffset;
    size_t        bytesMissing;
    off_t         hdSize;


    memset((char *)blankBuf, ' ', (size_t)slxFITS_HD_BLOCK_LEN);

    /* Be sure the file pointer points to the end of the file
     */
    if ((hdSize = lseek(fd, (off_t)0, SEEK_END)) == -1)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, NULL,
                  modUNIX, ".");
        return FAILURE;
        }

    /* Check if the number of blocks is a multiple of slxFITS_HD_BLOCK_LEN
     */
    completeBlocks = (int)(hdSize / slxFITS_HD_BLOCK_LEN);
    bytesMissing = (size_t)(hdSize - (completeBlocks * slxFITS_HD_BLOCK_LEN));

    /* If there are not enough bytes we'll have to append some more
     */
    if (bytesMissing > 0)
        {
        if (write(fd, (char *)blankBuf, 
                  (size_t)(slxFITS_HD_BLOCK_LEN - bytesMissing)) == -1)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                      NULL, modUNIX, ".");
            return FAILURE;
            }
        }

    /* Check if there should be appended some extra blocks (of blanks)
     */
    for (count = 0; count < noOfBlocks; count++)
        {
        if (write(fd, (char *)blankBuf, (size_t)slxFITS_HD_BLOCK_LEN) == -1)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                      NULL, modUNIX, ".");
            return FAILURE;
            }
        }

    /* If it is requested to write the END keyword - just do it!
     */
    if (writeEND == TRUE)
        {
        if ((endOffset = lseek(fd, (off_t)0, SEEK_END)) == -1)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, NULL,
                      modUNIX, ".");
            return FAILURE;
            }
        endOffset -= slxMAX_FITS_LI_LEN;
        if (lseek(fd, endOffset, SEEK_SET) == -1)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, NULL,
                      modUNIX, ".");
            return FAILURE;
            }
        if (write(fd, "END", (size_t)3) == -1)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, 
                      NULL, modUNIX, ".");
            return FAILURE;
            }
        }    /* end-if (writeEND == TRUE) */

    /* Make the file pointer point to the end of the file
     */
    if (lseek(fd, (off_t)0, SEEK_END) == -1)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, NULL,
                  modUNIX, ".");
        return FAILURE;
        }

    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/
