/******************************************************************************
* E.S.O. - VLT project
*
* slxCreStoreShFITSDicBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  04/09/95  created
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   02/02/04  Problems with APR2004, static variable initialized
*                     is initialized to ccsFALSE, VLTSW20040034
*/

/************************************************************************
*   NAME
*   slxCreStoreShFITSDicBuf - create short-FITS keyword, store in Dyn. Buf.
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxCreStoreShFITSDicBuf(slxDYN_BUF           *setupBuf,
*                                         const slxCATEGORY    category,
*			                  const slxSUBSYSTEM   subSystem,
*                                         const slxPARAMETER   parameter,
*                                         const void           *value,
*                                         const slxDICTIONARY  dictionary,
*                                         ccsERROR             *error)
* 
* 
*   DESCRIPTION
*   The function generates a Short-FITS keyword based on the knowledge about
*   the keyword extracted from a Data Interface Dictionary. It is thus 
*   mandatory to specify a Dictionary.
*
*   The created FITS line is automatically stored in the SLX Dynamic Buffer.
*
*
*   setupBuf    IN  SLX Dynamic Buffer in which the setup keyword line should
*                   be stored.
*
*   category    IN  The category. Can be an empty string ("").
*
*   subSystem   IN  The sunsystem for the keyword. Can be an empty string ("").
*
*   parameter   IN  The parameter name. Must be set.
* 
*   value       IN  Pointer to the value. The function deducts from the
*                   Dictionary what type the parameter is, and generates then
*                   the FITS line. A string must be NULL terminated. A logical
*                   must be of type "vltLOGICAL", an integer of type "vltINT32"
*                   and a floating point of "vltDOUBLE".
*
*   dictionary  IN  Name of the Data Interface Dictionary to use for generating
*                   the keywords. The Dictionary is only loaded once, and is
*                   afterwards stored in static memory. If a new Dictionary
*                   name is specified, the function loads the new Dictionary
*                   automatically.
*
*   error       OUT CCS error structure indicating a possible error 
*                   occurring.
* 
*
*   FILES
*   Data Interface Dictionary  Dictionary containing the defintion of the 
*                              keywords for the context.
*
*   ENVIRONMENT
*   INS_ROOT
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successfull.
*   FAILURE   In case the operation failed.
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxCreStoreShFITSDicBuf.c,v 1.145 2005/07/27 09:05:51 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxCreStoreShFITSDicBuf(slxDYN_BUF           *setupBuf,
				      const slxCATEGORY    category,
				      const slxSUBSYSTEM   subSystem,
				      const slxPARAMETER   parameter,
				      const void           *value,
				      const slxDICTIONARY  dictionary,
				      ccsERROR             *error)
{
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL       initialized=ccsFALSE;
    static miscHASH_TABLE   dicHashTable;
    static slxDICTIONARY    curDic;
    char                    *chrPtr;
    char                    tempBuf[16];
    slxDIC_HANDLE           *dicInfo;
    slxKEYW_LINE            keyword;
    slxKEYW_LINE            fitsLine; 
    vltLOGICAL              exponential;
    vltUINT8                precision;
    vltINT32                lineNo;


    if (!initialized)
	{
	miscHashInit(dicHashTable);
	*curDic = '\0';
	initialized = TRUE;
	}
    if (strcmp(curDic, dictionary) != 0)
	{
	miscHashDeleteTable(dicHashTable);
	if (slxLoadDicHash(dicHashTable, dictionary, error) == FAILURE)
	    return FAILURE;
	slxCOPY_MAX_N_BYTES(curDic, dictionary, sizeof(slxDICTIONARY), error);
	}

    *keyword = '\0';
    if (*category != '\0')
	{
	strcpy(keyword, category);
	strcat(keyword, ".");
	}
    if (*subSystem != '\0')
	{
	strcat(keyword, subSystem);
	strcat(keyword, ".");
	}
    strcat(keyword, parameter);

    if (slxLookUpKeywordHashX(dicHashTable, keyword, &dicInfo,
			      error) == FAILURE)
	return FAILURE;

    /* Create the FITS line
     */
    switch (dicInfo->dicParamType)
	{
	case slxSTRING:
	    if (slxCreateShortFITS(category, subSystem, parameter, slxSTRING,
				   (char *)value, FALSE, 0, 0, FALSE,
				   0.0, dicInfo->comField, fitsLine, 
				   error) == FAILURE)
		return FAILURE;
	    break;
	case slxLOGICAL:
	    if (slxCreateShortFITS(category, subSystem, parameter, slxLOGICAL,
				   "", *(vltLOGICAL *)value, 0, 0, FALSE,
				   0.0, dicInfo->comField, fitsLine, 
				   error) == FAILURE)
		return FAILURE;
	    break;
	case slxINTEGER:
	    if (slxCreateShortFITS(category, subSystem, parameter, slxINTEGER,
				   "", FALSE, *(vltINT32 *)value, 0, 
				   FALSE, 0.0, dicInfo->comField, fitsLine, 
				   error) == FAILURE)
		return FAILURE;
	    break;
	case slxDOUBLE: 
	    if ((chrPtr = strstr(dicInfo->valFormat, ".")) == NULL)
		precision = 3;
	    else
		{
		chrPtr++;
		*tempBuf = '\0';
		while (isdigit((int)*chrPtr))
		    {
		    strcat(tempBuf, chrPtr);
		    chrPtr++;
		    }
		precision = atoi(tempBuf);
		}
	    if ((*chrPtr == 'e') || (*chrPtr == 'E'))
		exponential = TRUE;
	    else
		exponential = FALSE;		
	    if (slxCreateShortFITS(category, subSystem, parameter, slxDOUBLE,
				   "", FALSE, 0, precision, exponential,
				   *(vltDOUBLE *)value, dicInfo->comField, 
				   fitsLine, error) == FAILURE)
		return FAILURE;		
	    break;
	default:
	    break;
	}

    /* Store the keyword line in the buffer
     */
    *(fitsLine + strlen(fitsLine)) = '\n';
    *(fitsLine + strlen(fitsLine) + 1) = '\0';
    if (slxStoreLineBuf(fitsLine, setupBuf, &lineNo, error) == FAILURE)
	return FAILURE;

    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/

