/******************************************************************************
* E.S.O. - VLT project
*
* slxCreateFits2.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  20/03/96  created
* - - - - - -MAR2002 - -- - - -
* epozna    27/05/02  fixed bug in va_list/va_end :slxCreateFits2()
*/

/************************************************************************
*   NAME
*   slxCreateFits2 - create a (Hierarchical) FITS keyword line
*
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxCreateFits2(const miscHASH_TABLE  dictionary,
*                                const slxFITS_VALUE   value,
*                                const vltLOGICAL      trimLine,
*                                slxKEYW_LINE          fitsLine,
*                                ccsERROR              *error, ..., NULL)
*
*   DESCRIPTION
*   The function generates a FITS keyword line from the input
*   parameters. This is done in accordance with the rules specified in
*   the "Data Interface Requirements" document (ARC-SPE-ESO-0000-0001/1.4),
*   and in "NOST Implementation of the Flexible Image Transport System 
*   - FITS, November 6, 1991".
*   
*   The FITS keyword generated can be either a standard FITS keyword, 
*   i.e. only a parameter, a value and a comment:
*
*   "<par>    = <value>                         / <comment>"
*
*   or it can be a hierarchical FITS keyword line, i.e. with the format: 
*
*   "HIERARCH ESO <cat> [<sub>] <par> = <value> / <comment>"
*  
*   The first two keywords ("HIERARCH ESO") are generated automatically.
*   
*   All strings passed to the function must be terminated with '\0'.
*
*   The function is depending on the availability of a Dictionary loaded 
*   into a MISC Hash Table.
*
*   The comment should be given only as the actual text -  do not append any
*   "/" or try to format it in any way.
*
*   A comment line, i.e. "COMMENT <comment>" shall be created, the variable
*   parameter list should contain only the keyword "COMMENT".
*
*   The generated FITS line is stored in the buffer pointed to by "fitsLine".
*   This buffer must have the a length of at least 81 characters (use
*   slxKEYW_LINE as type).
*
*   If the generated FITS line is longer than the maximum FITS line length 
*   (80 characters) the characters beyond the 80th position, are ignored if
*   the flag "trimLine" is TRUE.
*
*   The function supports automatic writing of converted units in the
*   Comment Field of a FITS Keyword Line. This requires that the definition
*   of the keyword in the Dictionary is correct. For the moment the following
*   is supported:
*   
*      =================================================
*      | Tag \ Unit | s | mjd | deg | rad | arcmin | K |
*      |===============================================|
*      | %TIME      | x                              - |
*      |- - - - - - - - - - - - - - - - - - - - - - - -|
*      | %DAYTIM    | x    x                         - |
*      |- - - - - - - - - - - - - - - - - - - - - - - -|
*      | %DEGREE    |            x     x      x      - |
*      |- - - - - - - - - - - - - - - - - - - - - - - -|
*      | %HOURANG   |            x     x      x      - |
*      |- - - - - - - - - - - - - - - - - - - - - - - -|
*      | %CELSIUS   | -    -     -     -      -      x |
*      =================================================
*
*   The tags trigger a translation into:
*
*       %TIME     =   hh:mm:ss.sss
*      
*       %DAYTIM   =   YYYY-MM-DDTHH:MM:SS.SSS
*        
*       %DEGREE   =   dd:mm:ss.s
*        
*       %HOURANG  =   hh:mm:ss.s
*      
*       %CELSIUS  =   <temp C>C
*
*   In the Dictionary the tag must be put in the comment and the unit
*   specified properly, e.g.:
*
*   Parameter Name:    CATi SUBi ARCMINi
*   Class:             header|setup
*   Context:           Detector
*   Type:              double
*   Value Format:      %f
*   Unit:              arcmin
*   Comment Field:     aaa "%HOURANG" bbb
*   Description:       
*   This is a test keyword.
*
*   When creating a FITS Line with these settings, the result will be 
*   something like:
*
*   HIERARCH ESO CAT1 SUB2 ARCMIN1  = 2.345000 / aaa "00:00:00.9" bbb
*
*
*   dictionary  IN     MISC Hash Table containing the Dictionary for the
*                      context. Information such as the type of keyword,
*                      value format, and comment is obtained from the 
*                      Dictionary.          
*
*   value       IN     Union parameter of type slxFITS_VALUE containing the
*                      value to be written in the FITS line. The members to
*                      set for each of the four types of data are:
* 
*                      value.s  - for a  string   (value.s = "string";)
*                      value.l  - for a  logical  (value.l = TRUE|FALSE;)
*                      value.i  - for an integer  (value.i = 1234;)
*                      value.d  - for a  double   (value.d = 1234.1234;)
*
*   trimLine    IN     If "trimLine" is FALSE, the FITS line generated will
*                      not be trimmed to 80 character but can have a maximum
*                      length of slxKEYW_LINE_LEN bytes.
*
*   fitsLine    OUT    Pointer to array where the new FITS line will be 
*                      written. The line must be at least 80 characters long;
*                      use possibly slxKEYW_LINE as type for the variable.
*
*   error       OUT    Pointer to CCS Error Structure.
*
*   ...         IN     The keyword that should be generated. It can either
*                      be given in as one Short-FITS keyword, e.g.:
*
*                      ..., error, "DET.WIN.STARTX", NULL);
*
*                      or it can be given as the individual sub-key 
*                      consituting the keyword to create, e.g.: 
*
*                      ..., error, "DET", "WIN", "STARTX", NULL);
*
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES 
*
*   CAUTIONS 
*
*   EXAMPLES
*   
*   #include "slx.h"
*
*   ccsERROR        error;
*   miscHASH_TABLE  dicHashTable,
*   slxKEYW_LINE    newFitsLine;
*   slxFITS_VALUE   value;
*
*
*   if (slxLoadDicHash(dicHashTable, "ISAAC", &error) == FAILURE)
*      ... error processing ...
*
*   value.d = 12.45;
*
*   * Create the FITS line
*   if (slxCreateFits2(dicHashTable, value, TRUE, newFitsLine, "DET", 
*                      "WIN1", "DIT", NULL) == FAILURE)
*      ... error processing ...
*   
*   
*   * The FITS is generated save it, print it ...
*   printf("New FITS line:\n\n%s\n", newFitsLine);
*
*   ...
*
*
*   Output ->
*
*   New FITS line:
*
*   HIERARCH ESO DET WIN1 DIT   = 1.245E+1         / Detector integration time
*
*   Note, it is also possible to enter the keyword in one piece in the
*   Short-FITS format, i.e.:
*
*   * Create the FITS line
*   if (slxCreateFits2(dicHashTable, value, TRUE, newFitsLine, 
*                      "DET.WIN1.DIT", NULL) == FAILURE)
*      ... error processing ...
*   
*
*   SEE ALSO
*   slxCreateWriteFits2(3), slxCreateShortFits2(3)
*
*   BUGS   
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxCreateFits2.c,v 1.145 2005/07/27 09:07:18 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>

/* Local Headers
 */
#include "slx.h"

#define _slxBUF_SAFETY_SIZE         512
#define _slxSTD_FITS_PREF_COM_POS   41
#define _slxSTD_FITS_LOG_VAL_POS    30
#define _slxHIER_FITS_PREF_COM_POS  45


/* Macros etc for performing the conversion
 */
#define _slxRAD_2_DEG  (360 / (2 * 3.1415927))

ccsCOMPL_STAT _slxSecs2DayTim(const slxFITS_VALUE   value,
			      const slxDIC_HANDLE   *dicEl,
			      const vltINT32        precision,
			      slxKEYW_LINE          dayTim,
			      ccsERROR              *error);

ccsCOMPL_STAT _slxSecs2Time(const slxFITS_VALUE   value,
			    const slxDIC_HANDLE   *dicEl,
			    const vltINT32        precision,
			    slxKEYW_LINE          utcTime,
			    ccsERROR              *error);

ccsCOMPL_STAT _slxMjd2DayTim(const slxFITS_VALUE   value,
			     const slxDIC_HANDLE   *dicEl,
			     const vltINT32        precision,
			     slxKEYW_LINE          dayTim,
			     ccsERROR              *error);

typedef enum {
    _slxDEG,
    _slxRAD,
    _slxARCM
} _slxANGLE_TYPE;

typedef enum {
    _slxDEGREE,
    _slxHOURANG
} _slxTIME_TYPE;

ccsCOMPL_STAT _slxAngle2Time(const slxFITS_VALUE   value,
			     const _slxANGLE_TYPE  angleType,
			     const _slxTIME_TYPE   targetTimeFormat,
			     const slxDIC_HANDLE   *dicEl,
			     slxKEYW_LINE          degree,
			     ccsERROR              *error);

int _slxEd_pic(char *dest, char *pic, long int value);

#define _slxJD_OFFSET  2400000.5 
#define _slxJD_NULL    2440587.5      /* JD for 1970-01-01 00:00:00.000000 */

void _slxJDToUTC(const vltDOUBLE   jd, 
		 unsigned long     *tv_sec,
		 long              *tv_usec)
{
    vltDOUBLE  time, sec;
    
    time = ((jd - _slxJD_NULL) * 86400.0);
    
    sec  = (int)time;
    *tv_sec  = (int)floor(sec);
    *tv_usec = (int)floor((time + 0.5e-6 - sec) * 1e6);
}


void _slxMJD2UTC(const vltDOUBLE   mjd,
		 unsigned long     *tv_sec,
		 long              *tv_usec)
{
    vltDOUBLE  jd;
    
    jd = (mjd + _slxJD_OFFSET);
    _slxJDToUTC(jd, tv_sec, tv_usec);
}

#define _slxKELVIN_0_GRAD_CEL 273.15

void _slxKelvin2Celsius(const vltDOUBLE   kelvin,
			vltDOUBLE         *celsius)
{
    *celsius = (kelvin - _slxKELVIN_0_GRAD_CEL);
}

/* Function definition 
 */
ccsCOMPL_STAT slxCreateFits2(const miscHASH_TABLE  dictionary,
			     const slxFITS_VALUE   value,
			     const vltLOGICAL      trimLine,
			     slxKEYW_LINE          fitsLine,
			     ccsERROR              *error, ...)
{
    va_list       argList;
    ccsCOMPL_STAT stat;

    va_start(argList, error);
    stat = slxCreateFits2_v(dictionary, value, trimLine, fitsLine, error, 
			    argList);
    va_end(argList);
    
    return stat;
}    


ccsCOMPL_STAT slxCreateFits2_v(const miscHASH_TABLE  dictionary,
			       const slxFITS_VALUE   value,
			       const vltLOGICAL      trimLine,
			       slxKEYW_LINE          fitsLine,
			       ccsERROR              *error, 
			       va_list               argList)
{
    char            *argPtr;
    char            *keyword;
    char            *commentPtr;
    char            *chrPtr;
    char            tmpFitsLine[_slxBUF_SAFETY_SIZE];
    char            tmpComment[_slxBUF_SAFETY_SIZE];
    char            tmpComment2[_slxBUF_SAFETY_SIZE];
    slxKEYW_LINE    key[slxMAX_NO_OF_KEY_ELS];
    slxKEYW_LINE    valueFormat;
    slxKEYW_LINE    tmpValue;
    slxKEYW_LINE    keyword2;
    slxDIC_HANDLE   *dicEl;
    slxVALUE        valueStr;
    slxCOMMENT      comment;
    vltINT32        count;
    vltINT32        i;
    vltINT32        precision;
    vltINT32        fitsLineLen, comSpace, comOffset;
    vltINT32        valOffset;
    vltINT32        len;
    vltLOGICAL      expo;
    vltLOGICAL      capitalExpo;
    vltLOGICAL      hierFits;


    *tmpFitsLine = '\0';
    precision = 0;
    memset(key, 0, (slxMAX_NO_OF_KEY_ELS * sizeof(slxKEYW_LINE)));
    /*for (i = 0; i < slxMAX_NO_OF_KEY_ELS; i++)
     *(key[i]) = '\0';
     */

    argPtr = va_arg(argList, char *);
    count = 0;
    while (argPtr != NULL)
	{
	slxCOPY_MAX_N_BYTES(key[count], argPtr, 
			    (vltINT32)sizeof(slxKEYW_LINE), error);
	if (*key[count] != '\0')
	    count++;
	argPtr = va_arg(argList, char *);
	}

    keyword = slxAssKey2(key[0], key[1], key[2], key[3], key[4], key[5], 
			 key[6], NULL);
    slxCOPY_MAX_N_BYTES(keyword2, keyword, slxKEYW_LINE_LEN, error);

    /* Check if the keyword is a Short-FITS keyword and split it up if yes
     */
    /*if (*(keyword2 + 3) == '.')*/
    if (slxIsShortFITS(keyword2))
	{
	if (slxSplitShortFitsDyn(keyword2, slxMAX_NO_OF_KEY_ELS, key, &count, 
				 valueStr, comment, error) == FAILURE)
	    return FAILURE;
	
	/*
	  if (slxSplitShortFits2(keyword2, key[0], key[1], key[2], key[3],
	  valueStr, comment, error) == FAILURE)
	  return FAILURE;
	  count = 0;
	  if ((*(key[0]) == '\0') && (*(key[1]) == '\0') &&
	  (*(key[2]) == '\0') && (*(key[3]) != '\0'))
	  {
	  "<par> = "
	  count = 1;
	  strcpy(key[0], key[3]);
	  *(key[3]) = '\0';
	  }
	  else if ((*(key[0]) != '\0') && (*(key[1]) == '\0') && 
	  (*(key[2]) == '\0') && (*(key[3]) != '\0'))
	  {
	  "HIERARCH ESO <cat> <par> = "
	  count = 2;
	  strcpy(key[1], key[3]);
	  *(key[3]) = '\0';
	  }
	  else if ((*(key[0]) != '\0') && (*(key[1]) != '\0') && 
	  (*(key[2]) == '\0') && (*(key[3]) != '\0'))
	  {
	  "HIERARCH ESO <cat> <sub> <par> = "
	  count = 3;
	  strcpy(key[2], key[3]);
	  *(key[3]) = '\0';
	  }
	  else 
	  {
	  "HIERARCH ESO <cat> <sub> <sub> <par> = "
	  count = 4;
	  }
	  */
	}

    /* Check if it is a comment
     */
    if ((count == 1) && (strstr(keyword2, "COMMENT") != NULL))
	{
	sprintf(tmpFitsLine, "COMMENT %s", value.s);
	if (trimLine)
	    {
	    strncpy(fitsLine, tmpFitsLine, slxMAX_FITS_LI_LEN);
	    *(fitsLine + slxMAX_FITS_LI_LEN) = '\0';
	    }
	else
	    {
	    if ((len = strlen(tmpFitsLine)) >= slxKEYW_LINE_LEN)
		{
		strncpy(fitsLine, tmpFitsLine, slxKEYW_LINE_LEN);
		*(fitsLine + slxKEYW_LINE_LEN - 1) = '\0';
		}
	    else
		strcpy(fitsLine, tmpFitsLine);
	    }
	    
	return SUCCESS;
	}

    /* Get the information about the keyword from the Dictionary
     */
    if (slxLookUpKeywordHashX(dictionary, keyword2, &dicEl, error) == FAILURE)
	return FAILURE;

    /* Take into account the correct positioning of '=', T,F, etc.
     */
    if (count == 1)
	{
	switch (dicEl->dicParamType)
	    {
	    case slxSTRING:
		sprintf(tmpFitsLine, "%-8s%s '%-8s' ", key[0], "=", value.s);
		break;
	    case slxLOGICAL:
		sprintf(tmpFitsLine, "%-8s%s ", key[0], "=");
		valOffset = (_slxSTD_FITS_LOG_VAL_POS-strlen(tmpFitsLine) - 1);
		sprintf(valueFormat, "%%-%dc%%c ", valOffset);
		if (value.l)
		    sprintf(tmpValue, valueFormat, ' ', 'T');
		else
		    sprintf(tmpValue, valueFormat, ' ', 'F');
		strcat(tmpFitsLine, tmpValue);
		break;
	    case slxINTEGER:
		sprintf(tmpFitsLine, "%-8s%s ", key[0], "=");
		sprintf(tmpValue, "%d", value.i);
		len = strlen(tmpFitsLine);
		valOffset = (_slxSTD_FITS_LOG_VAL_POS - len -strlen(tmpValue));
		if (valOffset > 0)
		    {
		    memset((tmpFitsLine + len), ' ', valOffset);
		    *(tmpFitsLine + len + valOffset) = '\0';
		    }
		else
		    *(tmpFitsLine + len) = '\0';
		strcat(tmpFitsLine, tmpValue);
		break;
	    default:
		sprintf(tmpFitsLine, "%-8s%s ", key[0], "=");

		/* slxDOUBLE; check the requested precision
		 */
		if (slxDecodeFormatDbl(dicEl->valFormat, &precision, &expo,
				       &capitalExpo, error) == FAILURE)
		    return FAILURE;
		if (!expo)
		    {
		    if (precision == 0)
			sprintf(valueFormat, "%%.%df.", precision);
		    else
			sprintf(valueFormat, "%%.%df", precision);
		    sprintf(tmpValue, valueFormat, value.d);
		    }
		else
		    {
		    sprintf(valueFormat, "%%.%dE", precision);
		    sprintf(tmpValue, valueFormat, value.d);
		    }
		len = strlen(tmpFitsLine);
		valOffset = (_slxSTD_FITS_LOG_VAL_POS - len -strlen(tmpValue));
		if (valOffset > 0)
		    {
		    memset((tmpFitsLine + len), ' ', valOffset);
		    *(tmpFitsLine + len + valOffset) = '\0';
		    }
		else
		    *(tmpFitsLine + len) = '\0';
		strcat(tmpFitsLine, tmpValue);
		break;
	    }
	/* Generate the comment, make an effort to make it start in the same 
	 * column for all keywords
	 */
	fitsLineLen = strlen(tmpFitsLine);
	comSpace = (slxMAX_FITS_LI_LEN - strlen(tmpFitsLine));
	if (comSpace > 0)
	    {
	    /* Check if we have to write converted values in the
	     * Comment Field
	     */
	    if (_slxConvert(value, dicEl, precision, 
			    &commentPtr, error) == FAILURE)
		return FAILURE;
	    comOffset = (_slxSTD_FITS_PREF_COM_POS - fitsLineLen - 1);
	    if (comOffset > 0)
		{
		sprintf(valueFormat, "%%-%dc%%s", comOffset);
		sprintf(tmpComment2, "/ %s", commentPtr);
		sprintf(tmpComment, valueFormat, ' ', tmpComment2);
		}
	    else
		sprintf(tmpComment, "/ %s", commentPtr);
	    strcat(tmpFitsLine, tmpComment);
	    }
	hierFits = FALSE;
	}
    else if (2 <= count <= slxMAX_NO_OF_KEY_ELS)
	{
	/* HIERARCH ESO <cat> [<sub>] <par>
	 */
	sprintf(tmpFitsLine, "%s%s", slxFITS_ESO_PREFIX, key[0]);
	for (i = 1; i < slxMAX_NO_OF_KEY_ELS; i++)
	    {
	    if (*key[i] != '\0')
		{
		strcat(tmpFitsLine, " ");
		strcat(tmpFitsLine, key[i]);
		}
	    }
	hierFits = TRUE;
	}
    else
	{
	/* Too many keyword components
	 */
	errAdd(error, slxMODULE_NAME, slxERR_CREATE_FITS, __FILE__, NULL);
	return FAILURE;
	}

    /* If hierarchical FITS, assemble the line
     */
    if (hierFits)
	{
	/* Write the equal sign
	 */
	sprintf(tmpValue, "%-29s%s ", tmpFitsLine, "=");
	strcpy(tmpFitsLine, tmpValue);

	/* Write the value as a string
	 */
	precision = 0;
	switch (dicEl->dicParamType)
	    {
	    case slxSTRING:
		sprintf(tmpValue, "\'%-8s\'", value.s);
		break;
	    case slxLOGICAL:
		valOffset = (_slxHIER_FITS_PREF_COM_POS - 
			     strlen(tmpFitsLine) - 3);
		if (valOffset < 0)
		    valOffset = 0;
		sprintf(valueFormat, "%%-%dc%%c", valOffset);
		if (value.l)
		    sprintf(tmpValue, valueFormat, ' ', 'T');
		else
		    sprintf(tmpValue, valueFormat, ' ', 'F');
		break;
	    case slxINTEGER:
		valOffset = (_slxHIER_FITS_PREF_COM_POS - 
			     strlen(tmpFitsLine) - 2);
		if (valOffset < 0)
		    valOffset = 0;
		sprintf(valueFormat, "%%%dd", valOffset);
		sprintf(tmpValue, valueFormat, value.i);
		break;
	    default:
		valOffset = (_slxHIER_FITS_PREF_COM_POS - 
			     strlen(tmpFitsLine) - 2);
		if (valOffset < 0)
		    valOffset = 0;
		/* slxDOUBLE; check the requested precision
		 */
		if (slxDecodeFormatDbl(dicEl->valFormat, &precision, &expo,
				       &capitalExpo, error) == FAILURE)
		    return FAILURE;
		if (!expo)
		    {
		    /* "%[.n]f"
		     */
		    sprintf(valueFormat, "%%%d.%df", valOffset, precision);
		    if (!precision)
			strcat(valueFormat, ".");
		    }
		else if (expo && !capitalExpo)
		    {
		    /* "%[.n]e"
		     */
		    sprintf(valueFormat, "%%%d.%de", valOffset, precision);
		    }
		else
		    {
		    /* "%[.n]E"
		     */
		    sprintf(valueFormat, "%%%d.%dE", valOffset, precision);
		    }
		sprintf(tmpValue, valueFormat, value.d);
		break;
	    }
	strcat(tmpFitsLine, tmpValue);
	strcat(tmpFitsLine, " ");

	/* Generate the comment, make an effort to make it start in the 
	 * same column for all keywords
	 */
	fitsLineLen = strlen(tmpFitsLine);
	comSpace = (slxMAX_FITS_LI_LEN - strlen(tmpFitsLine));
	if (comSpace > 0)
	    {
	    /* Check if we have to write converted values in the
	     * Comment Field
	     */
	    if (_slxConvert(value, dicEl, precision, 
			    &commentPtr, error) == FAILURE)
		return FAILURE;
	    comOffset = (_slxHIER_FITS_PREF_COM_POS - fitsLineLen - 1);
	    if (comOffset > 0)
		{
		sprintf(valueFormat, "%%-%dc%%s", comOffset);
		sprintf(tmpComment2, "/ %s", commentPtr);
		sprintf(tmpComment, valueFormat, ' ', tmpComment2);
		}
	    else
		sprintf(tmpComment, "/ %s", commentPtr);
	    strcat(tmpFitsLine, tmpComment);
	    }
	}

    /* Pad with blanks up till 80 characters if needed
     */
    len = strlen(tmpFitsLine);

    if (trimLine)
	{
	if (len < slxMAX_FITS_LI_LEN)
	    memset((tmpFitsLine + len), ' ', (slxMAX_FITS_LI_LEN - len));
	strncpy(fitsLine, tmpFitsLine, slxMAX_FITS_LI_LEN);
	*(fitsLine + slxMAX_FITS_LI_LEN) = '\0';
	
	/* If the length is longer than 80 characters and a possible
	 * string value has been truncated we have to put a single quote
	 * at the 80th character
	 */
	if ((dicEl->dicParamType == slxSTRING) && (len > slxMAX_FITS_LI_LEN))
	    {
	    /* Check if the FITS line contains two single quotes
	     */
	    if ((chrPtr = strchr(fitsLine, '\'')) == NULL)
		{
		errAdd(error,slxMODULE_NAME,slxERR_CREATE_FITS,__FILE__,NULL);
		return FAILURE;
		}
	    chrPtr++;
	    if ((chrPtr = strchr(chrPtr, '\'')) == NULL)
		*(fitsLine + (slxMAX_FITS_LI_LEN - 1)) = '\'';
	    }   /* end-if ((dicEl->dicParamType == slxSTRING) && ... */
	}
    else
	{
	if (len >= slxKEYW_LINE_LEN)
	    {
	    strncpy(fitsLine, tmpFitsLine, slxKEYW_LINE_LEN);
	    *(fitsLine + slxKEYW_LINE_LEN - 1) = '\0';
	    }
	else
	    strcpy(fitsLine, tmpFitsLine);
	}

    return SUCCESS;
}


ccsCOMPL_STAT slxDecodeFormatDbl(const char   *format,
				 vltINT32     *precision,
				 vltLOGICAL   *expo,
				 vltLOGICAL   *capitalExpo,
				 ccsERROR     *error)
{
    slxKEYW_LINE  tmpBuf;
    vltINT32      index, i;


    /* Default is "%f"
     */
    *precision = -1;
    *capitalExpo = FALSE;


    /* This version can decode the following:
     * %f, %.nf, %e, %.ne, %E, %.nE
     */
    if ((*format != '%') || (*format == '\0'))
	{
	errAdd(error, slxMODULE_NAME, slxERR_VALUE_FORMAT, __FILE__, NULL,
	       format);
	return FAILURE;
	}

    index = 1;
    i = 0;
    if (*(format + index) == '.')
	{
	index++;
	while (isdigit((int)*(format + index)) && (i < slxKEYW_LINE_LEN))
	    *(tmpBuf + i++) = *(format + index++);
	*(tmpBuf + i) = '\0';
	*precision = atoi(tmpBuf);
	}
    if (i == slxKEYW_LINE_LEN)
	{
	errAdd(error, slxMODULE_NAME, slxERR_VALUE_FORMAT, __FILE__, NULL,
	       format);
	return FAILURE;
	}
   
    switch (*(format + index))
	{
	case 'f':
	    *expo = FALSE;
	    break;
	case 'e':
	    *expo = TRUE;
	    break;
	default: 
	    /* case 'E'
	     */
	    *expo = TRUE;
	    *capitalExpo = TRUE;
	    break;
	}

    /* If the precision is not specified we take the default, namely 6
     */
    if (*precision == -1)
	*precision = 6;

    return SUCCESS;
}

ccsCOMPL_STAT _slxSecs2DayTim(const slxFITS_VALUE   value,
			      const slxDIC_HANDLE   *dicEl,
			      const vltINT32        precision,
			      slxKEYW_LINE          dayTim,
			      ccsERROR              *error)
{
    slxKEYW_LINE  tmpBuf;
    vltDOUBLE     dblVal;
    unsigned long tv_sec;
    long          tv_usec;
    vltLOGICAL    negative;

    negative = FALSE;
    switch (dicEl->dicParamType)
	{
	case slxSTRING:
	    dblVal = atof(value.s);
	    if (dblVal < 0)
		{
		negative = TRUE;
		dblVal *= -1;
		}
	    tv_sec = (unsigned long)dblVal;
	    tv_usec = (long)(1e6 * (dblVal - tv_sec));
	    break;
	case slxLOGICAL:
	    tv_sec = tv_usec = 0;
	    break;
	case slxINTEGER:
	    tv_sec = value.i;
	    tv_usec = 0;
	    if (tv_sec < 0)
		{
		negative = TRUE;
		tv_sec *= -1;
		}
	    break;
	default:
	    dblVal = (unsigned long)value.d;
	    if (dblVal < 0)
		{
		negative = TRUE;
		dblVal *= -1;
		}
	    tv_sec = dblVal;
	    tv_usec = (long)(1e6 * (value.d - tv_sec));
	    break;
	}
    
    slxGenDayTimBasic(tv_sec, tv_usec, precision, tmpBuf);
    if (negative)
	sprintf(dayTim, "-%s", tmpBuf);
    else
	sprintf(dayTim, "%s", tmpBuf);

    return SUCCESS;
}

ccsCOMPL_STAT _slxSecs2Time(const slxFITS_VALUE   value,
			    const slxDIC_HANDLE   *dicEl,
			    const vltINT32        precision,
			    slxKEYW_LINE          utcTime,
			    ccsERROR              *error)
{
    slxKEYW_LINE  tmpBuf;
    vltDOUBLE     dblVal;
    unsigned long tv_sec;
    long          tv_usec;
    vltLOGICAL    negative;

    negative = FALSE;
    switch (dicEl->dicParamType)
	{
	case slxSTRING:
	    dblVal = atof(value.s);
	    if (dblVal < 0)
		{
		negative = TRUE;
		dblVal *= -1;
		}
	    tv_sec = (unsigned long)dblVal;
	    tv_usec = (long)(1e6 * (dblVal - tv_sec));
	    break;
	case slxLOGICAL:
	    tv_sec = tv_usec = 0;
	    break;
	case slxINTEGER:
	    tv_sec = value.i;
	    if (tv_sec < 0)
		{
		negative = TRUE;
		tv_sec *= -1;
		}
	    tv_usec = 0;
	    break;
	default:
	    dblVal = value.d;
	    if (dblVal < 0)
		{
		negative = TRUE;
		dblVal *= -1;
		}
	    tv_sec = dblVal;
	    tv_usec = (long)(1e6 * (value.d - tv_sec));
	    break;
	}

    slxGenUTC(tv_sec, tv_usec, precision, tmpBuf);
    if (negative)
	sprintf(utcTime, "-%s", tmpBuf);
    else
	sprintf(utcTime, "%s", tmpBuf);

    return SUCCESS;
}

ccsCOMPL_STAT _slxMjd2DayTim(const slxFITS_VALUE   value,
			     const slxDIC_HANDLE   *dicEl,
			     const vltINT32        precision,
			     slxKEYW_LINE          dayTim,
			     ccsERROR              *error)
{
    slxKEYW_LINE  tmpBuf;
    vltDOUBLE     mjd;
    unsigned long tv_sec;
    long          tv_usec;
    vltLOGICAL    negative;

    negative = FALSE;
    switch (dicEl->dicParamType)
	{
	case slxSTRING:
	    mjd = atof(value.s);
	    if (mjd < 0)
		{
		negative = TRUE;
		mjd *= -1;
		}
	    break;
	case slxLOGICAL:
	    mjd = 0;
	    break;
	case slxINTEGER:
	    mjd = value.i;
	    if (mjd < 0)
		{
		negative = TRUE;
		mjd *= -1;
		}
	    break;
	default:
	    mjd = value.d;
	    if (mjd < 0)
		{
		negative = TRUE;
		mjd *= -1;
		}
	    break;
	}
    _slxMJD2UTC(mjd, &tv_sec, &tv_usec);
    slxGenDayTimBasic(tv_sec, tv_usec, precision, tmpBuf);
    if (negative)
	sprintf(dayTim, "-%s", tmpBuf);
    else
	sprintf(dayTim, "%s", tmpBuf);

    return SUCCESS;
}

ccsCOMPL_STAT _slxAngle2Time(const slxFITS_VALUE   value,
			     const _slxANGLE_TYPE  angleType,
			     const _slxTIME_TYPE   targetTimeFormat,
			     const slxDIC_HANDLE   *dicEl,
			     slxKEYW_LINE          degree,
			     ccsERROR              *error)
{
    slxKEYW_LINE  tmpBuf;
    vltDOUBLE     angle;
    vltLOGICAL    negative;

    negative = FALSE;
    switch (dicEl->dicParamType)
	{
	case slxSTRING:
	    if ((angle = atof(value.s)) < 0)
		{
		negative = TRUE;
		angle *= -1;
		}
	    break;
	case slxLOGICAL:
	    angle = 0;
	    break;
	case slxINTEGER:
	    if ((angle = value.i) < 0)
		{
		negative = TRUE;
		angle *= -1;
		}
	    break;
	default:
	    if ((angle = value.d) < 0)
		{
		negative = TRUE;
		angle *= -1;
		}
	    break;
	}
 
    if ((angleType == _slxDEG) && (targetTimeFormat == _slxHOURANG))
	_slxEd_pic(tmpBuf, "99:59:59.9", (angle * 36000. / 15.));
    else if ((angleType == _slxDEG) && (targetTimeFormat == _slxDEGREE))
	_slxEd_pic(tmpBuf, "99:59:59.9", (angle * 36000));
    else if ((angleType == _slxRAD) && (targetTimeFormat == _slxHOURANG))
	_slxEd_pic(tmpBuf, "99:59:59.9", ((angle * 57.29578) * 36000. / 15.));
    else if ((angleType == _slxRAD) && (targetTimeFormat == _slxDEGREE))
	_slxEd_pic(tmpBuf, "99:59:59.9", ((angle * 57.29578) * 36000));
    else if ((angleType == _slxARCM) && (targetTimeFormat == _slxHOURANG))
	_slxEd_pic(tmpBuf, "99:59:59.9", (angle * 60. / 15.));
    else 
	{
	/* if ((angleType == _slxDEG) && (targetTimeFormat == _slxDEGREE)) */
	_slxEd_pic(tmpBuf, "99:59:59.9", (angle * 60.));
	}

    if (negative)
	sprintf(degree, "-%s", tmpBuf);
    else
	sprintf(degree, "%s", tmpBuf);

    return SUCCESS;
}

ccsCOMPL_STAT _slxConvert(const slxFITS_VALUE   value,
			  const slxDIC_HANDLE   *dicRec,
			  const vltINT32        precision,
			  char                  **commentPtr,
			  ccsERROR              *error)
{
    static slxKEYW_LINE commentBuf;
    slxKEYW_LINE        tmpValue;
    slxKEYW_LINE        valueFormat;
    vltDOUBLE           dblVal;

    switch (dicRec->conversion)
	{
	case slxCONV_NONE:
	    *commentPtr = (char *)dicRec->comField;
	    break;
	case slxCONV_SECS_TO_TIME:
	    if (_slxSecs2Time(value, dicRec, 3, tmpValue,
			      error) == FAILURE)
		return FAILURE;
	    sprintf(commentBuf, dicRec->comFieldFormat, tmpValue);
	    *commentPtr = commentBuf;
	    break;
	case slxCONV_SECS_TO_DAYTIM:
	    if (_slxSecs2DayTim(value, dicRec, 3, tmpValue,
				error) == FAILURE)
		return FAILURE;
	    sprintf(commentBuf, dicRec->comFieldFormat, tmpValue);
	    *commentPtr = commentBuf;
	    break;
	case slxCONV_MJD_TO_DAYTIM:
	    if (_slxMjd2DayTim(value, dicRec, 3, tmpValue,
			       error) == FAILURE)
		return FAILURE;
	    sprintf(commentBuf, dicRec->comFieldFormat, tmpValue);
	    *commentPtr = commentBuf;
	    break;
	case slxCONV_DEG_TO_DEGREE:
	    if (_slxAngle2Time(value, _slxDEG, _slxDEGREE, dicRec, 
			       tmpValue, error) == FAILURE)
		return FAILURE;
	    sprintf(commentBuf, dicRec->comFieldFormat, tmpValue);
	    *commentPtr = commentBuf;
	    break;
	case slxCONV_DEG_TO_HOURANG:
	    if (_slxAngle2Time(value, _slxDEG, _slxHOURANG, dicRec, 
			       tmpValue, error) == FAILURE)
		return FAILURE;
	    sprintf(commentBuf, dicRec->comFieldFormat, tmpValue);
	    *commentPtr = commentBuf;
	    break;
	case slxCONV_RAD_TO_DEGREE:
	    if (_slxAngle2Time(value, _slxRAD, _slxDEGREE, dicRec, 
			       tmpValue, error) == FAILURE)
		return FAILURE;
	    sprintf(commentBuf, dicRec->comFieldFormat, tmpValue);
	    *commentPtr = commentBuf;
	    break;
	case slxCONV_RAD_TO_HOURANG:
	    if (_slxAngle2Time(value, _slxRAD, _slxHOURANG, dicRec, 
			       tmpValue, error) == FAILURE)
		return FAILURE;
	    sprintf(commentBuf, dicRec->comFieldFormat, tmpValue);
	    *commentPtr = commentBuf;
	    break;
	case slxCONV_ARCM_TO_DEGREE:
	    if (_slxAngle2Time(value, _slxARCM, _slxDEGREE, dicRec, 
			       tmpValue, error) == FAILURE)
		return FAILURE;
	    sprintf(commentBuf, dicRec->comFieldFormat, tmpValue);
	    *commentPtr = commentBuf;
	    break;
	case slxCONV_ARCM_TO_HOURANG:
	    if (_slxAngle2Time(value, _slxARCM, _slxHOURANG, dicRec, 
			       tmpValue, error) == FAILURE)
		return FAILURE;
	    sprintf(commentBuf, dicRec->comFieldFormat, tmpValue);
	    *commentPtr = commentBuf;
	    break;
	case slxCONV_KELVIN_TO_CELSIUS:
	    _slxKelvin2Celsius(value.d, &dblVal);		    
	    sprintf(valueFormat, "%%.%dfC", precision);
	    sprintf(tmpValue, valueFormat, dblVal);
	    sprintf(commentBuf, dicRec->comFieldFormat, tmpValue);
	    *commentPtr = commentBuf;
	    break;
	default:
	    errAdd(error, slxMODULE_NAME, slxERR_CREATE_FITS, __FILE__, NULL);
	    return FAILURE;
	    break;		
	}
    
    return SUCCESS;
}

/*___oOo___*/
