/******************************************************************************
* E.S.O. - VLT project
*
* slxCreateShortFITS.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  28/07/94  Created 1. version
*/

/************************************************************************
*   NAME
*   slxCreateShortFITS - generates a FITS keyword command line
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxCreateShortFITS(const char         *category,
*                                    const char         *subSystem,
*                                    const char         *parameter,
*                                    const slxPAR_TYPE  parameterType,
*                                    const char         *stringValue,
*                                    const vltLOGICAL   logicalValue,
*                                    const long int     integerValue,
*                                    const vltUINT8     precision,
*                                    const vltLOGICAL   exponential,
*                                    const vltDOUBLE    doubleValue,
*                                    const char         *comment,
*                                    char               *fitsLine,
*                                    ccsERROR           *error)
*
*
*   DESCRIPTION
*   NOTE: A new function slxCreateShortFits2() is now supported providing a 
*   more convenient interface, and providing two sub-system keywords.
*   This new function should be used.
*
*   The function generates a Short-FITS keyword line from the input 
*   parameters. This is done in accordance with the rules specified in
*   "INS Common SW -Part 2, Set-Up Files and FITS Log Handling - SLX,
*   Doc. No.: VLT-SPE-ESO-17240-0666", which is based on the original 
*   definition of FITS in the "Data Interface Requirements" document 
*   (ARC-SPE-ESO-0000-0001/1.4), and in "NOST Implementation of the Flexible 
*   Image Transport System - FITS, November 6, 1991".
*   
*   The Short-FITS keyword line generated can be either a non-hierarchical 
*   keyword, i.e. only a parameter, a value and a comment:
*
*   "<par>    <value>;                         # <comment>"
*
*   or it can be a hierarchical Short-FITS keyword line, i.e. with the format: 
*
*   "<cat>.[<sub>.]<par>   <value>; # <comment>"
*  
*   Finally the function can generate a line which only consists of a comment:
*
*   # <comment>
*
*   All strings passed to the function must be NULL terminated.
*
*   If a non-hierarchical Short-FITS keyword should be generated, slxSTD_FITS, 
*   should used as input for the category. If there is no subsystem, 
*   slxNULL_STR, should be the input value for this parameter. 
*
*   The comment should be given only as the actual text -  do not append any
*   "#" or try to format it in any way.
*
*   The generated Short-FITS line is stored in the buffer pointed to by 
*   "fitsLine". If the generated Short-FITS line is longer than the maximum 
*   FITS line length (80 characters) the characters beyond the 80th position,
*   are not taken into account.
*
*
*   category      IN  Pointer to string buffer containing the category of the 
*                     Short-FITS keyword, e.g. "INS", "DET", ... . If it is a 
*                     non-hierarchical FITS keyword (only the parameter), 
*                     this string should be slxSTD_FITS.
*
*   subSystem     IN  Pointer to string buffer containing the subsystem.
*                     If this is not used, slxNULL_STR, must be the input
*                     parameter.
*   
*   parameter     IN  Pointer to string buffer containing the name of the 
*                     parameter. This input parameter is mandatory.
*
*   parameterType IN  The type of the parameter. Can be either slxSTRING,
*                     slxLOGICAL, slxINTEGER or slxDOUBLE.
*
*   stringValue   IN  Pointer to buffer containing the possible string value.
*                     If not used, i.e. "parameterType" is not a string, this
*                     parameter is ignored (can be slxNULL_STR).
*
*   logicalValue  IN  Can be either TRUE or FALSE. If not used, the 
*                     parameter is ignored.
*
*   integerValue  IN  Contains an integer value. If "parameterType" is not
*                     integer, this is ignored.
*
*   precision     IN  Used when the input value is of type double. Indicates
*                     the number of decimals after the point. If the value is
*                     not a double, this parameter is ignored.
*
*   exponential  IN   Indicates for a double type parameter if it should be 
*                     be written as an exponential number or a plain number.
*                     Can be either TRUE for exponential or FALSE. If 
*                     the value is not a double, this parameter is ignored.
*
*   doubleValue  IN   Contains the value when parameter type is double. If the
*                     type is not double, this parameter is ignored.
*
*   comment       IN  Used to give in the comment; should be plain text
*                     terminated with "\x0".
*
*   fitsLine      OUT The buffer in which the generated Short-FITS line will 
*                     be stored. Must be allocated by the calling application.
*                     Use the contant slxMAX_FITS_LI_LEN to allocate the 
*                     buffer.
*
*
*   RETURN VALUES 
*   SUCCESS    If the operation was successful.
*   FAILURE    If the operation failed. The function only fails if the 
*              parameter type is unknown, otherwise it will always try to
*              generate a Short-FITS line.
*
*              The function make calls to slxCreateFITS() and 
*              slxNormalToShortFITS() and may return errors generated by these.
*
*   CAUTIONS 
*   Beware that all string buffers must be allocated by the calling process,
*   and terminated by a null character ('\0').
*
*
*   EXAMPLES
*   See slxCreateFITS(3).
*    
*   SEE ALSO
*   slxStoreLine()
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxCreateShortFITS.c,v 1.145 2005/07/27 09:07:56 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <string.h>

/* Local Headers
 */
#include "slx.h"         

/* Function definition
 */
ccsCOMPL_STAT slxCreateShortFITS(const char         *category,
                                 const char         *subSystem,
                                 const char         *parameter,
                                 const slxPAR_TYPE  parameterType,
                                 const char         *stringValue,
                                 const vltLOGICAL   logicalValue,
                                 const long int     integerValue,
                                 const vltUINT8     precision,
                                 const vltLOGICAL   exponential,
                                 const vltDOUBLE    doubleValue,
                                 const char         *comment,
                                 char               *fitsLine,
                                 ccsERROR           *error)
{
    char normalFITS[slxMAX_FITS_LI_LEN + 1];

    if (slxCreateFITS(category, subSystem, parameter, parameterType, 
                      stringValue, logicalValue, integerValue, precision, 
                      exponential, doubleValue, comment, normalFITS, 
                      error) == FAILURE)
        return FAILURE;

    if (slxNormalToShortFITS(normalFITS, fitsLine, error) == FAILURE)
        return FAILURE;

    return SUCCESS;

}

/*
 *****************************************************************************
 */

 
/*___oOo___*/
