/******************************************************************************
* E.S.O. - VLT project
*
* slxCreateShortFits2.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  28/03/96  created
* - - - - - -MAR2002 - -- - - -
* epozna    27/05/02  fixed bug in va_list/va_end :
*                     slxCreateShortFitsX()slxCreateShortFits2
*/

/************************************************************************
*   NAME
*   slxCreateShortFits2 - generates a FITS keyword command line
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxCreateShortFits2(const miscHASH_TABLE  dictionary,
*                                     const slxFITS_VALUE   value,
*                                     slxKEYW_LINE          fitsLine,
*                                     ccsERROR              *error, ..., NULL)
*
*
*   DESCRIPTION
*   The function generates a Short-FITS keyword line from the input 
*   parameters. This is done in accordance with the rules specified in
*   "INS Common SW -Part 2, Set-Up Files and FITS Log Handling - SLX,
*   Doc. No.: VLT-SPE-ESO-17240-0666", which is based on the original 
*   definition of FITS in the "Data Interface Requirements" document 
*   (ARC-SPE-ESO-0000-0001/1.4), and in "NOST Implementation of the Flexible 
*   Image Transport System - FITS, November 6, 1991".
*   
*   The Short-FITS keyword line generated can be either a non-hierarchical 
*   keyword, i.e. only a parameter, a value and a comment:
*
*   "<par>    <value>;                         # <comment>"
*
*   or it can be a hierarchical Short-FITS keyword line, i.e. with the format: 
*
*   "<cat>.[<sub>.]<par>   <value>; # <comment>"
*  
*   Finally the function can generate a line which only consists of a comment:
*
*   # <comment>
*
*   All strings passed to the function must be NULL terminated.
*
*   The comment should be given only as the actual text -  do not append any
*   "#" or try to format it in any way.
*
*   The generated Short-FITS line is stored in the buffer pointed to by 
*   "fitsLine".
*
*   If for a string keyword the macro slxFITS_NO_VALUE is used, a line without
*   a value is generated, like for instance:
*
*   PAF.HDR.START;                          # Start of PAF Header
*
*
*   dictionary  IN     MISC Hash Table containing the Dictionary for the
*                      context. Information such as the type of keyword,
*                      value format, and comment is obtained from the 
*                      Dictionary.          
*
*   value       IN     Pointer to the value to be written in the FITS line.
*
*   fitsLine    OUT    Pointer to array where the new FITS line will be 
*                      written. The line must be at least 80 characters long;
*                      use possibly slxKEYW_LINE as type for the variable.
*
*   error       OUT    Pointer to CCS Error Structure.
*
*
*   RETURN VALUES 
*   SUCCESS    If the operation was successful.
*   FAILURE    If the operation failed.
*
*   CAUTIONS 
*
*   EXAMPLES
*   See slxCreateFits2(3)
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxCreateShortFits2.c,v 1.145 2005/07/27 09:08:37 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);


/* System Headers
 */
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

/* Local Headers
 */
#include "slx.h"         

/* Function definition
 */
#define  _slxPAD_BUF "                                                       "
ccsCOMPL_STAT slxCreateShortFitsX(const miscHASH_TABLE  dictionary,
				  const slxFITS_VALUE   value,
				  slxKEYW_LINE          fitsLine,
				  ccsERROR              *error, ...)
{
    char            *argPtr;  
    char            *commentPtr;
    char            *chrPtr;
    va_list         argList; 
    slxCOMMENT      comment;
    slxDIC_HANDLE   *dicEl;
    slxKEYW_LINE    keyword;
    slxKEYW_LINE    valueFormat;
    vltINT32        precision;
    vltLOGICAL      expo; 
    vltLOGICAL      capitalExpo;
    slxVALUE        valueStr;
    slxVALUE        valStr;
    vltINT32        padLen;


    /* Generate keyword
     */
    *keyword = '\0';
    va_start(argList, error);
    argPtr = va_arg(argList, char *);
    do
	{
	if (*argPtr != '\0')
	    {
	    if (*keyword != '\0')
		strcat(keyword, ".");
	    strcat(keyword, argPtr);
	    }
	argPtr = va_arg(argList, char *);
	}
    while (argPtr != NULL);
    va_end(argList);

    if (slxLookUpKeywordHashX(dictionary, keyword, &dicEl, error) == FAILURE)
	return FAILURE;

    /* Generate value
     */
    precision = 0;
    switch (dicEl->dicParamType)
	{
	case slxSTRING:
	    strcpy(valStr, value.s);
	    slxStripQuotes(valStr);
	    slxTrimString(valStr, " ");
	    sprintf(valueStr, "\"%s\"", valStr);
	    break;
	case slxLOGICAL:
	    if (value.l)
		sprintf(valueStr, "\"T\"");
	    else
		sprintf(valueStr, "\"F\"");
	    break;
	case slxINTEGER:
	    sprintf(valueStr, "%d", value.i);
	    break;
	default:
	    /* slxDOUBLE; check the requested precision
	     */
	    if (slxDecodeFormatDbl(dicEl->valFormat, &precision, &expo,
				   &capitalExpo, error) == FAILURE)
		return FAILURE;
	    if (!expo)
		{
		if (precision == 0)
		    sprintf(valueFormat, "%%.%df.", precision);
		else
		    sprintf(valueFormat, "%%.%df", precision);
		sprintf(valueStr, valueFormat, value.d);
		}
	    else
		{
		sprintf(valueFormat, "%%.%dE", precision);
		sprintf(valueStr, valueFormat, value.d);
		}
	    break;
	}

    /* Generate comment
     */
    if (_slxConvert(value, dicEl, precision, &commentPtr, error) == FAILURE)
	return FAILURE;
    strcpy(comment, commentPtr);

    /*
     * 1                  25 27             52 54                     
     * |                   | |               | |                      
     * INS.OPTI3.ID.TYPExxxx string;         # Keyword type
     *
     */
    memset(fitsLine, 0, sizeof(slxKEYW_LINE));
    sprintf(fitsLine, "%-25s %s; ", keyword, valueStr);
 
    /* If the line is shorter than 49 we pad up to 49 
     */
    padLen = (51 - strlen(fitsLine));
    if (padLen > 0)
	strncpy((fitsLine + strlen(fitsLine)), _slxPAD_BUF, padLen);

    /* Write the comment
     */
    strcat(fitsLine, "# ");
    strcat(fitsLine, comment);

    /* If the Short-FITS line contains the slxFITS_NO_VALUE string, the value
     * is removed and a semicolon put after the keyword. E.g.:
     *
     *   PAF.HDR.START          "$$_no_value_$$";  # Start of PAF Header
     *                                 |
     *                                 V
     *   PAF.HDR.START;                            # Start of PAF Header
     */
    if ((chrPtr = strstr(fitsLine, slxFITS_NO_VALUE)) != NULL)
	{
	chrPtr--;
	while ((*chrPtr != ';') && (*chrPtr != '\0'))
	    {
	    *chrPtr = ' ';
	    chrPtr++;
	    }
	if ((*chrPtr != '\0') && (*chrPtr != ' ') && (*chrPtr != '#'))
	    *chrPtr = ' ';
	chrPtr = strchr(fitsLine, ' ');
	*chrPtr = ';';
	}

    return SUCCESS;
}


ccsCOMPL_STAT slxCreateShortFits2(const miscHASH_TABLE  dictionary,
				  const slxFITS_VALUE   value,
				  slxKEYW_LINE          fitsLine,
				  ccsERROR              *error, ...)
{
    char            *chrPtr;
    va_list         argList; 
    ccsCOMPL_STAT   stat;
    slxKEYW_LINE    normalFits;
  

    va_start(argList, error);
    stat = slxCreateFits2_v(dictionary, value, FALSE,normalFits,error,argList);
    va_end(argList);

    if (stat == FAILURE)
	return FAILURE;

    if (slxNormalToShortFITS(normalFits, fitsLine, error) == FAILURE)
        return FAILURE;

    /* If the Short-FITS line contains the slxFITS_NO_VALUE string, the value
     * is removed and a semicolon put after the keyword. E.g.:
     *
     *   PAF.HDR.START          "$$_no_value_$$";  # Start of PAF Header
     *                                 |
     *                                 V
     *   PAF.HDR.START;                            # Start of PAF Header
     */
    if ((chrPtr = strstr(fitsLine, slxFITS_NO_VALUE)) != NULL)
	{
	chrPtr--;
	while ((*chrPtr != ';') && (*chrPtr != '\0'))
	    {
	    *chrPtr = ' ';
	    chrPtr++;
	    }
	if ((*chrPtr != '\0') && (*chrPtr != ' ') && (*chrPtr != '#'))
	    *chrPtr = ' ';
	chrPtr = strchr(fitsLine, ' ');
	*chrPtr = ';';
	}

    return SUCCESS;
}

/*
 *****************************************************************************
 */

 
/*___oOo___*/
