/*****************************************************************************
* E.S.O. - VLT project
*
* slxCreateWriteFITS.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  02/09/94  Created 1. version
*/

/************************************************************************
*   NAME
*   slxCreateWriteFITS - creates FITS line and writes this to an image file
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxCreateWriteFITS(const int          fd,
*                                    const char         *category,
*                                    const char         *subSystem,
*                                    const char         *parameter,
*                                    const slxPAR_TYPE  parameterType,
*                                    const char         *stringValue,
*                                    const vltLOGICAL   logicalValue,
*                                    const vltINT32     integerValue,
*                                    const vltUINT8     precision,
*                                    const vltLOGICAL   exponential,
*                                    const vltDOUBLE    doubleValue,
*                                    const char         *comment,
*                                    ccsERROR           *error)
*
* 
*   DESCRIPTION
*   NOTE: A new function slxCreateFits2() is now supported providing a more
*   convenient interface, and providing two sub-system keywords.
*   This new function should be used.
*
*   The function creates a FITS line by calling the function "slxCreateFITS()",
*   and appends this in the file specified by the file descriptor. The function
*   assumes that the file pointer points to the correct place in the file, and
*   simply writes the new FITS line at this position.
*
*   For further information about the generation of the FITS keywords, see the
*   man page for "slxCreateFITS()".
*
* 
*   fd    IN  File descriptor for the image data file which is opened and ready
*             to be written in. File pointer must point to the correct location
*             in the file.
*
*   For an explanation of the rest of the parameters, consult the man page for
*   the function "slxCreateFITS()".
*
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successful and the FITS line was 
*             written successfully in the file.
*   FAILURE   If something went wrong. The following error codes can be 
*             returned:
*
*             slxERR_WRONG_FD       The error descriptor is wrong
*
*             slxERR_WRITE_FILE_FD  An error occurred while writing in a file
*                                   referenced by the file descriptor.
*
*             The function calls slxCreateFITS() and may return errors 
*             generated by this function.
*
*   CAUTIONS 
*   See man page for "slxCreateFITS()".
*
*   EXAMPLES
*   See slxCreateFITS(3).
*
*   SEE ALSO
*   slxCreateFITS()
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"


static char *rcsId="@(#) $Id: slxCreateWriteFITS.c,v 1.145 2005/07/27 09:09:15 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);


/* 
 * System Headers
 */
#include <unistd.h>
#include <sys/stat.h>


/* 
 * Local Headers
 */
#include "slx.h"

/* 
 * Function definition
 */
ccsCOMPL_STAT slxCreateWriteFITS(const int          fd,
                                 const char         *category,
                                 const char         *subSystem,
                                 const char         *parameter,
                                 const slxPAR_TYPE  parameterType,
                                 const char         *stringValue,
                                 const vltLOGICAL   logicalValue,
                                 const vltINT32     integerValue,
                                 const vltUINT8     precision,
                                 const vltLOGICAL   exponential,
                                 const vltDOUBLE    doubleValue,
                                 const char         *comment,
                                 ccsERROR           *error)

{
    char        fitsLine[slxMAX_FITS_LI_LEN];
    struct stat buf;


    /* Check the file descriptor
     */
    if (fstat(fd, &buf) != 0)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_WRONG_FD, __FILE__, NULL,
                  modUNIX, fd);
        return FAILURE;
        }

    /* Create the FITS line
     */
    if (slxCreateFITS(category, subSystem, parameter, parameterType, 
                      stringValue, logicalValue, integerValue, precision,
                      exponential, doubleValue, comment, fitsLine, 
                      error) == FAILURE)
        return FAILURE;

    if (write(fd, fitsLine, (int)slxMAX_FITS_LI_LEN) < slxMAX_FITS_LI_LEN)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE_FD, __FILE__, NULL,
                  modUNIX, fd);
        return FAILURE;
        }
    return SUCCESS;
}

/*
 *****************************************************************************
 */

/*___oOo___*/


