/******************************************************************************
* E.S.O. - VLT project
*
* slxDetectFormat.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  27/07/94  Created 1. version
*/

/************************************************************************
*   NAME
*   slxDetectFormat - detects the format of a set-up file
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxDetectFormat(const slxFILENAME  fileName,
*				  slxSETUP_FORMAT    *type,
*                                 ccsERROR           *error)
*
* 
*   DESCRIPTION
*   The function checks the format of a set-up to determine whether the format
*   is the normal FITS or the Short-FITS. The type is written to the variable
*   pointed to by "type".
* 
*
*   filePtr	IN	Points to the file that should be checked.
*
*   type	IN	Points to variable (provided from calling application)
*    			in which the result of the check is stored. Can be 
*			either "slxNORMAL_FITS", "slxSHORT_FITS" or 
*			"slxUNKNOWN_FORMAT".
*			
*
*   error       OUT     CCS error structure in which a possible error 
*                       occurring is returned.
*
*
*   RETURN VALUES 
*   SUCCESS	In case the operation was successful.
*
*   FAILURE	In case the operation went wrong. Could be a problem with the
*    		file access. The following error codes can be returned:
*
*               slxERR_INS_SETUPPATH  Could get the value of this
*                                     environement variable.
*
*               slxERR_OPEN_FILE      An error occurred while opening the file.
*
*
*               The function makes a call to slxLocateSetupFile() and may
*               return errors generated by this function.
*
*   SEE ALSO
*   slxShortToNormalFITS(), slxNormalToShortFITS(), slxConvertToNormal(),
*   slxConvertToShort()   
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxDetectFormat.c,v 1.145 2005/07/27 09:10:27 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <string.h>

/* Local Headers
 */
#include "slx.h" 

/* Function definition
 */
ccsCOMPL_STAT slxDetectFormat(const slxFILENAME fileName,
			      slxSETUP_FORMAT   *type,
                              ccsERROR          *error)

{
    char           lineRead[slxMAX_FITS_LI_LEN + 1];
    char           *readStatus;
    FILE           *filePtr;
    slxFILENAME    completeName;
    vltUINT8       found = 0;



    if (slxLocateSetupFile(fileName, completeName, error) == FAILURE)
        return FAILURE;
   
    if ((filePtr = fopen(completeName, "r")) == NULL)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, "%s",
                  modUNIX, completeName);
        return FAILURE;
        }

    /*
     * Keep on reading in lines until we are sure.
     *
     * The criteria to detect the Normal hierarchical format is
     * to find the string "HIERARCH ESO" as the first part of the
     * line. 
     * 
     * For Short-FITS the critetia is to find a dot (".") on the
     * fourth position and a ";" in the line.
     */
  
    do 
        {
        readStatus = fgets(lineRead, (slxMAX_FITS_LI_LEN + 1), filePtr);

        /*
         * Normal FITS?
         */
          
        if (strncmp(lineRead, slxFITS_ESO_PREFIX, 
            strlen(slxFITS_ESO_PREFIX)) == 0)
            {
            *type = slxNORMAL_FITS;
            found = 1;
            } 
 
        /*
         * Short-FITS?
         */
     
        if ((*(lineRead + 3) == '.') && (strstr(lineRead, ";") != NULL))
	    {
            *type = slxSHORT_FITS;
            found = 1;
            }
        }
    while (!found && (readStatus != NULL));

    if (found == FALSE)
        *type = slxUNKNOWN_FORMAT;        
       
    fclose(filePtr);

    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/






