/******************************************************************************
* E.S.O. - VLT project
*
* slxGetKeyword.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  23/10/94  created
* jknudstr  31/05/95  Fixed bug when storing a new keyword in a buffer
*                     with a keyword being a subset of this  
* epozna    17/08/01  VLTSW20010283 manpage update
*                                   example added
*/

/************************************************************************
*   NAME
*   slxGetKeyword - searches for a keyword in a set-up file
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxGetKeyword(const char        *setupFilename,
*                               const char        *keyword,
*                               const vltUINT16   keywordNumber,
*                               slxKEYW_INFO      *keywordInfo,
*                               ccsERROR          *error)
*
* 
*   DESCRIPTION
*   The function  searches for a  keyword in the given set-up file. 
*   The keyword can be identified by its name or its serial number.
*   If the keyword is found, the information about this keyword
*   is returned in the slxKEYW_INFO structure. 
*
*   When the name of the keyword i.e. the argument 'keyword' is given as 
*   a non-zero-length string, the function search for the kewyord with 
*   the given name. In this case the parameter 'keywordNumber' is ignored.
*   
*   When the name of the keyword i.e. the argument 'keyword' is given as  
*   zero-length string, the function will search for the keyword with the 
*   given serial number as indicated by argument 'keywordNumber'.
*
*   Arguments of the function:
*   setupFilename   IN  The name of the set-up file where the
*                       keyword will be searched.
*
*   keyword         IN  The name of the keyword. Must be in the Short-FITS
*                       format. If given as a zero-length string, the  
*                       search is carried out for the specified keyword 
*                       number.
*
*   keywordNumber   IN  The number of the keyword in the set-up file. The 
*                       set-up header is considered. The first keyword has
*                       number 1. This argument is ignored when 'keyword' 
*                       is not a zero-length string. 
*
*   keywordInfo     OUT Structure that contains the various information about
*                       the keyword.
*                       This structure contains the following members:
*      
*                       vltUINT8  found     - Indicates if the keyword was 
*                                             found.
*                       vltUINT16 number    - Line number in set-up file
*                       char   keyword[200] - The complete keyword 
*                                             (Short-FITS)
*                       char   category[4]  - The category of the keyword, 
*                                             or empty if not defined.
*                       char   subSystem[16]- The subsystem of the keyword, 
*                                             or empty if not defined.
*                       char   parameter[16]- The parameter of the keyword.
*                       char   value[200]   - The value as a string. A string
*                                             value keeps the quotes. 
*                       char   comment[200] - The possible comment of the 
*                                             keyword line.
*
*   error           OUT CCS error structure that contains information about
*                       a possible error occurring.
*
*
*   FILES
*   The set-up file to be accessed must be in the Short-FITS format, and must
*   be located in one of the directories defined by INS_SETUPPATH.
*
*   ENVIRONMENT
*   INS_SETUPPATH   Is a colon separated list of paths, in which set-up files
*                   and set-up related files must be situated.
*                   If not set, uses the current directory.
*
*   RETURN VALUES 
*   SUCCESS   If the keyword was found and the information read successfully.
*   FAILURE   If something went wrong during the operation, the following 
*             error codes can be returned by the function:
*
*                   slxERR_OPEN_FILE         Error occurred while opening the
*                                            specified file.
*
*                   slxERR_LOCATE_KEYWORD    The keyword was not located.
*
*                   slxERR_KEYWORD_NOT_FOUND The specified keyword was not
*                                            found.
*
*                   The errors of the function slxLocateSetupFile() are 
*                   returned when file with 'setupFilename' is not found.
*
*   EXAMPLES
*    example-1: search according to the name of a keyword 
*    ^^^^^^^^^
*    slxKEYW_INFO     keywordInfo;
*    char * filename="$INS_ROOT/SYSTEM/COMMON/SETUPFILES/DET/ccd.det";
*    if (slxGetKeyword(filename,"DET2.FRAM.FITSMTD",
*	   	       0,&keywordInfo, stack)!=SUCCESS) {error handling}
*    
*    printf("found=%d\n",keywordInfo.found); 
*    printf("line number=%d\n",keywordInfo.number); 
*    printf("complte kw=%s\n",keywordInfo.keyword);
*    printf("value =%s\n",keywordInfo.value);
*    printf("comment=%s\n",keywordInfo.comment);
*
*    example-2: search according to serial number
*    ^^^^^^^^^
*    if (slxGetKeyword( filename,"\0", 7,&keywordInfo,
*		    stack)!=SUCCESS) {error handling}
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxGetKeyword.c,v 1.145 2005/07/27 09:14:40 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxGetKeyword(const char       *setupFilename,
                            const char       *keyword,
                            const vltUINT16  keywordNumber,
                            slxKEYW_INFO     *keywordInfo,
                            ccsERROR         *error)
{         
    char           *charPtr;
    char           *bufPtr;
    FILE           *filePtr;
    slxFILENAME    completeName; 
    slxKEYW_LINE   wantedLine; 
    slxKEYW_LINE   inBuf;
    slxKEYW_LINE   inKeyword;
    vltUINT8       found;
    vltUINT16      keywordCount;
    

    if (slxLocateSetupFile(setupFilename, completeName, error) == FAILURE)
        return FAILURE;

    if ((filePtr = fopen(completeName, "r")) == NULL)
        {
        errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, NULL, 
                  modUNIX, completeName);  
        return FAILURE;
        }

    found = 0;
    keywordInfo->number = 0;
    if (strlen(keyword) == 0)
        {

        /*
         * Search for a specific keyword number
         */

        keywordCount = 0;
        while ((fgets(inBuf, 128, filePtr) != NULL) && 
               (keywordNumber != keywordCount))
            {
            (keywordInfo->number)++;
            if ((*inBuf != ' ') && (*inBuf != '#') && (*inBuf != '\t') &&
                (*inBuf != '\n') && (*(inBuf + 3) == '.'))
                keywordCount++;
            if (keywordNumber == keywordCount)
                strcpy((char *)wantedLine, (char *)inBuf);
            }

        if ((keywordNumber == keywordCount) && (keywordNumber > 0))
            {
            found = 1;
            if ((charPtr = strchr((char *)wantedLine, ' ')) != NULL)
                {
                strncpy((char *)keywordInfo->keyword, wantedLine, 
                        (size_t)(charPtr - wantedLine));
                *(keywordInfo->keyword + (charPtr - wantedLine)) = '\0';
                }
            }
        }
    else
        {
        while ((fgets(inBuf, slxKEYW_LINE_LEN, filePtr) != NULL) && !found)
            {
            (keywordInfo->number)++;
	    bufPtr = inBuf;
	    if (slxGetNextField(bufPtr, slxKEYW_LINE_LEN, inKeyword,
				&bufPtr, error) == FAILURE)
		return FAILURE;
	    if (strcmp((char *)keyword, inKeyword) == 0)
		{
		found = 1;
		strcpy((char *)wantedLine, (char *)inBuf);
		}
            }
        strcpy((char *)keywordInfo->keyword, keyword);
        }    /* end-if-else (strlen(keyword) == 0) */

    fclose(filePtr);

    if (!found)
        {
        keywordInfo->found = 0;
        errAdd(error, slxMODULE_NAME, slxERR_KEYWORD_NOT_FOUND, __FILE__, 
               NULL, keyword, setupFilename); 
        return FAILURE;       
        }

    keywordInfo->found = 1;
    if (slxSplitShortFITS(wantedLine, keywordInfo->category,
                          keywordInfo->subSystem, keywordInfo->parameter,
                          keywordInfo->value, keywordInfo->comment,
                          error) == FAILURE)
         return FAILURE;

    return SUCCESS;
}

/*
 *****************************************************************************
 */


/*___oOo___*/
