/******************************************************************************
* E.S.O. - VLT project
*
* slxGetNextField.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  02/03/95  Created
* epozna    17/08/01  VLTSW20010283 manpage update
*                                   example added
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   02/02/04  Problems with APR2004, static variable initialized  
*                     is initialized to ccsFALSE, VLTSW20040034
*/

/************************************************************************
*   NAME
*   slxGetNextField, slxGetNextFieldMulti - get the Next Field in a string 
*     buffer
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxGetNextFieldMulti(const char       *separators,
*				       const char       *bufPtr,
*			   	       char             **nextField,
*				       char             **newPos,
*				       ccsERROR         *error)
*
*   ccsCOMPL_STAT slxGetNextField(const char       *bufPtr,
*                                 const vltUINT16  maxLen, 
*  			          char             *nextField,
*                                 char             **newPos,
*  			          ccsERROR         *error)
*
*   ccsCOMPL_STAT slxGetNextField2(const char       separator,
*                                  const char       *bufPtr,
*                                  const vltUINT16  maxLen, 
*  			           char             *nextField,
*                                  char             **newPos,
*  			           ccsERROR         *error)
* 
*   DESCRIPTION
*   The functions return the Next Field of a text buffer which could have a
*   format like: 
*
*   "<string 1>  <string 2>\t\t<string 3>\n<string4>\0"
*
*   A string can contain all characters but separators, 
*   e.g.: ' ', '\t', '\n', '='.
*
*   The "newPos" points to the first character after the string in 
*   "nextField" (if any). Can be used to make the function search from
*   this position to get the Next Field in the text buffer.
*   E.g.:
*      "first_string second_string"
*       ^
*       |--- bufPtr
*   The 'nextField' will be: "first_string" and 'newPos' will point to
*   the first letter of "second_string". If you call the function again 
*   the 'nextField' will be "second_string".
*
*   Should the user manually modify the BufPtr to point in the middle of 
*   the string, like 
*      "first_string  second_string"
*         ^
*         |--- bufPtr
*    The "nextField" will be: "st_string\0".
*
*   If there is no Next Field, the "nextField" will be initiated 
*   with a '\0'.
* 
*   If a separator appear within a string it is not taken into account. 
*   E.g.: string = "aaa bbb \"ccc ddd\" eee"
*
*   "ccc ddd" is considered as Next Field even though it includes the SPACE 
*   separator.
*
*   Description of the various versions of the function:
*
*   slxGetNextFieldMulti():
*   Makes it possible for the user to specify and number of separators
*   like e.g.: " \t.", whereby separators are considers as SPACE,
*   TAB and period.
*
*   It allocates memory internally for storing the Next Field. I.e., the
*   memory should be copied before calling the function again. The
*   Next Field will always be NULL terminated.
*
*   slxGetNextField():
*   Separators are: ' ', '\t', '\n', '='.
*   The Next Field is copied into a buffer allocated by the user. 
*   The length of the buffer (i.e. argument maxLen) must be given to 
*   enable the  function to check for buffer overflow. 
*   When the length of the next element is greater then maxLen, the function 
*   returns error. In this case 'nextField' will not be updated at all.
*
*   slxGetNextField2():of the various versions of the function:
*
*   Separator: One separator can be specified by the user.
*   
*   The Next Field is copied into a buffer allocated by the user. The length
*   of the buffer must be given to enable the function to check for buffer
*   overflow.
*
*   RETURN VALUES 
*
*   CAUTIONS
*   Since slxGetNextFieldMulti() returns a pointer to internally allocated
*   memory, subsequent calls overwrite the previous 'Next Field'. I.e.,
*   make a local copy of the Next Field if this should be preserved.
*
*   EXAMPLES
*   Get the elements of the following string one by one.
*   Move the bufPtr into a middle of the string to see the effect. 
*
*   char * bufPtr="ABCDEFG bbb\tabcdefg\nddd";
*   char** newpos=&bufPtr;
*   char nextField[10]="";
*   int maxLen=10;
*   
*   if(slxGetNextField(bufPtr,maxLen,nextField,newpos,error)!=SUCCESS){}
*   printf("1 nextfield='%s'\n",nextField);
*
*   if(slxGetNextField(bufPtr,maxLen,nextField,newpos,error)!=SUCCESS){}
*   printf("2 nextfield='%s'\n",nextField);
*
*   bufPtr=bufPtr+3; // modify the buffer manually
*   if(slxGetNextField(bufPtr,maxLen,nextField,newpos,error)!=SUCCESS){}
*   printf("3 nextfield='%s'\n",nextField);
*
*   if(slxGetNextField(bufPtr,maxLen,nextField,newpos,error)!=SUCCESS){}
*   printf("4 nextfield='%s'\n",nextField);
*
*   results in:
*    1 nextfield='ABCDEFG'
*    2 nextfield='bbb'
*    3 nextfield='defg'
*    4 nextfield='ddd'
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxGetNextField.c,v 1.145 2005/07/27 09:15:56 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <errno.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxGetNextFieldMulti(const char       *separators,
				   const char       *bufPtr,
				   char             **nextField,
				   char             **newPos,
				   ccsERROR         *error)
{
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL  initialized=ccsFALSE;
    static slxDYN_BUF  nextFieldBuf;
    char               *chrPtr;
    vltUINT8           inAString;

    if (!initialized)
	{
	if (slxInitDynBuf(&nextFieldBuf, error) == FAILURE)
	    return FAILURE;
	if (slxDynBufAlloc(&nextFieldBuf, 512, error) == FAILURE)
	    return FAILURE;
	initialized = TRUE;
	}
    if (slxDynBufAddEOS(&nextFieldBuf, error) == FAILURE)
	return FAILURE;
    *nextField = slxDynBuf(&nextFieldBuf);
    if (slxSoftResetDynBuf(&nextFieldBuf, error) == FAILURE)
	return FAILURE;
    
    if (bufPtr == NULL)
	{
	if (slxDynBufAddEOS(&nextFieldBuf, error) == FAILURE)
	    return FAILURE;
	*nextField = slxDynBuf(&nextFieldBuf);
	return SUCCESS;
	}
    if (*bufPtr == '\0')
	{
	if (slxDynBufAddEOS(&nextFieldBuf, error) == FAILURE)
	    return FAILURE;
	*nextField = slxDynBuf(&nextFieldBuf);
	return SUCCESS;
	}

    /* Skip possible separators
     */
    chrPtr = (char *)bufPtr;
    while (strchr(separators, *chrPtr) != NULL)
	chrPtr++;

    /* Copy out characters until the next separator. A separator within
     * quotes, is not taken into account
     */
    inAString = 0;
    while (((strchr(separators, *chrPtr) == NULL) || 
	    ((strchr(separators, *chrPtr) != NULL) && inAString)) && 
	   (*chrPtr != '\0'))
	{
	if ((*chrPtr == '\"') && !inAString) /* " */
	    inAString = 1; 
	else if ((*chrPtr == '\"') && inAString) /* " */
	    inAString = 0;
	if (slxStoreBufDynBuf(&nextFieldBuf, chrPtr++, 1, 
			      error) == FAILURE)
	    {
	    *nextField = slxDynBuf(&nextFieldBuf);
	    return FAILURE;
	    }
	}

    if (*chrPtr != '\0')
	{
	if (strchr(separators, *chrPtr) != NULL)
	    chrPtr++;
	}
	
    if (slxSizeDynBuf(&nextFieldBuf) < slxAllocDynBuf(&nextFieldBuf))
        *(slxDynBuf(&nextFieldBuf) + slxSizeDynBuf(&nextFieldBuf)) = '\0';
    else
	{
	if (slxDynBufAddEOS(&nextFieldBuf, error) == FAILURE)
	    {
	    *nextField = slxDynBuf(&nextFieldBuf);
	    return FAILURE;
	    }
	}

    *nextField = slxDynBuf(&nextFieldBuf);
    *newPos = chrPtr;

    return SUCCESS;
}

ccsCOMPL_STAT slxGetNextField(const char       *bufPtr,
			      const vltUINT16  maxLen, 
			      char             *nextField,
			      char             **newPos,
			      ccsERROR         *error)
{
    char   *nextFieldPtr;


    if (slxGetNextFieldMulti(" \t\n=", bufPtr, &nextFieldPtr, newPos,
			     error) == FAILURE)
	return FAILURE;
    slxCOPY_MAX_N_BYTES(nextField, nextFieldPtr, maxLen, error);

    return SUCCESS;
    
    /*char         *chrPtr;
    vltUINT8     inAString;
    vltUINT16    index;
    vltUINT16    maxLenLoc;*/

    /* Skip possible separators
     */
    /*chrPtr = (char *)bufPtr;
    while ((*chrPtr == ' ') || (*chrPtr == '\t') || (*chrPtr == '\n'))
	chrPtr++;*/

    /* Copy out characters until the next separator
     */
    /*maxLenLoc = (maxLen - 1);
    index = 0;
    inAString = 0;
    while (((*chrPtr != ' ') || ((*chrPtr == ' ') && inAString)) && 
	   (*chrPtr != '\t') && (*chrPtr != '\n') && 
	   (*chrPtr != '\0') && (*chrPtr != '=') && (index < maxLenLoc))
	{
	if ((*chrPtr == '\"') && !inAString)
	    inAString = 1; 
	else if ((*chrPtr == '\"') && inAString)
	    inAString = 0;
	*(nextField + index) = *chrPtr++;
	index++;
	}

    if (index == maxLenLoc)
	{
	errAdd(error, slxMODULE_NAME, slxERR_BUF_OVERFLOW, __FILE__, NULL);
	return FAILURE;       
	}

    *(nextField + index) = '\0';
    *newPos = chrPtr;

    return SUCCESS;*/
}

ccsCOMPL_STAT slxGetNextField2(const char       separator,
			       const char       *bufPtr,
			       const vltUINT16  maxLen, 
			       char             *nextField,
			       char             **newPos,
			       ccsERROR         *error)
{
    char         *chrPtr;
    vltUINT8     inAString;
    vltUINT16    index;
    vltUINT16    maxLenLoc;


    chrPtr = (char *)bufPtr;
    
    /* Skip a possible separator
     */
    if (*chrPtr == separator)
	{
	*nextField = '\0';
	chrPtr++; 
	*newPos = chrPtr;
	return SUCCESS;
	}

    /* Copy out characters until the next separator
     */
    maxLenLoc = (maxLen - 1);
    index = 0;
    inAString = 0;
    while ((*chrPtr != separator) && (*chrPtr != '\0') && (index < maxLenLoc))
	{
	*(nextField + index) = *chrPtr++;
	index++;
	}

    if (index == maxLenLoc)
	{
	errAdd(error, slxMODULE_NAME, slxERR_BUF_OVERFLOW, __FILE__, NULL);
	return FAILURE;       
	}

    if (*chrPtr == separator)
	chrPtr++;

    *(nextField + index) = '\0';
    *newPos = chrPtr;

    return SUCCESS;
}

ccsCOMPL_STAT slxGetNextField3(const char       separator,
			       const char       *bufPtr,
			       char             **nextField,
			       char             **newPos,
			       ccsERROR         *error)
{
 /* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
   static vltLOGICAL  initialized=ccsFALSE;
    static slxDYN_BUF  nextFieldBuf;
    char               *chrPtr;
    vltUINT8           inAString;

    if (!initialized)
      {
      if (slxInitDynBuf(&nextFieldBuf, error) == FAILURE)
	return FAILURE;
      if (slxDynBufAlloc(&nextFieldBuf, 512, error) == FAILURE)
	return FAILURE;
      initialized = TRUE;
      }
    if (slxSoftResetDynBuf(&nextFieldBuf, error) == FAILURE)
	return FAILURE;

    if (bufPtr == NULL)
	{
	if (slxDynBufAddEOS(&nextFieldBuf, error) == FAILURE)
	    return FAILURE;
	*nextField = slxDynBuf(&nextFieldBuf);
	return SUCCESS;
	}

    /* Skip possible separators
     */
    chrPtr = (char *)bufPtr;
    while (*chrPtr == separator)
	chrPtr++;

    /* Copy out characters until the next separator
     */
    inAString = 0;
    while ((*chrPtr != separator) && (*chrPtr != '\0'))
	{
	if (slxStoreBufDynBuf(&nextFieldBuf, chrPtr++, 1, 
			      error) == FAILURE)
	  return FAILURE;
	}

    if (*chrPtr == separator)
	chrPtr++;

    if (slxSizeDynBuf(&nextFieldBuf) < slxAllocDynBuf(&nextFieldBuf))
        *(slxDynBuf(&nextFieldBuf) + slxSizeDynBuf(&nextFieldBuf)) = '\0';
    else
	{
	if (slxDynBufAddEOS(&nextFieldBuf, error) == FAILURE)
	    return FAILURE;
	}

    *nextField = slxDynBuf(&nextFieldBuf);
    *newPos = chrPtr;

    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/


