/******************************************************************************
* E.S.O. - VLT project
*
* slxLoadSetup.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  22/01/94  created
* gchiozzi  22/05/95  changed test on errno with feof()
* gchiozzi  06/07/95  initialized lineNo = 0
*/

/************************************************************************
*   NAME
*   slxLoadSetup - loads a setup into an SLX Dynamic Buffer
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   
*  ccsCOMPL_STAT slxLoadSetup(const slxFILENAME   fileName,
*	 		      slxDYN_BUF          *setupHandle,
*                             slxCHANGE_RECORD    changeRec,
*			      ccsERROR            *error)
*
* 
*   DESCRIPTION
*   The function loads a setup file into a SLX Dynamic Buffer. If setup
*   keywords are already loaded in the Dynamic Buffer, the function will merge
*   the new keywords with the old ones -- the new ones overwrites the old.
*   Before loading a new setup it may be necessary to reset the Dynamic Buffer
*   (slxResetDynBuf()) in order not to mix up old and new parameters, if this
*   is not desirable.
*
*   Only actual keywords are loaded. I.e. no lines starting with the "#"
*   (comment) will be loaded, and blank lines will be skipped.
*
*
*   fileName     IN/OUT   Name of the setup file to load (/merge) into the 
*                         buffer. The function will write the complete filename
*                         of the file loaded in the buffer at successfull 
*                         return, if not the complete filename was specified.
*
*   setupHandle  IN/OUT   SLX Dynamic Buffer to handle a setup in memory. The
*                         application should normally not themselves access
*                         the contents of the structure, but should use the
*                         access routines.
*
*   changeRec    OUT      Array indicating which keywords that changed
*                         while the function was operating on the Dynamic
*                         Buffer. Should be declared of type slxCHANGE_RECORD.
*                         The function does not reset the Change Record when
*                         it starts its operations. Note that the first
*                         keyword line has number _1_ (first field of the
*                         Change Record is not used).
*
*   error        OUT      CCS Error Structure indicating a possible error
*                         occurring.
*
*
*   FILES
*   Setup files must be in the Short-FITS format.
*
*
*   ENVIRONMENT
*   INS_ROOT        Defines the root directory of the INS directory structure
*                   used by this application.
*
*   INS_USER        Defines the current working point in the INS directory 
*                   structure. Could e.g. be "SYSTEM".
*
*   INS_SETUPPATH   Colon separated list of paths where Setup Files may be 
*                   located.
*
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successfull.
*   FAILURE   In case the operation failed. The following error codes can be
*             be generated by the function:
*
*             slxERR_FILE_FORMAT  Format of setup file wrong.
*
*             slxERR_OPEN_FILE    Cannot open setup file.
* 
*             slxERR_READ_FILE    Cannot read from setup file.
*
*             Furthermore errors generated by the functions
*             slxLocateSetupFile(), slxDetectFormat(), slxStoreLineBuf(),
*             can be returned.
*
*
*   CAUTIONS 
*   If a setup file for a new exposure shall be loaded into memory, the 
*   Dynamic Buffer should probably be reset using slxResetDynBuf().
*
*
*   SEE ALSO
*   slxLoadFile(), slxResetDynBuf(), slxMergeSetupBuf()
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxLoadSetup.c,v 1.145 2005/07/27 09:18:25 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <errno.h>
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxLoadSetup(slxFILENAME         fileName,
			   slxDYN_BUF          *setupHandle,
			   slxCHANGE_RECORD    changeRec,
			   ccsERROR            *error)
{
    char             completeName[slxFILENAME_LEN];
    char             keywordLine[slxKEYW_LINE_LEN];
    FILE             *filePtr;
    slxSETUP_FORMAT  type;
    vltINT16         len, len2;
    vltINT32         lineNo = 0;


    len = strlen("SUP.");
    len2 = strlen("SUP.PROT");

    if (slxLocateSetupFile(fileName, completeName, error) == FAILURE)
	return FAILURE;
    if (slxDetectFormat(completeName, &type, error) == FAILURE)
	return FAILURE;
    if (type != slxSHORT_FITS)
	{
	errAdd(error, slxMODULE_NAME, slxERR_FILE_FORMAT, __FILE__, "%s", 
	       completeName);
	return FAILURE;
	}

    if ((filePtr = fopen(completeName, "r")) == NULL)
	{
	errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, "%s",
		  modUNIX, completeName);
	return FAILURE;
	}
	
    /* Read in lines from the file and store them in the Dynamic Buffer
     */ 

    while (fgets(keywordLine, (int)slxKEYW_LINE_LEN, filePtr) != NULL)
	{
	/* All "SUP" category keyword minus "SUP.PROT;" are ignored
	 */
	if (strncmp(keywordLine, "SUP.", len) == 0)
	    {
	    if (strncmp(keywordLine, "SUP.PROT", len2) == 0)
		{
		if (slxStoreLineBuf(keywordLine, setupHandle, &lineNo,  
				    error) == FAILURE)
		    {
		    fclose(filePtr);
		    return FAILURE;
		    }			
		}
	    }
	else 
	    {
	    if (slxStoreLineBuf(keywordLine, setupHandle, &lineNo,
				error) == FAILURE)
		{
		fclose(filePtr);
		return FAILURE;
		}
	    if (lineNo < slxNO_OF_CH_REC_ELS && lineNo > 0 )
		changeRec[lineNo] = 1;
	    }
	}    /* end-while (fgets(keywordLine, (int)slxSYNTAX_LINE_READ, ... */
	   
    /* Check errno --> G.Chiozzi: replaced with feof()
     */ 
    if (!feof(filePtr) && (errno != slxNO_ERROR))
	{
	errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, __FILE__, "%s",
		  modUNIX, completeName);
	fclose(filePtr);
	return FAILURE;
	}

    fclose(filePtr);

    if (*fileName != '/')
	strcpy(fileName, completeName);
    
    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/



