/******************************************************************************
* E.S.O. - VLT project
*
* slxLookUpKeyword.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  10/10/94  Created 1. version
*/

/************************************************************************
*   NAME
*   slxLookUpKeyword - find information about a keyword in dictionary
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxLookUpKeyword(const slxDICTIONARY dicName,
*                                  const char          *keywordName,
*                                  const vltUINT16     recordNumber,
*                                  slxDIC_HANDLE       *dicInfo,
*                                  ccsERROR            *error)
*                                  
*
*   DESCRIPTION
*   The function searches in the specified VLT dictionary after the keyword
*   specified. First it checks if it can access the referenced dictionary
*   at all, if not it will return with a FAILURE.
*
*   If the record for the parameter is found, the values from the dictionary
*   are stored in the SLX Dictionary Handle Structure. The context of this
*   structure is:
*
*   Parameter Name:     Name of the parameter, e.g. "DET DATE"
*   Class:              Class of the parameter.
*   Context:            The context in which it is used.
*   Type:               The type of the parameter. Possible types are "string"
*                       "logical", "integer" and "double".
*   Value Format:       Format in which the parameter should be represented.
*                       The format representation from ANSI-C is used (see
*                       e.g. printf()).
*   Unit:               The SI or derived SI unit for the parameter.
*   Comment Format:     The standard comment (if any) for the parameter.
*   Description:        A (detailed) description of the parameter.
*
*
*   dicName       IN    The name of the dictionary to use. Only the extension
*                       of the dictionary name must be entered. E.g. for the 
*                       dictionary ESO-VLT-DIC.ISAAC only ISAAC should be 
*                       entered. Must be null terminated.
*
*   keywordName   IN    Name of the parameter to search for. The parameter
*                       must be given in in the Short-FITS format. Must
*                       be null terminated.
*
*   recordNumber  IN    Used to request a record according to its position in
*                       the dictionary. If this option is used the parameter
*                       "keywordName" must be set to "". The first record 
*                       starts at 1. If this parameter is not used it is 
*                       ignored. If a record number is specified larger 
*                       than the actual number of records in the dictionary
*                       the Dictionary Handle structure will be empty.
*
*   dicInfo     OUT     Structure that contains the information about the
*                       parameter in case it was found. The members in the
*                       structure are:
*                      
*   typedef struct {
*       vltUINT8 found;   
*       char     parName[slxDIC_BUF_LEN];                  
*       char     parClass[slxDIC_BUF_LEN];    
*       char     parContext[slxDIC_BUF_LEN];
*       char     parType[slxDIC_BUF_LEN];
*       char     valFormat[slxDIC_BUF_LEN];
*       char     parUnit[slxDIC_BUF_LEN];
*       char     comField[slxDIC_BUF_LEN];
*       char     **description;
*   } slxDicHandle;
*
*   The member "found" will be 1 if the parameter was found the other members
*   intialized successfully, otherwise it will be 0. All strings will be 
*   zero terminated ('\0') or initiated if nothing was stored in the buffer.
*   The buffer to contain the possible description will be allocated by the 
*   function and will be overwritten at the next call to the function.
*
*
*   error     OUT      CCS Error Structure. Indicates the type of a possible
*                      error occurring, e.g. the specified dictionary could
*                      not be found.
*
*
*   FILES
*   ESO-VLT-DIC.<name> Dictionaries for the VLT. There are a number of 
*                      dictionaries defined, e.g. for the instruments
*                      ISAAC, FORS and UVES.
*
*   ENVIRONMENT
*   INS_SETUPPATH      The environment varible contains a colon separated 
*                      list of paths where set-up files and set-up related 
*                      files (like dictionaries) can be located.
*                  
*
*   RETURN VALUES 
*   SUCCESS            If the operation was successful.
*   FAILURE            If the operation failed. The following error codes
*                      can be returned:
*
*                      slxERR_KEYWORD_NOT_FOUND The keyword (or keyword with
*                                               the number specified) was not
*                                               found.
*
*                      slxERR_SYN_LOCATE_DICO   Error in locating dictionary.
*
*                      slxERR_SYN_OPEN_DICO     Error in opening the 
*                                               dictionary.
*
*                      slxERR_SYN_FORMAT_DICO   Error in format of dictionary.
*
*                      slxERR_SYN_HEADER_DICO   Header of dictionary not OK.
*                                               Dictionary may be inconsistent.
*
*                      slxERR_SYN_READ_DICO     Error in reading the 
*                                               dictionary.
*
*                      slxERR_ALLOC_MEM         Error occurred while allocating
*                                               memory (for the description).
*
*                      The function makes a call to slxLocateSetupFile() and
*                      may return errors generated by this function.
*
*   CAUTIONS
*   The function allocates a buffer in which the possible description of the
*   parameter is stored. This description will be overwritten at the next
*   call to the function.
*
*   If the keyword searched for is not found, FAILURE is returned and
*   the error type slxERR_KEYWORD_NOT_FOUND is returned in the error 
*   structure.
*
*   SEE ALSO
*   slxResetDicRecord()
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxLookUpKeyword.c,v 1.145 2005/07/27 09:20:25 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxLookUpKeyword(const slxDICTIONARY  dicName,
                               const char           *keywordName,
                               const vltUINT16      recordNumber,
                               slxDIC_HANDLE        *dicInfo,
                               ccsERROR             *error)
{
    static vltUINT8   freeDescript = 0;
    static char       *descriptPtr;
    vltUINT16         record;
    vltLOGICAL        gotRecord;
    slxDYN_BUF        dicBuf;
    slxDIC_HANDLE     tmpDicInfo;
    slxFILENAME       dicFullName;
    slxKEYW_LINE      tmpBuf, tmpBuf2;

    memset((char *)dicInfo, 0, sizeof(slxDIC_HANDLE));
    slxInitDynBuf(&dicBuf, error);
    if (freeDescript)
        {
        freeDescript = 0;
        free(descriptPtr);
        }

    if (*keywordName != '\0')
        {
	if (slxLookUpKeywordHash(dicName, keywordName, &dicInfo,
				 error) == FAILURE)
	    return FAILURE;
	
	return SUCCESS;
	}

    /* Get a certain record number
     */
    if (slxSearchDictionary(dicName, dicFullName, error) == FAILURE)
	return FAILURE;
    if (slxLoadFile(dicFullName, "", &dicBuf, error) == FAILURE)
	 goto errorCond;
    if (slxGetDicRecSeq(TRUE, &gotRecord, &dicBuf, &tmpDicInfo,
			error) == FAILURE)
	goto errorCond;
    record = 0;
    do 
	{
	if (slxGetDicRecSeq(FALSE, &gotRecord, &dicBuf, dicInfo,
			    error) == FAILURE)
	    goto errorCond;
	record++;	
	if (gotRecord && (record < recordNumber))
	    {
	    if (slxResetDicRecord(dicInfo, error) == FAILURE)
		goto errorCond;
	    }
	}
    while (gotRecord && (record < recordNumber));

    if (record == recordNumber)
	{ 
	memset(dicInfo, 0, sizeof(slxDIC_HANDLE));
	sprintf(tmpBuf, "number \"%d\"", recordNumber);
	sprintf(tmpBuf2, "Dictionary \"%s\"", dicName);
	errAdd(error, slxMODULE_NAME, slxERR_KEYWORD_NOT_FOUND, __FILE__,NULL, 
	       tmpBuf, tmpBuf2);
	goto errorCond;
	}

    slxResetDynBuf(&dicBuf, error);

    return SUCCESS;

errorCond:
    slxResetDynBuf(&dicBuf, error);
    return FAILURE;
}

/*
 *****************************************************************************
 */

/*___oOo___*/
