/******************************************************************************
* E.S.O. - VLT project
*
* slxMergeSetupBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  22/01/95  created
* gchiozzi  22/05/95  modified test on errno with feof()
* gchiozzi  06/07/95  initialized lineNo = 0, aded checks on lineNo
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   02/02/04  Problems with APR2004, static variable initialized 
*                     is initialized to ccsFALSE, VLTSW20040034
*/

/************************************************************************
*   NAME
*   slxMergeSetupBuf - merges a number of setup files, keywords or buffers
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxMergeSetupBuf(slxDYN_BUF        *setupHandle,
*                                  const char        *instrument,
*                                  slxCHANGE_RECORD  changeRec,
*		                   ccsERROR          *error, 
*                                  ..., NULL)
* 
* 
*   DESCRIPTION
*   Merge a number of setup files and keywords into a SLX Dynamic Buffer.
*   The syntax of variable list of parameters is as follows:
*
*   "-k", "<Short-FITS keyword>"[, "<Short-FITS keyword>"]
*   "-f", "<filename>"[, "<filename>"]
*   "-b", <SLX Dynamic Buffer ptr>[, <SLX Dynamic Buffer ptr>]
*                                            ("-b" = Setup Buffer (slxDYN_BUF))
*
*   It is possible to have as many "-k", "-f" and "-b" indicators as desired.
*   The order is not important.
*
*   Note that the SLX Dynamic Buffer only will contain the actual keywords
*   entered (found in files). I.e. no setup file header will be generated.
*  
*   Blank lines and setup header keywords ("SUP." keywords) are discarded.
*
*   setupHandle IN/OUT  Buffer in which the setup parameters will be merged.
*
*   instrument  IN      Designates the dictionary to use. Could be e.g.
*                       "ISAAC" or "UVES". If this is initiated with '\0' no 
*                       Dictionary is used and setup keywords simply merged 
*                       into the buffer.
*
*   changeRec    OUT    Array indicating which keywords that changed
*                       while the function was operating on the Dynamic
*                       Buffer. Should be declared of type slxCHANGE_RECORD.
*                       The function does not reset the Change Record when
*                       it starts its operations. Note that the first
*                       keyword line has number _1_ (first field of the
*                       Change Record is not used).
*
*   error       OUT     CCS error structure indicating a possible error 
*                       occurring.
*
*   ...         IN      Variable list of parameters. Note, that strings must
*                       be given in _with_ quotes. References to SLX Dynamic
*                       Buffers, must be pointers.
*
*
*   FILES
*   Setup files must be terminated with ".ref", ".ins", ".det", or ".targ",
*   and must be in the Short-FITS format.
*
*
*   ENVIRONMENT
*   INS_ROOT        Defines the root directory of the INS directory structure
*                   used by this application.
*
*   INS_USER        Defines the current working point in the INS directory 
*                   structure. Could e.g. be "SYSTEM".
*
*   INS_SETUPPATH   Colon separated list of paths where Setup Files may be 
*                   located.
*
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successfull.
*   FAILURE   In case the operation failed. The following error codes can be
*             be generated by the function: 
*
*             slxERR_FILE_FORMAT 
*
*             slxERR_OPEN_FILE 
*
*             slxERR_READ_FILE 
*
*             slxERR_INPUT_PARS
*
*             The function makes calls to slxGetNextLineBuf(),
*             slxStoreLineBuf(), slxLocateSetupFile(), and slxDetectFormat(),
*             and may return errors generated by these functions.
*
*   CAUTIONS 
*   If not the complete path of a setup filename is given in, the first 
*   occurence of a setup file in the INS_SETUPPATH is used.
*
*   It might be usefull to reset the SLX Dynamical Buffer before performing
*   a new merge, since other 'old' setup parameters will be contained in the
*   buffer.
*
*
*   EXAMPLES
*
*   slxDYN_BUF        setupHandle1;
*   slxDYN_BUF        setupHandle2;
*   ccsERROR          error;
*   slxCHANGE_RECORD  changeRec;
*
*   slxInitDynBuf(&setupHandle1, &error);
*   slxInitDynBuf(&setupHandle2, &error);
*
*   if (slxLoadSetup("mode1.ref", &setupHandle2, &error) == FAILURE)
*      ... error processing ...
*
*   slxResetDynBuf(&setupHandle, &error);
*   memset(changeRec, 0, sizeof(slxCHANGE_RECORD));
*   if (slxMergeSetupBuf(&setupHandle1, 
*                        "ISAAC",
*                        changeRec,
*                        &error,
*                        "-b", &setupHandle2, 
*                        "-k", "DET.DIT   3.02E-2 # Integration Time",
*                        "-f", "detector2.det", 
*                        "-k", "DET.NDIT       10 # Averaged DITs",
*                        NULL) == FAILURE)
*      ... error processing ...
*
*
*   CAUTIONS
*   When entering keywords (using the "-k" specifier) with their value 
*   (and possible comment), this must be given in as one string as shown in 
*   the example above. If the value is a string the whole keyword string must
*   have the format "<keyword>   \"<string>\";  # <comment>".
*
*   The variable list of commands must be terminated with a NULL.
*
*
*   SEE ALSO
*   slxResetDynBuf()
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxMergeSetupBuf.c,v 1.145 2005/07/27 09:23:14 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdarg.h>
#include <string.h>
#include <errno.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxMergeSetupBuf(slxDYN_BUF        *setupHandle,
			       const char        *instrument,
			       slxCHANGE_RECORD  changeRec,
   	                       ccsERROR          *error, ...)
{
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL       initialized=ccsFALSE;
    static miscHASH_TABLE   dicHashTable;
    static slxDICTIONARY    curDic;
    char                    *argPtr;
    char                    *chrPtr;
    char                    *linePtr;
    FILE                    *filePtr;
    va_list                 ap;
    void                    *tempArgPtr;
    slxKEYW_LINE            keywordLine;
    slxFILENAME             completeName;
    slxKEYW_LINE            inBuf;
    slxDYN_BUF              *dynBufPtr;
    slxKEYW_LINE            keyword;
    slxSETUP_FORMAT         type; 
    slxDIC_HANDLE           *dicInfo;
    vltINT32                lineNo = 0;
    

    if (!initialized)
	{
	miscHashInit(dicHashTable);
	*curDic = '\0';
	initialized = TRUE;
	}

    if (strcmp(curDic, instrument) != 0)
	{
	miscHashDeleteTable(dicHashTable);
	if (slxLoadDicHash(dicHashTable, instrument, error) == FAILURE)
	    return FAILURE;
	slxCOPY_MAX_N_BYTES(curDic, instrument, sizeof(slxDICTIONARY),
			    error);
	}

    va_start(ap, error);

    argPtr = va_arg(ap, char *);
    while (argPtr != NULL)
	{	
	if (strcmp(argPtr, "-b") == 0)
	    {
	    /* The next strings may be either 1 or more SLX Dynamic Buffers:
	     * ..., "<Dyn. Buf. ptr.>"[", <Dyn. Buf. ptr.>"], ...
	     * or it may be a new format ID, i.e. "-b", "-k" or "-f".
	     */
	    tempArgPtr = va_arg(ap, void *);
	    while ((strcmp((char *)tempArgPtr, "-b") != 0) && 
		   (strcmp((char *)tempArgPtr, "-k") != 0) &&
		   (strcmp((char *)tempArgPtr, "-f") != 0) &&
		   (argPtr != NULL))
		{
		dynBufPtr = (slxDYN_BUF *)tempArgPtr;
		if (slxGetNextLineBuf2(dynBufPtr, TRUE, &linePtr,
				       error) == FAILURE)
		    return FAILURE;
		do
		    {
		    if (slxGetNextLineBuf2(dynBufPtr, FALSE, &linePtr,
					   error) == FAILURE)
			return FAILURE;
		    if (strlen(linePtr) > 0)
			{
			if ((strstr(linePtr, "SUP.") == NULL) &&
			    !slxSkipLine(inBuf))
			    {
			    if (*instrument != '\0')
				{
				if (slxGetNextField(linePtr,
						    slxKEYW_LINE_LEN,
						    keyword, &chrPtr, 
						    error) == FAILURE)
				    return FAILURE;
				/*if (slxGetNextField(keywordLine, 
						    slxKEYW_LINE_LEN,
						    keyword, &chrPtr, 
						    error) == FAILURE)
				    return FAILURE;*/
				if (slxLookUpKeywordHashX(dicHashTable,
							  keyword, &dicInfo,
							 error) == FAILURE)
				    return FAILURE;
				/*if (slxLookUpKeywordHash(instrument,keyword, 
							 &dicInfo,
							 error) == FAILURE)
				    return FAILURE;*/
				}
			    if (slxStoreLineBuf(linePtr, setupHandle, 
						&lineNo,error) == FAILURE)
				return FAILURE;
			    /*if (slxStoreLineBuf(keywordLine, setupHandle, 
						&lineNo,error) == FAILURE)
				return FAILURE;*/
			    if (lineNo < slxNO_OF_CH_REC_ELS && lineNo > 0)
				changeRec[lineNo] = 1;			    
			    }
			}    /* end-if (keywordLine[0] != '\0') */
		    }
		while (strlen(linePtr) > 0);
		/*while (keywordLine[0] != '\0');*/
		
		tempArgPtr = va_arg(ap, void *);
		}    /* end-while ((strcmp(argPtr, "-b") != 0) && ... */

	    argPtr = (char *)tempArgPtr;
	    }
	else if (strcmp(argPtr, "-k") == 0)
	    {
	    /* The next strings may be either 1 or more setup keywords:
	     * ..., "<keyword>", "<value>"[", #<comment>"], ["<keyword>", ...] 
	     * or it may be a new format ID, i.e. "-b", "-k" or "-f".
	     */
	    argPtr = va_arg(ap, char *);
	    while ((strcmp(argPtr, "-b") != 0) && 
		   (strcmp(argPtr, "-k") != 0) &&
		   (strcmp(argPtr, "-f") != 0) &&
		   (argPtr != NULL))
		{
		/* Get keyword, value and possible comment
		 * Note, this is supposed to be one string
		 */
		memset((char *)keywordLine, 0, (size_t)slxKEYW_LINE_LEN);
		strcpy((char *)keywordLine, argPtr); 

		if (keywordLine[strlen(keywordLine) - 1] != '\n')
		    keywordLine[strlen(keywordLine)] = '\n';

		if ((strstr((char *)keywordLine, "SUP.") == NULL) &&
		    !slxSkipLine(inBuf))
		    {
		    if (*instrument != '\0')
			{
			if (slxGetNextField(keywordLine, slxKEYW_LINE_LEN,
					    keyword, &chrPtr, 
					    error) == FAILURE)
			    return FAILURE;
			if (slxLookUpKeywordHashX(dicHashTable,
						  keyword, &dicInfo,
						  error) == FAILURE)
			    return FAILURE;
			/*if(slxLookUpKeywordHash(instrument,keyword, &dicInfo,
						 error) == FAILURE)
			    return FAILURE;*/
			}
		    if (slxStoreLineBuf(keywordLine, setupHandle, &lineNo,
					error) == FAILURE)
			return FAILURE;
		    if (lineNo < slxNO_OF_CH_REC_ELS && lineNo > 0)
			changeRec[lineNo] = 1;	
		    }
		argPtr = va_arg(ap, char *);
		}    /* end-while ((strcmp(argPtr, "-b") != 0) && ... */
	    }
	else if (strcmp(argPtr, "-f") == 0)
	    {
	    /* The next strings may be either 1 or more setup keywords:
	     * ..., "<filename>"[, "<filename>"], ...
	     * or it may be a new format ID, i.e. "-b", "-k" or "-f".
	     */

	    argPtr = va_arg(ap, char *);

	    while ((strcmp((char *)argPtr, "-b") != 0) && 
		   (strcmp((char *)argPtr, "-k") != 0) &&
		   (strcmp((char *)argPtr, "-f") != 0) &&
		   (argPtr != NULL))
		{
		/* Check that the setups to merge exist, and that the format is
		 * Short-FITS
		 */
		if (slxLocateSetupFile(argPtr, completeName, error) == FAILURE)
		    return FAILURE;
		if (slxDetectFormat(completeName, &type, error) == FAILURE)
		    return FAILURE;			  
		if (type != slxSHORT_FITS)
		    {	
		    errAdd(error, slxMODULE_NAME, slxERR_FILE_FORMAT,__FILE__,
			   NULL, completeName);
		    return FAILURE;
		    }

		if ((filePtr = fopen(completeName, "r")) == NULL)
		    {
		    errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, 
			      __FILE__, NULL, modUNIX, completeName);  
		    return FAILURE;
		    }

		/* Have to reset errno??
		 */
		errno = slxNO_ERROR;
		while (fgets(inBuf, slxKEYW_LINE_LEN, filePtr) != NULL)
		    {
		    if ((strstr(inBuf, "SUP.") == NULL) &&
			!slxSkipLine(inBuf))
			{
			if (*instrument != '\0')
			    {
			    if (slxGetNextField(inBuf, slxKEYW_LINE_LEN,
						keyword, &chrPtr, 
						error) == FAILURE)
				return FAILURE;
			    if (slxLookUpKeywordHashX(dicHashTable,
						      keyword, &dicInfo,
						      error) == FAILURE)
				return FAILURE;
			    /*if (slxLookUpKeywordHash(instrument, keyword, 
						     &dicInfo,
						     error) == FAILURE)
				return FAILURE;*/
			    }
			if (slxStoreLineBuf(inBuf, setupHandle, &lineNo,
					    error) == FAILURE)
			    {
			    fclose (filePtr);
			    return FAILURE;
			    }
			if (lineNo < slxNO_OF_CH_REC_ELS && lineNo > 0)
			    changeRec[lineNo] = 1;	
			}
		    }    /* end-while (fgets(inBuf, 128, filePtr) != NULL) */

		/* Check errno --> G.Chiozzi, added test on feof()
		 */ 
		if (!feof(filePtr) && (errno != slxNO_ERROR))
		    {
		    fclose (filePtr);
		    errSysAdd(error, slxMODULE_NAME, slxERR_READ_FILE, 
			      __FILE__, NULL, modUNIX, completeName);
		    return FAILURE;
		    }
		fclose (filePtr);

		argPtr = va_arg(ap, char *);
		}    /* end-while ((strcmp((char *)tempArgPtr, "-b") ... */
	    }
	else
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_INPUT_PARS, __FILE__, NULL);
	    return FAILURE;
	    }
	}    /* end-while ((argPtr = va_arg(ap, char *)) != NULL) */

    return SUCCESS;
}


/*
 *****************************************************************************
 */



/*___oOo___*/




