/******************************************************************************
* E.S.O. - VLT project
*
* slxSaveFile.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi  22/05/95  created
* jknudstr  26/05/95  changed/defined strategy for selecting directory
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   02/02/04  Problems with APR2004, static variable initialized
*                     is initialized to ccsFALSE, VLTSW20040034
*/

/************************************************************************
*   NAME
*   slxSaveFile - save contents of an SLX Dynamic Buffer into a file
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxSaveFile(const slxFILENAME   fileName,
*                             const char          *envVar,
*	  		      slxDYN_BUF          *fileBuf,
*			      ccsERROR            *error)
*
* 
*   DESCRIPTION
*   The function saves a file with an SLX Dynamic Buffer. The contents of the 
*   Dynamic Buffer is flushed directly into the file without any kind of 
*   reformatting. The contents of the Dynamic Buffer remains untouched.
*
*   If the filename is a complete path (must start with a "/") the function
*   tries to store the file according to the name as it is. Else the function 
*   checks if the file already exists in one of the directories given by the 
*   environment variable "envVar". If this is the case, the old file is 
*   overwritten with the new one (if possible). If the file didn't exists and 
*   the path was  not given in, the function tries to store the file in the 
*   first directory given by "envVar".
*
*
*   fileName     IN/OUT   Name of the setup file to save. If the name was not
*                         given in with the complete path, the name plus the
*                         path used will be returned in case of SUCCESS.
*
*   envVar       IN       Environment variable which is a colon separated list
*                         of paths where this type of file is stored.
*
*   fileBuf      IN/OUT   SLX Dynamic Buffer to handle a file in memory. The
*                         application should normally not themselves access
*                         the contents of the structure, but should use the
*                         access routines.
*
*   error        OUT      CCS Error Structure indicating a possible error
*                         occurring.
*
*
*   RETURN VALUES 
*   SUCCESS   In case the operation was successfull.
*   FAILURE   In case the operation failed. The following error codes can be
*             be generated by the function:
*
*             slxERR_OPEN_FILE    Cannot open setup file.
* 
*             slxERR_WRITE_FILE   Cannot read from setup file.
*
*             Furthermore errors generated by the functions
*             slxLocateSetupFile().
*
*  CAUTIONS
*  The function is not ment for generating Setup Files, i.e. it will not 
*  generate any Setup File Header.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxSaveFile.c,v 1.145 2005/07/27 09:25:27 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdio.h>
#include <errno.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxSaveFile(slxFILENAME   fileName,
			  const char    *envVar,			  
	 	          slxDYN_BUF    *fileBuf,
  		          ccsERROR      *error)
{
/* 
   MNY: Static variable is initialzed to ccsFALSE
   Updated due to problem with APR2004 release, VLTSW20040034
 */
    static vltLOGICAL  initialized=ccsFALSE;
    static ccsERROR    intError;
    char               *pathPtr;
    char               *chrPtr;
    FILE               *filePtr;
    slxFILENAME        fileNameLoc;
    slxFILENAME        completeName;

    if (!initialized)
	{
	errResetStack(&intError);
	initialized = TRUE;
	}

    if (slxResolvePath(fileName, &pathPtr, error) == FAILURE)
	return FAILURE;
    slxCOPY_MAX_N_BYTES(fileNameLoc, pathPtr, sizeof(slxFILENAME), error);

    /* Check if the filename is the complete path and if it can be opened
     */
    if (*fileNameLoc == '/')
	strcpy(completeName, fileNameLoc);
    else if (slxLocateFile(envVar, fileNameLoc, completeName, 
			   &intError) == SUCCESS)
	strcpy(completeName, fileNameLoc);
    else
	{
	errResetStack(&intError);

	/* Store the file in the first path of the "envVar" path - get the
	 * path name
	 */
	if ((pathPtr = getenv(envVar)) == NULL)
	    {
	    errSysAdd(error, slxMODULE_NAME, slxERR_ENV_VAR, __FILE__, 
		      "%s", modUNIX, envVar);
	    return FAILURE;
	    }
	if (*pathPtr == '\0')
	    {
	    errSysAdd(error, slxMODULE_NAME, slxERR_ENV_VAR, __FILE__, 
		      "%s", modUNIX, envVar);
	    return FAILURE;
	    }
	if ((chrPtr = strstr(pathPtr, ":")) != NULL)
	    {
	    strncpy(completeName, pathPtr, ((chrPtr - pathPtr) + 1));
	    *(completeName + ((chrPtr - pathPtr) + 1)) = '\0';
	    if (*(completeName + (strlen(completeName) - 1)) == ':')
		*(completeName + (strlen(completeName) - 1)) = '\0';
	    }			
	else
	    strcpy(completeName, pathPtr);
		
	if (*(completeName + strlen(completeName) - 1) != '/')
	    strcat(completeName, "/");
	strcat(completeName, fileNameLoc);	    
	}

    if ((filePtr = fopen(completeName, "w")) == NULL)
	{
	errSysAdd(error, slxMODULE_NAME, slxERR_OPEN_FILE, __FILE__, "%s",
		  modUNIX, completeName);
	return FAILURE;
	}
	
    /* Write the lines in the Dynamic Buffer to the file
     */ 
    if (fwrite(fileBuf->dynBuf, sizeof(char), (size_t)fileBuf->bytesStored, 
	      filePtr) != fileBuf->bytesStored)
	{
	fclose(filePtr);
	errSysAdd(error, slxMODULE_NAME, slxERR_WRITE_FILE, __FILE__, "%s",
		  modUNIX, completeName);
	return FAILURE;
	}

    strcpy(fileName, completeName);

    fclose(filePtr);
    
    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/


