/******************************************************************************
* E.S.O. - VLT project
*
* slxSplitShortFITS.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  30/09/94  Created 1. version
* - - - - - - - - - - APR2004 - - - - - - - - - - - - - - - - - - - 
* mnylund   07/11/03  VLTSW20030044
*/

/************************************************************************
*   NAME
*   slxSplitShortFITS, slxSplitShortFitsDyn, slxSplitShortFits2 - splits up 
*     a Short-FITS keyword line
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxSplitShortFitsDyn(const slxKEYW_LINE  shortFitsLine,
*			               const vltINT32      maxNoOfKeyEls, 
*				       slxKEYW_LINE        keyElements[],
*				       vltINT32            *noOfKeyElements,
*				       slxVALUE            value,
*				       slxCOMMENT          comment,
*				       ccsERROR            *error)
*
*   ccsCOMPL_STAT slxAssShortFitsKey(const slxKEYW_LINE   keyElements[],
*				     const vltINT32       noOfKeyElements,
*                                    slxKEYW_LINE         keyword,
*				     ccsERROR             *error)
*
*   ccsCOMPL_STAT slxSplitShortFits2(const slxKEYW_LINE  shortFitsLine, 
*                                    slxCATEGORY         category,
*                                    slxSUBSYSTEM        subSystem1,
*                                    slxSUBSYSTEM        subSystem2,
*                                    slxPARAMETER        parameter,
*                                    slxVALUE            value,
*                                    slxCOMMENT          comment,
*                                    ccsERROR            *error)
* 
* 
*   DESCRIPTION
*   The functions splits a Short FITS line in its components and return these 
*   in the output parameters.
*
*   All output buffers must be allocated by the calling application. They 
*   should be at least one byte longer than the maximum length of the string to
*   fill in, i.e. for the category, 4 bytes.
*
*   If an output buffer is not defined, the first element in this will be 
*   a null charcter ('\0'). E.g. if a standard FITS line is parted, the 
*   "category" and "subSystem" buffers will start with a '\0'.
* 
*   
*   shortfitsLine   IN  Pointer to buffer containing the Short FITS line. 
*
*   For slxSplitShortFitsDyn():
*   maxNoOfKeyEls   IN  Maximum number of key elements that can be stored in
*                       "keyElements" array.
*
*   keyElements     OUT String array in which the keyword components will be
*                       stored. This array could typically be allocated as
*                       follows:
*
*                       slxKEYW_LINE  keyArray[10];
*
*                       whereby "maxNoOfKeyEls = 10".
*
*   noOfKeyElements OUT Number of keyword components encountered.
*
*   For slxSplitShortFits2():
*   category        OUT  Pointer to buffer which will contain the possible
*                        category.
*
*   subSystem1/2    OUT  Pointer to buffer which will contain the possible 
*                        subsystem.
*
*   parameter       OUT  Pointer to buffer which will contain the parameter.
*
*
*
*   value           OUT  Pointer to buffer which will contain the value 
*                        (as a string). A string value is returned with quotes
*                        (also a logical value).
*
*   comment         OUT  Pointer to buffer which will contain the possible 
*                        commment.
*
*   error           OUT  CCS error structure indicating s possible error 
*                        occurring.
*
*
*   RETURN VALUES
*
*   CAUTIONS 
*   The function does not check for the syntax of the line, so if it is not 
*   correctly formatted, the result in the output buffers may be unpredictable.
*
*   SEE ALSO
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxSplitShortFITS.c,v 1.145 2005/07/27 09:31:58 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <string.h>

/* Local Headers
 */
#include "slx.h"

ccsCOMPL_STAT _slxSplitGetValueAndCom(const char          *afterKeywordPtr,
				      slxVALUE            value,
				      slxCOMMENT          comment,
				      ccsERROR            *error);

ccsCOMPL_STAT slxSplitShortFitsDyn(const slxKEYW_LINE  shortFitsLine,
				   const vltINT32      maxNoOfKeyEls,
				   slxKEYW_LINE        keyElements[],
				   vltINT32            *noOfKeyElements,
				   slxVALUE            value,
				   slxCOMMENT          comment,
				   ccsERROR            *error)
{
    /* SPR VLTSW20030044 Dangling pointer */
    char          *shFits, *bufPtr, *nextField, *afterKeywordPtr=NULL;
    slxKEYW_LINE  tempBuf;
    vltINT32      count = 0;
    vltINT32      index = 0;
 

    *comment = '\0';
    shFits = (char *)shortFitsLine;
    memset(keyElements, 0, (maxNoOfKeyEls * sizeof(slxKEYW_LINE)));

    /* Check if the keyword is a standard Short-FITS or a hierarchical 
     * Short-FITS one - Short-FITS will have the following format 
     * # <comment>
     * PAR   VAL;   [# <comment>], or
     * CAT[.SUB].PAR  VAL;   [# <comment>], or
     */
    if (*shFits == '#')
        {
        /* It's a Short-FITS comment
         */
        index++;
        while (*(shFits + index) == ' ')
            index++;
	slxCOPY_MAX_N_BYTES(comment, (shFits + index),slxKEYW_LINE_LEN, error);
	*noOfKeyElements = 0;
  
        return SUCCESS;
        }
    else if (slxIsShortFITS(shFits))
        {
        /* It's Hierarchical Short-FITS keyword line
         */

	/* Get keyword
	 */
	bufPtr = (char *)shortFitsLine;
	if (slxGetNextField(bufPtr, sizeof(slxKEYW_LINE), tempBuf, 
			    &bufPtr, error) == FAILURE)
	    return FAILURE;
	afterKeywordPtr = bufPtr;
	count = 0;
	bufPtr = (char *)tempBuf;
	do
	    {
	    if (slxGetNextFieldMulti(". \t", bufPtr, &nextField,
				     &bufPtr, error) == FAILURE)
		return FAILURE;
	    if (*nextField != '\0')
		{
		slxCOPY_MAX_N_BYTES(keyElements[count++], nextField,
				    sizeof(slxKEYW_LINE), error);
		}
	    }
	while ((*nextField != '\0') && (count < maxNoOfKeyEls));
	if (count > maxNoOfKeyEls)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_FORMAT_KEYWORD, 
		   __FILE__, NULL, shortFitsLine);
	    return FAILURE;
	    }
	*noOfKeyElements = count;
        }
    else
        {
        /* It's a Standard Short-FITS keyword line
         */
	bufPtr = (char *)shortFitsLine;
	if (slxGetNextField(bufPtr, sizeof(slxKEYW_LINE), tempBuf, 
			    &afterKeywordPtr, error) == FAILURE)
	    return FAILURE;
	strcpy(keyElements[0], tempBuf);
	*noOfKeyElements = 1;
        }

    /* Remove a possible semicolon + TABs after the parameter
     */
    bufPtr = keyElements[*noOfKeyElements - 1];
    if (slxGetLastChar(bufPtr) == ';') 
	*(bufPtr + strlen(bufPtr) - 1) = '\0';
    while (*(bufPtr + strlen(bufPtr) - 1) == '\t')
	*(bufPtr + strlen(bufPtr) - 1) = '\0';
    
    if (_slxSplitGetValueAndCom(afterKeywordPtr, value, comment, 
				error)==FAILURE)
	return FAILURE;

    return SUCCESS;
}


ccsCOMPL_STAT _slxSplitGetValueAndCom(const char          *afterKeywordPtr,
				      slxVALUE            value,
				      slxCOMMENT          comment,
				      ccsERROR            *error)
{
    char      *shFits;
    vltINT32  index, index2, len, run = FALSE;
    vltINT32  maxIndex;

    maxIndex = (slxKEYW_LINE_LEN - 56);
    *value = *comment = '\0';
    if (afterKeywordPtr == NULL)
	return SUCCESS;
    if (*afterKeywordPtr == '\0')
	return SUCCESS;    

    shFits = (char *)afterKeywordPtr;
    index = 0;

    /* Skip possible blanks in the Short-FITS line
     */
    while (((*(shFits + index) == ' ') ||
	   (*(shFits + index) == '\t')) && 
	   (*(shFits + index) != '\0') && 
	   (index < maxIndex))
        index++;
    if (index >= maxIndex)
        {
        errAdd(error, slxMODULE_NAME, slxERR_FORMAT_KEYWORD, __FILE__, NULL,
	       shFits);
        return FAILURE;
        }

    /* If by now we have already reached the end of the line, return
     */
    if (*(shFits + index) == '\0')
	return SUCCESS;

    /* Get the value (as a string) - strings keep the quotes for a string.
     */
    index2 = 0;    
    len = (maxIndex - 1);
    while ((*(shFits + index) != '\0') && 
	   (*(shFits + index) != ';') && 
	   (index < len))
        *(value + index2++) = *(shFits + index++);
    *(value + index2) = '\0';
    
    /* Yank possible trailing blanks
     */
    len = (strlen(value) - 1);
    while (*(value + len) == ' ')
	*(value + len) = '\0';
    index2 = strlen(value);
    
    /* Get a possible comment
     */
    if (*(shFits + index) != '#')
	run = TRUE;
    while (run && (*(shFits + index) != '\0') && (index < maxIndex))
	{
	if (*(shFits + index) == '#') 
	    run = FALSE;  
	else
	    index++;
	}
    
    if (index < maxIndex)
	{
	if (*(shFits + index) == '#')
	    slxCOPY_MAX_N_BYTES(comment, (shFits + index + 2), 
				maxIndex, error);
	}

    return SUCCESS;
}


/*************************************************************************
 *  Kept for backwards compatiability
 *************************************************************************/
ccsCOMPL_STAT slxSplitShortFits2(const slxKEYW_LINE  shortFitsLine, 
				 slxCATEGORY         category,
				 slxSUBSYSTEM        subSystem1,
				 slxSUBSYSTEM        subSystem2,
				 slxPARAMETER        parameter,
				 slxVALUE            value,
				 slxCOMMENT          comment,
				 ccsERROR            *error)
{
    static slxKEYW_LINE   keyElements[4];
    vltINT32              noOfKeyElements;
 

    if (slxSplitShortFitsDyn(shortFitsLine, 4, keyElements, &noOfKeyElements,
			     value, comment, error) == FAILURE)
	return FAILURE;

    switch (noOfKeyElements)
	{
	case 1:
	    *category = *subSystem1 = *subSystem2 = '\0';
	    slxCOPY_MAX_N_BYTES(parameter, keyElements[0], 
				(sizeof(slxCATEGORY) - 2), error);
	    break;
	case 2:
	    *subSystem1 = *subSystem2 = '\0';
	    slxCOPY_MAX_N_BYTES(category, keyElements[0], 
				(sizeof(slxCATEGORY) - 2), error);
	    slxCOPY_MAX_N_BYTES(parameter, keyElements[1],
				(sizeof(slxPARAMETER) - 2), error);
	    break;
	case 3:
	    *subSystem2 = '\0';
	    slxCOPY_MAX_N_BYTES(category, keyElements[0], 
				(sizeof(slxCATEGORY) - 2), error);
	    slxCOPY_MAX_N_BYTES(subSystem1, keyElements[1],
				(sizeof(slxSUBSYSTEM) - 2), error);
	    slxCOPY_MAX_N_BYTES(parameter, keyElements[2],
				(sizeof(slxPARAMETER) - 1), error);
	    break;
	default:
	    slxCOPY_MAX_N_BYTES(category, keyElements[0], 
				(sizeof(slxCATEGORY) - 2), error);
	    slxCOPY_MAX_N_BYTES(subSystem1, keyElements[1], 
				(sizeof(slxSUBSYSTEM) - 2), error);
	    slxCOPY_MAX_N_BYTES(subSystem2, keyElements[2], 
				(sizeof(slxSUBSYSTEM) - 2), error);
	    slxCOPY_MAX_N_BYTES(parameter, keyElements[3], 
				(sizeof(slxPARAMETER) - 2), error);
	    break;
	}
    
    return SUCCESS;
}

ccsCOMPL_STAT slxSplitShortFITS(const slxKEYW_LINE  shortFitsLine, 
				slxCATEGORY         category,
				slxSUBSYSTEM        subSystem,
				slxPARAMETER        parameter,
				slxVALUE            value,
				slxCOMMENT          comment,
				ccsERROR            *error)
{
    slxSUBSYSTEM        subSystem2;

    if (slxSplitShortFits2(shortFitsLine, category, subSystem, subSystem2,
			   parameter, value, comment, error) == FAILURE)
	return FAILURE;

    return SUCCESS;
}

ccsCOMPL_STAT slxAssShortFitsKey(const slxKEYW_LINE   keyElements[],
				 const vltINT32       noOfKeyElements,
				 slxKEYW_LINE         keyword,
				 ccsERROR             *error)
{
    vltINT32 i;

    *keyword = '\0';
    for (i = 0; i < noOfKeyElements; i++)
	{
	strcat(keyword, keyElements[i]);
	strcat(keyword, ".");
	}
    *(keyword + (strlen(keyword) - 1)) = '\0';

    return SUCCESS;
}

/*
 *****************************************************************************
 */


/*___oOo___*/





