/******************************************************************************
* E.S.O. - VLT project
*
* slxStoreLineBuf.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* gchiozzi  20/07/95  Corrected bug handling lineCount when adding new lines
* jknudstr  22/01/94  Created
* jknudstr  30/05/95  Fixed bug when a keyword was a subset of another
*                     -> it overwrote the other if placed before
*/

/************************************************************************
*   NAME
*   slxStoreLineBuf - stores a set-up line in a SLX Dynamic Buffer
* 
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxStoreLineBuf(const slxKEYW_LINE  keywordLine,
*       		          slxDYN_BUF          *setupHandle,
*                                 vltINT32            *lineNo,
*			          ccsERROR            *error)
*
*
*   DESCRIPTION
*   The function searches in the specified Dynamic Buffer for a set-up keyword
*   identical to the one in the buffer pointed to by "keywordLine". If it 
*   finds this line, it replaces the old with the new. Note that this includes
*   overwriting the old comment which might be there with the new one. If not
*   found, the new keyword is appended in the Dynamic Buffer.
*
*   The keywords must be in the Short-FITS format.
*
* 
*   keywordLine	 IN	The keyword command line which shall be stored in the
*			set-up file. Must be in the Short-FITS format, and
*                       NULL terminated.
* 
*   setupHandle  IN     Pointer to a SLX Dynamic Buffer.
*
*   lineNo       OUT    Indicates in which line the new line was stored. First
*                       line has number 1.
*
*   error        OUT    CCS Error Structure. Indicates a possible error 
*                       occurring.
*    
*
*   RETURN VALUES 
*   SUCCESS	If the operation was successful.
*   FAILURE	If something went wrong. The following error codes can be
*               returned:
*
*               slxERR_FORMAT_KEYWORD The format of the keyword line entered
*                                     is wrong.
*
*               slxERR_ALLOC_MEM      Could not allocate memory.
*
*               Furthermore calls are made to slxGetNextLineBuf(), and errors
*               generated by this function may be returned.
*
*   CAUTIONS
*   The subroutine assumes that the set-up buffer is syntactically correct.
*
*------------------------------------------------------------------------
*/
#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxStoreLineBuf.c,v 1.145 2005/07/27 09:33:20 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

/* Local Headers
 */
#include "slx.h"

/* Function definition
 */
ccsCOMPL_STAT slxStoreLineBuf(const slxKEYW_LINE  keywordLine,
			      slxDYN_BUF          *setupHandle,
			      vltINT32            *lineNo,
			      ccsERROR            *error)
{
    char         refFitsLine[slxKEYW_LINE_LEN];
    char         *tempPtr;
    char         keywordLocal[slxKEYW_LINE_LEN]; 
    char         *bufPtr;
    size_t       protMarkLen;
    slxKEYW_LINE keyword;
    vltUINT8     lineFound;
    vltUINT16    lineCount;
    vltUINT16    refLength;
    vltUINT16    n;
    vltUINT16    length;
    vltUINT16    index;
    vltUINT32    newAllocSize;
 
    /* Initialize always to 0 first
     */
    *lineNo = 0;

    protMarkLen = strlen(slxPROT_REG_MARKER);  

    /* We disregard comments, blank lines, etc.
     */
    if ((*keywordLine == '#') || (*keywordLine == ' ') ||
	(*keywordLine == '\t') || (*keywordLine == '\n'))
	return SUCCESS;
    
    /* If the keyword line has not got a '\n' in the end we have to append it.
     */
    strcpy((char *)keywordLocal, (char *)keywordLine);
    length = strlen(keywordLine);
    if (keywordLine[length - 1] != '\n')
	{
	keywordLocal[length] = '\n';
	keywordLocal[length + 1] = '\0';
	}

    /* Get the keyword part of the new FITS line ("<cat>.<sub>.<par>")
     */
    bufPtr = (char *)keywordLine;
    if (slxGetNextField(bufPtr, slxKEYW_LINE_LEN, refFitsLine, &bufPtr,
			error) == FAILURE)
	return FAILURE;    
    strcat(refFitsLine, " ");
    refLength = strlen(refFitsLine);

    /* Search after FITS line in the Dynamic Buffer.
     *
     * There is searched only according to the following keywords:
     * "<cat>.[<sub>.]<par>".
     */
    if (slxGetNextLineBuf(setupHandle, slxINIT_DYN_BUF, keyword, 
			  error) == FAILURE)
	return FAILURE;
    lineFound = 0;
    lineCount = 0;
    do
        {
	if (slxGetNextLineBuf(setupHandle, slxDO_NOT_INIT, keyword,
			      error) == FAILURE)
	    return FAILURE;

	lineCount++;

	/* For now, ignore the SUP.PROT markers (should be changed)
         */
	if (strncmp(keyword, slxPROT_REG_MARKER, protMarkLen) != 0)
	    {
	    if (strncmp(keyword, refFitsLine, (size_t)refLength) == 0)
		lineFound = 1;
	    }
        }
    while ((keyword[0] != '\0') && (!lineFound));

    lineCount--; /* GCH 20/07/95 I have always to go back one step */

    /* If the line was found:
     * 
     * 1. Create a new buffer + extra size.
     * 2. Copy over the lines until the line to be replaced.
     * 3. Write the new line.
     * 4. Copy the rest of the lines.
     * 5. Free the original "dynBuf" pointer, and make it point to
     *    the new buffer.
     *
     * If the line wasn't found, just append it in the original set-up file.
     */
    if (lineFound)
        {
	newAllocSize = setupHandle->bytesStored + strlen(keywordLocal) + 1;
	if ((tempPtr = (char *)malloc(newAllocSize)) == NULL)
	    {
	    errSysAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, 
		      NULL, modUNIX); 
	    return FAILURE;
	    }

	index = 0;
	if (slxGetNextLineBuf(setupHandle, slxINIT_DYN_BUF, keyword,
			      error) == FAILURE)
	    return FAILURE;
        for (n = 0; n < lineCount; n++)
            { 
	    if (slxGetNextLineBuf(setupHandle, slxDO_NOT_INIT, keyword,
				  error) == FAILURE)
		return FAILURE;
               
	    length = strlen(keyword);
	    memcpy((char *)(tempPtr + index), (char *)keyword, (size_t)length);
	    index += length;
            }     /* for n < lineCount */

	length = strlen(keywordLocal);
	memcpy((char *)(tempPtr + index), (char *)keywordLocal, 
	       (size_t)length);
	index += length;

	/* Jump a line in the original buffer
	 */ 
	if (slxGetNextLineBuf(setupHandle, slxDO_NOT_INIT, keyword,
			      error) == FAILURE)
	    return FAILURE;

	do
	    {
	    if (slxGetNextLineBuf(setupHandle, slxDO_NOT_INIT, keyword,
				  error) == FAILURE)
		return FAILURE;

	    if (keyword[0] != '\0')
		{
		length = strlen(keyword);
		memcpy((char *)(tempPtr + index), (char *)keyword, 
		       (size_t)length);
		index += length;
		}
	    }
	while (keyword[0] != '\0');
	
	free(setupHandle->dynBuf);
	setupHandle->dynBuf = tempPtr;
	setupHandle->bytesAlloc = newAllocSize;
	setupHandle->bytesStored = index;
	/* IMPL.: DYN_BUF:
	 */
	/*setupHandle->nextPos = setupHandle->prevPos = setupHandle->dynBuf;*/
	setupHandle->count = 0;
	*(setupHandle->dynBuf + setupHandle->bytesStored) = '\0';
        }    /* end-if line found */
    else
        {
        /* Append the new line in the buf - first allocate more memory
         */
        if ((tempPtr = realloc(setupHandle->dynBuf, 
			       (setupHandle->bytesAlloc + 
				strlen(keywordLocal)))) == NULL)
            {
            errSysAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, 
                      NULL, modUNIX);
            return FAILURE;
            }  
	setupHandle->dynBuf = tempPtr;

	length = strlen(keywordLocal);
	memcpy((char *)(setupHandle->dynBuf + setupHandle->bytesStored),
	       (char *)keywordLocal, (size_t)length);
	setupHandle->bytesStored += length;
	setupHandle->bytesAlloc += strlen(keywordLocal);

	if (keywordLocal[length - 1] != '\n')
	    {
	    if ((tempPtr = realloc(setupHandle->dynBuf, 
				   (setupHandle->bytesAlloc + 
				    strlen(keywordLocal) + 1))) == NULL)
		{
		errSysAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, 
			  NULL, modUNIX);
		return FAILURE;
		}
	    setupHandle->dynBuf = tempPtr;
	    setupHandle->bytesAlloc++;
	    *(setupHandle->dynBuf + setupHandle->bytesStored) = '\n';
	    setupHandle->bytesStored++;
	    *(setupHandle->dynBuf + setupHandle->bytesStored) = '\0';
	    }
        }    /* else-append line */

    /* Line count starts from 0, but lineNo start from one */
    *lineNo = (lineCount + 1);
    
    return SUCCESS;
}


/*
 *****************************************************************************
 */


/*___oOo___*/







