/******************************************************************************
* E.S.O. - VLT project
*
* slxStringList.c
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  04/10/96  created
*/

/************************************************************************
*   NAME
*   slxStringList, slxInitStrList, slxFreeStrList, slxDelStrEl, slxAddStrEl, 
*   slxStrListMerge, slxStrNextEl - functions to handle the SLX String List 
*   feature
*
*   SYNOPSIS
*   #include "slx.h"
*
*   ccsCOMPL_STAT slxInitStrList(slxSTRING_LIST  **strElPtr,
*			         const char      *hdrString,
*			         ccsERROR        *error)
*
*   ccsCOMPL_STAT slxFreeStrList(slxSTRING_LIST  *strElPtr,
*			         ccsERROR        *error)
*
*   char *slxStrNextEl(const slxSTRING_LIST  *strListHdr,
*		       const vltLOGICAL      init)
*
*   ccsCOMPL_STAT slxAddStrEl(slxSTRING_LIST    *strElPtr, 
*			      const char        *string,
*			      const vltLOGICAL  addFirstInList,
*			      const vltINT32    id,
*			      ccsERROR          *error)
*
*   ccsCOMPL_STAT slxDelStrEl(slxSTRING_LIST  *strElPtr, 
*			      ccsERROR        *error)
*
*   ccsCOMPL_STAT slxStrListMerge(char            *string,
*  	 	    	          slxSTRING_LIST  *targList,
*			          vltINT32        *elsStored,
*			          ccsERROR        *error)
*
*   ccsCOMPL_STAT slxStrListGetEl(slxSTRING_LIST  *stringList,
*			          vltINT32        elNumber,
*                                 char            **string,
*			          ccsERROR        *error)
*
*   DESCRIPTION
*   The SLX String List feature is used to maintain a list of strings.
*   Before starting to use a string list it should be initialized using the
*   slxInitStrList() function.
*
*   The functions slxAddStrEl(), respectively slxDelStrEl(), can be used to
*   add and delete string element in the list.
*
*   slxAddStrEl() 
*   The "addFirstInList" can be used to specify if the new element should be
*   added first in the list. The "id" can be used to obtain a certain 
*   sequence of the elements. The lower the ID the more in front of the list
*   the element will be inserted. The Header Element has ID 0.
*
*   slxStrListMerge()
*   Inserts a string on the "*elsStored" place in the list. If an element
*   exists it is overwritten, otherwise a new element is inserted.
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES 
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS   
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"

static char *rcsId="@(#) $Id: slxStringList.c,v 1.145 2005/07/27 09:33:56 vltsccm Exp $";
static void *use_rcsId = ((void)&use_rcsId,(void *) &rcsId);

/* System Headers
 */
#include <stdlib.h>

/* Local Headers
 */
#include "slx.h"

/* Creates a header element for the String List
 */
ccsCOMPL_STAT slxInitStrList(slxSTRING_LIST  **strElPtr,
			     char            *hdrString,
			     ccsERROR        *error)
{
    if ((*strElPtr = (slxSTRING_LIST *)malloc(sizeof(slxSTRING_LIST))) == NULL)
	{
	errAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, NULL);
	return FAILURE;
	}   
    (*strElPtr)->next = *strElPtr;
    (*strElPtr)->prev = *strElPtr;
    (*strElPtr)->id = 0;
    (*strElPtr)->header = TRUE;
    (*strElPtr)->elPtr = *strElPtr;
    
    if (((*strElPtr)->string = slxStrDup(hdrString)) == NULL)
	{
	free(*strElPtr);
	errAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, NULL);
	return FAILURE;
	}

    return SUCCESS;
}

ccsCOMPL_STAT slxFreeStrList(slxSTRING_LIST  *strElPtr,
			     ccsERROR        *error)
{
    slxSTRING_LIST  *elPtr, *delPtr;

    elPtr = strElPtr->next;
    while (elPtr != strElPtr)
	{
	delPtr = elPtr;
	elPtr = elPtr->next;
	free(delPtr->string);
	free(delPtr);	
	}
    free(elPtr->string); 
    free(elPtr);

    return SUCCESS;
}


ccsCOMPL_STAT slxDelStrEl(slxSTRING_LIST  *strElPtr, 
			  ccsERROR        *error)
{
    
    strElPtr->prev->next = strElPtr->next;
    strElPtr->next->prev = strElPtr->prev;

    if (strElPtr->string != NULL)
	free(strElPtr->string);
    free(strElPtr);

    strElPtr = NULL;

    return SUCCESS;
}

ccsCOMPL_STAT slxAddStrEl(slxSTRING_LIST  *strElPtr, 
			  char            *string,
			  vltLOGICAL      addFirstInList,
			  vltINT32        id,
			  ccsERROR        *error)
{
    slxSTRING_LIST  *newEl;
    slxSTRING_LIST  *elPtr;

    
    if ((newEl = (slxSTRING_LIST *)malloc(sizeof(slxSTRING_LIST))) == NULL)
	{
	errAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, NULL);
	return FAILURE;
	}

    if (addFirstInList)
	{
	/* Search in the list until the header is found or the ID of the
	 * element checked is larger than the ID of the new element.
	 */
	elPtr = strElPtr->next;
	while ((elPtr->id <= id) && (elPtr != strElPtr))
	    elPtr = elPtr->next;
	/* Step one back
	 */
	elPtr = elPtr->prev;
	}
    else
	{
	/* Search towards the left until header is found or an ID is 
	 * encountered which is equal or smaller than the new ID
	 */
	elPtr = strElPtr->prev;
	while ((elPtr->id > id) && (elPtr != strElPtr))
	    elPtr = elPtr->prev;
	}

    /* Insert the element
     */
    newEl->next = elPtr->next;
    newEl->prev = elPtr;
    elPtr->next = newEl;
    newEl->next->prev = newEl;
    newEl->id = id;
    newEl->header = FALSE;
    newEl->elPtr = NULL;

    if ((newEl->string = slxStrDup(string)) == NULL)
	{
	slxDelStrEl(newEl, error);
	errAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, NULL);
	return FAILURE;
	}

    return SUCCESS;
}	


ccsCOMPL_STAT slxStrListMerge(char            *string,
			      slxSTRING_LIST  *targList,
			      vltINT32        *elsStored,
			      ccsERROR        *error)
{
    slxSTRING_LIST   *listPtr;
    vltINT32         i, count;
    

    listPtr = targList->next;
    count = *elsStored;
    for (i = 0; i < count; i++)
	listPtr = listPtr->next;

    if (listPtr != targList)
	{ 
	free(listPtr->string);
	if ((listPtr->string = slxStrDup(string)) == NULL)
	    {
	    errAdd(error, slxMODULE_NAME, slxERR_ALLOC_MEM, __FILE__, NULL);
	    return FAILURE;
	    }
	}
    else
	{
	if (slxAddStrEl(targList, string, FALSE, 0, error) == FAILURE)
	    return FAILURE;
	}	

    *elsStored = count + 1;

    return SUCCESS;
}

char *slxStrNextEl(slxSTRING_LIST        *strListHdr,
		   const vltLOGICAL      init)
{
    if (init)
	strListHdr->elPtr = strListHdr;

    strListHdr->elPtr = strListHdr->elPtr->next;

    if (strListHdr->elPtr == strListHdr)
	return NULL;
    else 
	return strListHdr->elPtr->string;
}

ccsCOMPL_STAT slxStrListGetEl(slxSTRING_LIST  *stringList,
			      vltINT32        elNumber,
			      char            **string,
			      ccsERROR        *error)
{
    vltINT32        i;
    slxSTRING_LIST  *listPtr;

    listPtr = stringList;
    for (i = 0; i < elNumber; i++)
	listPtr = listPtr->next;
    
    *string = listPtr->string;

    return SUCCESS;
}

/*___oOo___*/
