
###!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxCheckCompl.test.tcl,v 1.145 2005/07/27 12:24:50 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxCheckCompl.test.prog - test script for slxCheckCompleteness
# 
#   SYNOPSIS
#   slxCheckCompl.test.prog
# 
#   DESCRIPTION
#   The script test the program slxCheckCompleteness that checks if a 
#   Reference Set-Up file contains all the keywords it should.
#
#   The script carries out the following tests:
#
#   1. Test of handling of failure in locating dictionary.
#      Error constant: slxERR_SYN_LOCATE_DICO.
#
#   2. Test of handling of failure in opening dictionary.
#      Test files used:  ESO-VLT-DIC.FORS, test.ref
#      Error constant: slxERR_SYN_OPEN_DICO.
#
#   3. Test of an error in the header of a dictionary.
#      Test files used: ESO-VLT-DIC.EMMI, test.ref
#      Error constant: slxERR_SYN_HEADER_DICO.
#
#   4. Test of correct functionining of the function. Test file is used that
#      provoke all detectable syntax errors.
#      Test files used: ESO-VLT-DIC.ISAAC, 
#      slxCheckCompl.test.4.bs.rf
#
#   5. Test of input parameters (correct number).
#
#   6. Test of handling of case when illegal instrument is specified.
#
#   7. Test of error in locating set-up file.
#
#   8. Test of handling of set-up file in wrong format (Normal-FITS intead of
#      Short-FITS).
#
#   9. Test of handling of set-up file in wrong format (unknown format).
#
#   10. Test of access to read protected set-up file.
#
#
#   The script writes an error report to standard output. 
#
#
#   FILES
#   ESO-VLT-DIC.ISAAC                      Correct dictionary containing the 
#                                          necessary definitions.
#
#   ESO-VLT-DIC.FORS                       rwx protected dictionary - cannot 
#                                          be opened.
#
#   ESO-VLT-DIC.EMMI                       Dictionary with an error in the 
#                                          dictionary header.
#   
#   test.ref                               Just a dummy set-up file to give an
#                                          input to the function.
#
#   slxCheckCompl.test.4.bs.rf  Input set-up file containing 
#                                          keywords that will provoke all 
#                                          detectable syntax errors.
#
#   slxCheckCompl.test.8.bs.rf  File in Normal-FITS format.
#
#   slxCheckCompl.test.9.bs.rf  File in unknown format.
#
#   slxCheckCompl.test.10.bs.rf Read protected file (is read 
#                                          protected during the test).
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs
set tf slxCheckCompl.test
set testPath [pwd]
set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM
#set slxCheckCompleteness ""
set slxCheckCompl ""

# PART 1: Failure in locating the dictionary
# ==========================================
set pwdTmp [pwd]
cd ../test
set complPath [pwd]/INS_ROOT/SYSTEM/Dictionary
cd $pwdTmp
test slxCheckCompl-1 {Failure in locating dictionary} {
    if {[catch "exec -keepnewline slxCheckCompl test.ref UVES" msg]} { 
	set testString "Could not open dictionary \"$complPath/ESO-VLT-DIC.UVES\""
	if {[string first $testString $msg] != -1} {
	    set msg ""	
	} else {
	    set msg "FAILURE"
	}	    
    } else {
	set msg "FAILURE"
    } 
    set msg
}    ""

exit
# PART 2: Failure in opening dictionary
# =====================================
exec chmod -rwx $env(INS_ROOT)/SYSTEM/Dictionary/ESO-VLT-DIC.FORS
test slxCheckCompleteness-2 {Attempt to open rwx protected dictionary} {
    if {[catch "exec -keepnewline slxCheckCompl.test.ref FORS" msg]} {
        set testString "Could not open dictionary \"$complPath/ESO-VLT-DIC.FORS\""
	if {[string first $testString $msg] != -1} {
	    set msg ""	
	} else {
	    set msg "FAILURE"
	}	
    } else {
	set msg "FAILURE"
    }
    set msg
} ""
catch {exec chmod +rw $env(INS_ROOT)/SYSTEM/Dictionary/ESO-VLT-DIC.FORS}


# PART 4: Test of basic functioning
# =================================
test slxCheckCompleteness-4 {Basic functionality test} {
    exec slxCheckCompleteness ../test/$tf.4.bs.rf ISAAC -t > $tf.4.temp
    set msg [exec diff $tf.4.temp $tf.4.rf]
    set msg
} ""
catch "exec rm $tf.4.temp"



# PART 5: Test of check of number of input parameters
# ======================================================
test slxCheckCompleteness-5 {Test of check of input parameters} {
    if {[catch "exec -keepnewline slxCheckCompleteness " msg]} {
        set msg
    }
}    "\n###########################################################################
  The SLX Completeness Checker for set-up files in the Short-FITS format
                        Version 1.0/September 1994

                              ESO VLT Project

---------------------------------------------------------------------------
Correct format is:

slxCheckCompleteness <set-up filename> <instrument>

---------------------------------------------------------------------------\n\nchild process exited abnormally"


# PART 6: Test of handling of specification of illegal instrument
# ===============================================================
test slxCheckCompleteness-6 {Test of specification of illegal instrument} {
    if {[catch "exec -keepnewline slxCheckCompl.test.ref unknown" msg]} {
	set testString "Could not open dictionary \"$complPath/ESO-VLT-DIC.unknown\""
	if {[string first $testString $msg] != -1} {
	    set msg ""	
	} else {
	    set msg "FAILURE"
	}	  	
    } else {
	set msg "FAILURE"
    }
    set msg
} ""


# PART 7: Test of error in locating set-up file
# =============================================
test slxCheckCompleteness-7 {Test of error locating set-up file} {
    if {[catch "exec -keepnewline slxCheckCompleteness NoTeXiStInG ISAAC" msg]} {
	set testString "Error in locating set-up file NoTeXiStInG!!"
	if {[string first $testString $msg] != -1} {
	    set msg ""	
	} else {
	    set msg "FAILURE"
	}
    } else {
	set msg "FAILURE"
    } 
    set msg
}    ""


# PART 8: Test of handling of set-up in Normal-FITS format
# ========================================================
test slxCheckCompleteness-8 {Set-up in Normal-FITS format} {
    if {[catch "exec -keepnewline slxCheckCompleteness slxCheckCompl.test.8.bs ISAAC -t" msg]} {
	set testString "Set-up file must be a Reference Set-Up:"
	if {[string first $testString $msg] != -1} {
	    set msg ""	
	} else {
	    set msg "FAILURE"
	}	
    } else {
	set msg "FAILURE"
    }
    set msg
}    ""

# PART 9: Test of handling of set-up in unknown format
# ====================================================
test slxCheckCompleteness-9 {Set-up in Normal-FITS format} {
    if {[catch "exec -keepnewline slxCheckCompleteness slxCheckCompl.test.9.bs.rf ISAAC -t" msg]} {
	set testString "Format of specified set-up file is wrong:"
	if {[string first $testString $msg] != -1} {
	    set msg ""	
	} else {
	    set msg "FAILURE"
	}	
    } else {
	set msg "FAILURE"
    } 
    set msg
}    ""


# PART 10: Test of handling of read protected set-up file
# =======================================================
exec chmod -rw $tf.10.bs.rf
test slxCheckCompleteness-10 {Read protected set-up file specified} {
    if {[catch "exec -keepnewline slxCheckCompleteness slxCheckCompl.test.10.bs ISAAC -t" msg]} {
	set testString "Set-up file may be read protected:"
	if {[string first $testString $msg] != -1} {
	    set msg  ""	
	} else {
	    set msg  "FAILURE"
	}	
    } else {
	set msg  "FAILURE"
    }
    set msg
}    ""

exec chmod +rw $tf.10.bs.rf


# PART 11: VLTSW20010753 slxCatFits
# =======================================================
test slxCheckCompleteness-11 {VLTSW20010753-slxCatFits } {
    exec slxCatFits ../test/$tf.4.bs.rf  > slxTestCatFits.tmp
    set msg [exec diff  slxTestCatFits.temp slxTestCatFits.rs]
    set msg
} ""
catch "exec rm slxTestCatFits.tmp"


# PART 11: VLTSW20010753 slxCatFits (file in current directory)
# =======================================================
test slxCheckCompleteness-11 {VLTSW20010753-slxCatFits } {
    exec cp ../test/$tf.4.bs.rf ./slxTestCatFits.bs.tmp
    exec slxCatFits ./slxTestCatFits.bs.tmp > slxTestCatFits.temp
    set msg [exec diff  slxTestCatFits.temp slxTestCatFits.rs]
    set msg
} ""
catch "exec rm slxTestCatFits.tmp"
catch "exec rm slxTestCatFits.bs.tmp"



# --- oOo ---
