#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxCheckModeLine.test.tcl,v 1.145 2005/07/27 10:34:04 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxCheckModeLine.test.prog - test script for slxCheckModeLine
# 
#   SYNOPSIS
#   slxCheckModeLine.test.prog
# 
#   DESCRIPTION
#   The script test the function slxCheckModeLine that tests if a keyword is
#   in contradiction with keywords specified in the Protected Region of a
#   Reference Set-Up (Observation Mode).
#
#   The script carries out the following tests:
#
#   1. Error in the definition of the INS_SETUPPATH environment variable.
#      
#   2. Error in opening the file.
#      
#   3. Keyword/file in wrong format.
#      
#   4. The Protected region is not well defined (start marker missing).
#
#   5. The Protected region is not well defined (end marker missing).
#
#   6. The Protected region is not well defined (both markers missing).
#
#   7. Test of basic functioning.
#
#      
#   The script writes an error report to standard output. 
#
#
#   FILES
#   slxCheckModeLine.test.bs.rf   'Reference Set-Up file' that contains
#                                     a 'well-defined' prottected region.
#
#   slxCheckModeLine.test.bs.ins   Neutral 'Partial Set-Up file'. Does
#                                     not have any lines that contradict with
#                                     protected region in 
#                                     slxCheckModeLine.test.bs.rf.
# 
#   slxCheckModeLine.test.3.bs     Corrupted file (no correct format).
#
#   slxCheckModeLine.test.4.bs.rf 'Reference Set-Up file' where start
#                                     marker of protected region is missing.
#
#   slxCheckModeLine.test.5.bs.rf 'Reference Set-Up file' where end
#                                     marker of protected region is missing.
#
#   slxCheckModeLine.test.6.bs.rf 'Reference Set-Up file' where both
#                                     markers of the protected region are
#                                     missing.
#
#   slxCheckModeLine.test.7.bs.ins Partial Set-Up file'. Contains lines in 
#                                     confilct with protected region in
#                                     slxCheckModeLine.test.bs.rf.
#
#
#   slxCheckModeLine.test.7.rf The result of test 7.
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs

set tf slxCheckModeLine.test
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM
set env(INS_SETUPPATH) [pwd]


if {0} {
# PART 1: INS_SETUPPATH not properly set
# =============================================
unset env(INS_SETUPPATH) 
test slxCheckModeLine-1 {INS_SETUPPATH not properly set} {
    if {[catch "exec -keepnewline slxCheckModeSetup xx slxCheckModeLine.test.bs" msg]} {
        set msg
    }
}    "\n###########################################################################
       The SLX Mode Checker for set-up files in the Short-FITS format
                         Version 1.0/September 1994

                               ESO VLT Project

---------------------------------------------------------------------------

Error in locating set-up file xx!!

---------------------------------------------------------------------------\n\nchild process exited abnormally"

set env(INS_SETUPPATH) [pwd]
}


# PART 2: Error in opening set-up file
# ====================================
test slxCheckModeLine-2 {Error opening set-up file} {
    if {[catch "exec -keepnewline slxCheckModeSetup NoTeXiStInG $tf.bs" msg]} {
        set msg
    }
}    "\n###########################################################################
       The SLX Mode Checker for set-up files in the Short-FITS format
                         Version 1.0/September 1994

                               ESO VLT Project

---------------------------------------------------------------------------

Error in locating set-up file NoTeXiStInG!!

---------------------------------------------------------------------------\n\nchild process exited abnormally"



# PART 3: Error in format of set-up file
# ======================================
test slxCheckModeLine-3 {Error in format of set-up file} {
    if {[catch "exec -keepnewline slxCheckModeSetup $tf.3.bs slxCheckModeLine.test.bs.rf -t" msg]} {
        set msg
    }
}    "\n###########################################################################
       The SLX Mode Checker for set-up files in the Short-FITS format
                         Version 1.0/September 1994

                               ESO VLT Project

---------------------------------------------------------------------------
An error occurred operating the Mode Checker!
The error diagnotics is:

The format of keyword line 21
- is wrong!!

Please correct the error and try again!
---------------------------------------------------------------------------\n\nchild process exited abnormally"



# PART 4: Protected region not well-defined (missing start marker)
# ================================================================
test slxCheckModeLine-4 {Protected region start marker missing} {
    if {[catch "exec -keepnewline slxCheckModeSetup $tf.bs.ins $tf.4.bs.rf -t" msg]} {
        set msg
    }
}    "\n###########################################################################
       The SLX Mode Checker for set-up files in the Short-FITS format
                         Version 1.0/September 1994

                               ESO VLT Project

---------------------------------------------------------------------------
An error occurred operating the Mode Checker!
The error diagnotics is:

Error with proteted region in Ref. Set-Up!

Please correct the error and try again!
---------------------------------------------------------------------------\n\nchild process exited abnormally"


# PART 5: Protected region not well-defined (missing end marker)
# ==============================================================
test slxCheckModeLine-5 {Protected region end marker missing} {
    if {[catch "exec -keepnewline slxCheckModeSetup $tf.bs.ins $tf.5.bs.rf -t" msg]} {
        set msg
    }
}    "\n###########################################################################
       The SLX Mode Checker for set-up files in the Short-FITS format
                         Version 1.0/September 1994

                               ESO VLT Project

---------------------------------------------------------------------------
An error occurred operating the Mode Checker!
The error diagnotics is:

Error with proteted region in Ref. Set-Up!

Please correct the error and try again!
---------------------------------------------------------------------------\n\nchild process exited abnormally"



# PART 6: Protected region not well-defined (missing both markers)
# ================================================================
test slxCheckModeLine-6 {Protected region missing both markers} {
    if {[catch "exec -keepnewline slxCheckModeSetup $tf.bs.ins $tf.6.bs.rf -t" msg]} {
        set msg
    }
}    "\n###########################################################################
       The SLX Mode Checker for set-up files in the Short-FITS format
                         Version 1.0/September 1994

                               ESO VLT Project

---------------------------------------------------------------------------
An error occurred operating the Mode Checker!
The error diagnotics is:

Error with proteted region in Ref. Set-Up!

Please correct the error and try again!
---------------------------------------------------------------------------\n\nchild process exited abnormally"



# PART 7: Test of basic functioning
# =================================
test slxCheckModeLine-7 {Basic functionality test} {
    exec slxCheckModeSetup $tf.7.bs.ins $tf.bs.rf -t > $tf.7.temp
    set msg [exec diff $tf.7.temp $tf.7.rf]
    exec rm $tf.7.temp
    set msg
} ""

