#!/usr/local/bin/tcl
#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxCheckSyntaxSp.test.tcl,v 1.145 2005/07/27 10:49:28 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxCheckSyntaxSetup.test.prog - test script for slxCheckSyntaxSetup
# 
#   SYNOPSIS
#   slxCheckSyntaxSetup.test.prog
# 
#   DESCRIPTION
#   The script test the program slxCheckSyntaxSetup taht checks the syntax
#   of a set-up file.
#
#   The script carries out the following tests:
#
#   1. Test of handling of failure in locating dictionary.
#      Error constant: slxERR_SYN_LOCATE_DICO.
#
#   2. Test of handling of failure in opening dictionary.
#      Test files used:  ESO-VLT-DIC.FORS, test.ref
#      Error constant: slxERR_SYN_OPEN_DICO.
#
#   3. Test of an error in the header of a dictionary.
#      Test files used: ESO-VLT-DIC.EMMI, test.ref
#      Error constant: slxERR_SYN_HEADER_DICO.
#
#   4. Test of correct functionining of the function. Test file is used that
#      provoke all detectable syntax errors.
#      Test files used: ESO-VLT-DIC.ISAAC, slxCheckSyntaxLine.test.5.bs.rf
#
#   5. Test of input parameters (correct number).
#
#   6. Test of handling of case when illegal instrument is specified.
#
#   7. Test of error in locating set-up file.
#
#   8. Test of handling of set-up file in wrong format (Normal-FITS intead of
#      Short-FITS).
#
#   9. Test of handling of set-up file in wrong format (unknown format).
#
#   10. Test of access to read protected set-up file.
#
#   11. Test of handling of more than slxMAX_NO_OF_SYN_ERR syntax errors.
#
#
#   The script writes an error report to standard output. 
#
#
#   FILES
#   ESO-VLT-DIC.ISAAC                     Correct dictionary containing the 
#                                         necessary definitions.
#
#   ESO-VLT-DIC.FORS                      rwx protected dictionary - cannot be 
#                                         opened.
#
#   ESO-VLT-DIC.EMMI                      Dictionary with an error in the 
#                                         dictionary header.
#   
#   test.ref                              Just a dummy set-up file to give an
#                                         input to the function.
#
#   slxCheckSyntaxSetup.test.4.bs.rf  Input set-up file containing keywords
#                                         that will provoke all detectable 
#                                         syntax errors.
#
#   slxCheckSyntaxSetup.test.8.bs      File in Normal-FITS format.
#
#   slxCheckSyntaxSetup.test.9.bs      File in unknown format.
#
#   slxCheckSyntaxSetup.test.10.bs.rf Read protected file.
#
#   slxCheckSyntaxSetup.test.11.bs     File containing lots of syntax
#                                         errors.
#
#   slxCheckSyntaxSetup.test.11.rf File containing expected output
#                                         from test.
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#
source ../test/defs
source ../test/slxTestCommon.tcl

set tf slxCheckSyntaxSp.test
set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM
set gvar(writeErrorMsg) 0



# PART 1: Failure in locating the dictionary
# ==========================================
test slxCheckSyntaxSetup-1 {Failure in locating dictionary} {
    if {[catch "exec -keepnewline slxCheckSyntaxSetup test.ref UVES" msg]} { 
        set msg [slxTestTestString "Error occurred opening dict. for" $msg]
    } else {
	set msg FAILURE
    }
    set msg
}    ""



# PART 2: Failure in opening dictionary
# =====================================
exec chmod -rwx $env(INS_ROOT)/SYSTEM/Dictionary/ESO-VLT-DIC.FORS
test slxCheckSyntaxSetup-2 {Attempt to open rwx protected dictionary} {
    if {[catch "exec -keepnewline slxCheckSyntaxSetup test.ref FORS -t" msg]} {
	set testString "Error occurred opening dict. for"
	set msg [slxTestTestString $testString $msg]
    } else {
	set msg FAILURE
    }
    set msg
}    ""
exec chmod +rw $env(INS_ROOT)/SYSTEM/Dictionary/ESO-VLT-DIC.FORS


# PART 4: Test of basic functioning
# =================================
test slxCheckSyntaxSetup-4 {Basic functionality test} {
    exec slxCheckSyntaxSetup $tf.4.bs.rf ISAAC -t > $tf.4.temp
    set msg [exec diff $tf.4.temp $tf.4.rf]
    set msg
} ""
exec rm $tf.4.temp


# PART 5: Test of check of number of input parameters
# ======================================================
test slxCheckSyntaxSetup-5 {Test of check of input parameters} {
    if {[catch "exec -keepnewline slxCheckSyntaxSetup " msg]} {
        set msg
    }
}    "\n###########################################################################
      The SLX Syntax Checker for set-up files in the Short-FITS format
                         Version 1.0/September 1994

                               ESO VLT Project

---------------------------------------------------------------------------
Correct format is:

slxCheckSyntaxSetup <set-up filename> <instrument>

---------------------------------------------------------------------------\n\nchild process exited abnormally"



# PART 6: Test of handling of specification of illegal instrument
# ===============================================================
test slxCheckSyntaxSetup-6 {Test of specification of illegal instrument} {
    if {[catch "exec -keepnewline slxCheckSyntaxSetup test.ref unknown" msg]} {
        set msg [slxTestTestString "Error occurred opening dict. for" $msg]
    } else {
	set msg FAILURE
    }
    set msg
}    ""



# PART 7: Test of error in locating set-up file
# =============================================
test slxCheckSyntaxSetup-7 {Test of error locating set-up file} {
    if {[catch "exec -keepnewline slxCheckSyntaxSetup NoTeXiStInG ISAAC" msg]} {
        set msg
    }
}    "\n###########################################################################
      The SLX Syntax Checker for set-up files in the Short-FITS format
                         Version 1.0/September 1994

                               ESO VLT Project

---------------------------------------------------------------------------

Error in locating set-up file NoTeXiStInG!!

---------------------------------------------------------------------------\n\nchild process exited abnormally"



# PART 8: Test of handling of set-up in Normal-FITS format
# ========================================================
test slxCheckSyntaxSetup-8 {Set-up in Normal-FITS format} {
    if {[catch "exec -keepnewline slxCheckSyntaxSetup slxCheckSyntaxSetup.test.8.bs ISAAC -t" msg]} {
        set msg
    }
}    "\n###########################################################################
      The SLX Syntax Checker for set-up files in the Short-FITS format
                         Version 1.0/September 1994

                               ESO VLT Project

---------------------------------------------------------------------------

Format of specified set-up file is wrong:
---------------------------------------------------------------------------\n\nchild process exited abnormally"



# PART 9: Test of handling of set-up in unknown format
# ====================================================
test slxCheckSyntaxSetup-9 {Set-up in Normal-FITS format} {
    if {[catch "exec -keepnewline slxCheckSyntaxSetup slxCheckSyntaxSetup.test.9.bs ISAAC -t" msg]} {
        set msg
    }
}    "\n###########################################################################
      The SLX Syntax Checker for set-up files in the Short-FITS format
                         Version 1.0/September 1994

                               ESO VLT Project

---------------------------------------------------------------------------

Format of specified set-up file is wrong:
---------------------------------------------------------------------------\n\nchild process exited abnormally"



# PART 10: Test of handling of read protected set-up file
# =======================================================
exec chmod -rw $tf.10.bs
test slxCheckSyntaxSetup-10 {Read protected set-up file specified} {
    if {[catch "exec -keepnewline slxCheckSyntaxSetup slxCheckSyntaxSetup.test.10.bs ISAAC -t" msg]} {
        set msg
    }
}    "\n###########################################################################
      The SLX Syntax Checker for set-up files in the Short-FITS format
                         Version 1.0/September 1994

                               ESO VLT Project

---------------------------------------------------------------------------

Set-up file may be read protected:

---------------------------------------------------------------------------\n\nchild process exited abnormally"
exec chmod +rw $tf.10.bs


# PART 11: Test of handling of many errors
# ========================================
test slxCheckSyntaxSetup-11 {Handling of many errors} {
    exec slxCheckSyntaxSetup $tf.11.bs ISAAC -t > $tf.11.temp
    set msg [exec diff $tf.11.temp $tf.11.rf]
    set msg
} ""
exec rm $tf.11.temp

