#!/usr/local/bin/tcl
#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxExtractSetup.test.tcl,v 1.145 2005/07/27 10:55:17 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ---------------------------------------------
# eallaert  15/10/94  initial version
#

#**************************************************************************
#   NAME
#   slxExtractSetup.test.tcl - test program for slxExtractSetup
#
#   SYNOPSIS
#   slxExtractSetup.test.tcl
#
#   DESCRIPTION
#   This script tests the slxExtractSetup program, as follows:
#	1.1 Wrong number of arguments (3)
#	1.2 Wrong number of arguments (3, plus "-t" option)
#	1.3 First argument is not absolute path
#	1.4 Second argument has ".ins" extension
#	1.5 Second argument has ".targ" extension
#	1.6 Second argument has ".det" extension
#	1.7 Fourth argument refers to non-existing dictionary
#	1.8 First argument refers to not existing image file
#	1.9 First argument refers to read-protected file - SKIPPED!!
#	1.10 Second argument describes already existing file
#	1.11 Setup file template is not in $INS_SETUPPATH
#	1.15 First argument describes non-FITS file
#	1.16 Second argument refers to write-protected directory
#
#	2.1 Normal execution of slxExtractSetup
#
#   If any of these tests fail, an error report will be sent to stdout
#
#   FILES
#   The following files are required and accessed under ~slx/test:
#	- ESO-VLT-DIC.TEST (tests 1.8 - 1.16, 2.1)
#	- slxExtractSetup.test1.9.bs (test 1.9)
#	- slxExtractSetup.test2.1.bs (tests 1.7, 1.10, 1.11, 1.16, 2.1)
#	- slxExtractSetup.test.tcl (test 1.15)
#
#   CAUTIONS
#   To be executed from the ~slx/test directory
#
#-----------------------------------------------------------------------------

# Take over Ousterhout's test procedure 
source ../test/defs
cd ../test
set slxExtractSetup slxExtractSetup
set env(INS_SETUPPATH) [pwd]
set env(INS_ROOT) [pwd]/INS_ROOT
set env(INS_USER) SYSTEM


# In case of errors, slxExtractSetup does an exit(1), so from Tcl
# we have to catch this error

# PART 1: argument testing
# ========================

catch {unset env(INS_SETUPPATH)} msg
# ---------------------------------------------------------------------------
# Produce error for wrong number of parameters (at least 4 needed, not counting
# the verbose option)
test slxExtractSetup-1.1 {number of arguments: 3} {
	catch "exec $slxExtractSetup A B C" msg
	set msg
}   "\nError, correct syntax is:\n
slxExtractSetup <<path>/<image file>> <<path>/<new setup>> <setup ID>\
<dictionary> \[-t\]\n"


 # ---------------------------------------------------------------------------
# Produce error for wrong number of parameters (at least 4 needed)
test slxExtractSetup-1.2 {number of arguments: 3, plus -t option} {
	catch "exec $slxExtractSetup A B C -t" msg
	set msg
}   "\nError, correct syntax is:\n
slxExtractSetup <<path>/<image file>> <<path>/<new setup>> <setup ID>\
<dictionary> \[-t\]\n"

# ---------------------------------------------------------------------------
# Produce error for 1st parameter not absolute path
test slxExtractSetup-1.3 {1st argument must be absolute path} {
	catch "exec $slxExtractSetup A B C D" msg
	set msg
}   "\nError, name of new setup file must include path!\n"

# ---------------------------------------------------------------------------
# Produce error for 2nd parameter having wrong extension
test slxExtractSetup-1.4 {2nd argument cannot have ".ins" extension} {
        set p1 [pwd]/A
        set p2 [pwd]/B
	catch "exec $slxExtractSetup $p1 $p2.ins C D" msg
	set msg
}   "\nError, extensions \".ins\", \".targ\", and \".det\" are illegal!\n"

# ---------------------------------------------------------------------------
# Produce error for 2nd parameter having wrong extension
test slxExtractSetup-1.5 {2nd argument cannot have ".targ" extension} {
        set p1 [pwd]/A
        set p2 [pwd]/B
	catch "exec $slxExtractSetup $p1 $p2.targ C D -t" msg
	set msg
}   "\nError, extensions \".ins\", \".targ\", and \".det\" are illegal!\n"

# ---------------------------------------------------------------------------
# Produce error for 2nd parameter having wrong extension
test slxExtractSetup-1.6 {2nd argument cannot have ".det" extension} {
        set p1 [pwd]/A
        set p2 [pwd]/B
	catch "exec $slxExtractSetup $p1 $p2.det C D -t" msg
	set msg
}   "\nError, extensions \".ins\", \".targ\", and \".det\" are illegal!\n"

# ---------------------------------------------------------------------------
# Produce error for 4th parameter referring to not-existing dictionary
set tf slxExtractSetup.test2.1
catch "exec rm [pwd]/../test/B.ref"
test slxExtractSetup-1.7 {4th argument refers to not existing dictionary} {
        set p1 [pwd]/$tf.bs
        set p2 [pwd]/B
	catch "exec $slxExtractSetup $p1 $p2 C D -t" msg
	set msg
}   "\nError in reading dictionary \"D\"!

slxERR_OPEN_DIC : Could not open dictionary \"$env(INS_ROOT)/SYSTEM/Dictionary/ESO-VLT-DIC.D\""

# ---------------------------------------------------------------------------
# Produce error for 1st parameter referring to not-existing image file
test slxExtractSetup-1.8 {1st argument refers to not existing image file} {
        set p1 A
        set p2 [pwd]/B
	set env(INS_SETUPPATH) $env(PWD)
	catch "exec $slxExtractSetup $p1 $p2 C TEST -t" msg
	set msg
}   "\nError in locating/accessing file \"A\"!\n
slxERR_LOCATE_FILE : Could not find \"A\" according to INS_SETUPPATH\
or path given with filename"

# ---------------------------------------------------------------------------
# Produce error for 1st parameter referring to read-protected image file
set tf slxExtractSetup.test1.9
cd ../test
set pwdPath [pwd]
test slxExtractSetup-1.9 {1st argument refers to read-protected image file} {
    catch "exec chmod ugo=+rwx $tf.temp"
    exec cp $tf.bs $tf.temp
    chmod 0222 $tf.temp
    set p1 $tf.temp
    set p2 [pwd]/B
    set env(INS_SETUPPATH) $pwdPath
    catch "exec $slxExtractSetup $p1 $p2 C TEST -t" msg
    catch "exec chmod ugo=+rwx $tf.temp"
#    exec rm $tf.temp
    set msg
}   "\nError in locating/accessing file \"slxExtractSetup.test1.9.temp\"!

slxERR_OPEN_FILE : Could not open file \"slxExtractSetup.test1.9.temp\""


# ---------------------------------------------------------------------------
# Produce error for resulting setupfile (par 2) already existing
set tf slxExtractSetup.test2.1
cd ../test
set pwdPath [pwd]
test slxExtractSetup-1.10 {2nd argument describes already existing file} {
    set env(INS_SETUPPATH) $pwdPath
    catch "exec chmod ugo=+rwx $tf.ref"
    exec cp $tf.bs $tf.ref
    exec chmod ugo=+rwx $tf.ref
    set p1 $pwdPath/$tf.ref
    exec chmod ugo=+rwx $p1
    set p2 $pwdPath/$tf.ref
    exec chmod ugo=+rwx $p2
    catch "exec $slxExtractSetup $p1 $p2 C TEST -t" msg
    catch "exec chmod ugo=+rwx $tf.ref"
    exec rm $tf.ref
    set msg
}   "\nError: \"$pwdPath/$tf.ref\" exists already!\n"


# ---------------------------------------------------------------------------
# Produce error for copying setupfile template => code does not do this!!

# ---------------------------------------------------------------------------
# Produce error for creating short fits lines -> How????

# ---------------------------------------------------------------------------
# Produce error for storing line -> How????

# ---------------------------------------------------------------------------
# Produce error for image file not in proper format
set tf slxExtractSetup.test.tcl
cd ../test
set pwdPath [pwd]
#set pwdPath $env(PWD)
test slxExtractSetup-1.15 {1st argument describes non-FITS file} {
    set wd [pwd]
    cd $pwdPath/../templates
    #	set env(INS_SETUPPATH) $env(PWD):[pwd]
    set env(INS_SETUPPATH) $pwdPath
    cd $wd
    exec cp $tf $tf.temp
    exec chmod ugo=+rwx $tf.temp
    set p1 $pwdPath/$tf.temp
    catch "exec chmod ugo=+rwx $p1"
    set p2 $pwdPath/$tf.ref
    catch "exec chmod ugo=+rwx $p2"
    catch "exec $slxExtractSetup $p1 $p2 C TEST -t" msg
    catch "exec chmod ugo=+rwx $p1"
    exec rm $p1
    set msg
}   "\nFormat of image file header must be FITS ($pwdPath/$tf.temp)!\n" 

# ---------------------------------------------------------------------------
# Produce error for new setup file on write protected directory
set tf slxExtractSetup.test2.1
test slxExtractSetup-1.16 {new setup file (2nd arg) on write protected dir} {
	set wd [pwd]
	cd $wd/../templates
	set env(INS_SETUPPATH) $wd:[pwd]
	cd $wd
	set p1 $wd/$tf.bs
	mkdir test1.16
	chmod -w test1.16
	set p2 $wd/test1.16/$tf.ref
	catch "exec $slxExtractSetup $p1 $p2 C TEST -t" msg
	rmdir test1.16
	set msg
}   "cp: cannot create $wd/test1.16/slxExtractSetup.test2.1.ref:\
Permission denied

Error in copying set-up header template!\n" 


# PART 2: functionality test
# ==========================
# ---------------------------------------------------------------------------
# extract a setup file from a FITS file
# if all is well, slxExtractSetup should take an exit(0) without 
# additional messages.
# Keep the output of "diff", to restore after removing temporary files

# IMPL.: There is some problem with this test when cmmCopy'ing the module - 
# so the FUNCTIONALITY TEST IS TEMPORARILY suspended.
# Anyway, this tool should probably be taken out, since the feature will be
# supported by the FHT
if {0} {
set msg ""
set tf slxExtractSetup.test2.1
test slxExtractSetup-2.1 {extract a setup file} {
    set wd [pwd]
    cd $wd/../templates
    set env(INS_SETUPPATH) $wd:[pwd]
    cd $wd
    set p1 $wd/$tf.bs
    catch "exec chmod ugo=+rwx $p1"
    set f1 $wd/$tf.ref;	        # resulting setup file
    catch "exec chmod ugo=+rwx $f1"
    catch "rm $f1"
    set f2 $wd/$tf.temp;	# temp file, f2 = f1 - SUP.CRTE.*
    catch "exec chmod ugo=+rwx $f2"

    if {![catch "exec -keepnewline $slxExtractSetup $p1 $f1 ID TEST -t" msg]} {
	set fileId1 [open $f1]
	set fileId2 [open $f2 w]
	while {[gets $fileId1 line] != -1} { 
	    if {![string match SUP.CRTE.* $line]} {
		puts $fileId2 $line
	    }
	}
	close $fileId1
	close $fileId2
	catch "exec diff $f2 $tf.rf" msg
    }
    catch "exec chmod ugo=+rwx $f1"
    catch "exec chmod ugo=+rwx $f2"    
    catch "exec rm $f1 $f2"
    set msg
} {}
}

