#!/usr/local/bin/tcl
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: slxSetupToNormalFITS.test.tcl,v 1.145 2005/07/27 11:22:51 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  13/10/94  created
#

#************************************************************************
#   NAME
#   slxSetupToNormalFITS.test.prog - test script for slxSetupToNormalFITS
# 
#   SYNOPSIS
#   slxSetupToNormalFITS.test.prog
# 
#   DESCRIPTION
#   The script test the slxSetupToNormalFITS to convert set-up files from the
#   Short-FITS format to the Normal-FITS format.
#
#   The script carries out the following tests:
#
#   1. Test of handling of missing input parameter
#
#   2. Test of attempt to convert non-existing file.
#
#   3. Test of attempt to convert obscure file.
#
#   4. Attempt to convert rwx protected file
#
#   5. Test of conversion of set-up file from Short-FITS to normal FITS.
#      The file converted contains the most of the cases possible. It may
#      not be syntactically correct.
#
#   6. Test of handling of INS_SETUPPATH not properly set.
#
#
#   The script writes an error report to standard output. 
#
#
#   FILES
#   slxSetupToNormalFITS.test.3.bs      Is an obscure set-up file.
#
#   slxSetupToNormalFITS.test.4.bs      File to be rwx protected.
#
#   slxSetupToNormalFITS.test.5.bs      Is a file in Short-FITS format 
#                                          containing a lot of various 
#                                          combinations.
#
#   slxSetupToNormalFITS.test.5.rf  Is the.rf to compare with
#                                          the result of converting the 
#                                          .5.bs file.
#
#
#   CAUTIONS
#   Should be executed from the ~slx/test directory.
#
#------------------------------------------------------------------------
#

source ../test/defs

#set verbose 1
set tf slxSetupToNormalFITS.test
set env(INS_SETUPPATH) [pwd]

# PART 1: argument testing
# ========================
test slxSetupToNormalFITS-1 {Number of arguments: none} {
    if {[catch "exec -keepnewline slxSetupToNormalFITS" msg]} { 
        set msg
    }
}    "\nCorrect syntax is:\n\nslxSetupToNormalFITS  <set-up filename>\n\nchild process exited abnormally"



# PART 2: attempting to open non-existing set-up file
# ===================================================
test slxSetUpToNormalFITS-2 {Non existing file} {
    if {[catch "exec -keepnewline slxSetupToNormalFITS NoNeXiStInGfIlE -v" \
        msg]} {
        set msg
    }
}    "\nCouldn't convert the file - check filename!\n"



# PART 3: format of file is wrong
# ================================
exec cp $tf.3.bs $tf.3.temp
exec chmod ugo=+rwx $tf.3.temp
test slxSetUpToNormalFITS-3 {File format wrong} {
    if {[catch "exec -keepnewline slxSetupToNormalFITS slxSetupToNormalFITS.test.3.temp -v" msg]} {
        set msg
    }
}    "\nCouldn't convert the file - check filename!\n"
exec rm $tf.3.temp



# PART 4: try to open full-protected (rwx) file
# =============================================
exec chmod -rwx $tf.4.bs
test slxSetUpToNormalFITS-4 {File rwx protected} {
    if {[catch "exec -keepnewline slxSetupToNormalFITS slxSetupToNormalFITS.test.4.temp -v" msg]} {
        set msg
    }
}   "\nCouldn't convert the file - check filename!\n"
exec chmod +rw $tf.4.bs



# PART 5: test basic functionality
# ================================
test slxSetUpToNormalFITS-5 {Basic functionality test} {
    exec cp $tf.5.bs $tf.5.temp
    exec chmod ugo=+rwx $tf.5.temp 
    exec slxSetupToNormalFITS $tf.5.temp -v
    set msg [exec diff $tf.5.temp $tf.5.rf]
    exec rm $tf.5.temp
    set msg
} ""



# PART 6: INS_SETUPPATH not properly set
# =============================================
unset env(INS_SETUPPATH) 
test slxSetUpToNormalFITS-6 {INS_SETUPPATH not properly set} {
    if {[catch "exec -keepnewline slxSetupToNormalFITS xxxx -v" msg]} {
        set msg
    }
}    "\nCouldn't convert the file - check filename!\n"
set env(INS_SETUPPATH) [pwd]






